/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.core;

import java.util.ArrayList;

/**
 * <p>Descriptions: Auxiliary class that contains the node of the parse
 * tree.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
class ParseTreeNode {

  private String tag;
  private String type;
  private ParseTreeNode parent;
  private ArrayList<Double> confScores;
  private ArrayList<String> texts;
  private ArrayList<ParseTreeNode> pointers;
  private ArrayList<String> prevChars;

  public ParseTreeNode(String tag, String type, ParseTreeNode parent,
                       ArrayList<Double> confScores) {
    this.tag = tag;
    this.type = type;
    this.parent = parent;
    this.confScores = confScores;
    this.texts = new ArrayList<String>();
    this.pointers = new ArrayList<ParseTreeNode>();
    this.prevChars = new ArrayList<String>();
  } // constructor: ParseTreeNode

  protected void setNextText(String text) {
    texts.add(text);
  } // setNextText

  protected void setNextPointer(ParseTreeNode p) {
    pointers.add(p);
  } // setNextPointer

  protected void setNextPrevChar(String prevChar) {
    prevChars.add(prevChar);
  } // setNextPrevChar

  public String getTag() {
    return tag;
  } // getTag

  public String getType() {
    return type;
  } // getType

  protected ParseTreeNode getParent() {
    return parent;
  } // getParent;

  protected ArrayList<Double> getConfScores() {
    return confScores;
  } // getConfScores;

  public String getText(int pos) {
    return texts.get(pos);
  } // getText

  protected ParseTreeNode getPointer(int pos) {
    return pointers.get(pos);
  } // getPointer

  protected String getPrevChar(int pos) {
    return prevChars.get(pos);
  } // getPrevChar

  protected int getPointersSize() {
    return pointers.size();
  } // getPointersSize

} // end of class ParseTreeNode
