/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.core;

import ipl.util.Constants;

/**
 * <p>Description: Auxiliary class that ontains the tokens.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class TokenNode {

  protected String token;
  protected String tagType;
  protected boolean endOfPeriod;
  protected boolean subtitle;
  protected boolean startSubtitle;
  protected boolean endSubtitle;
  protected double predicted_probability;
  protected String prevChar;

  public TokenNode(String token, String tagType, boolean endOfPeriod,
                   boolean subtitle, String prevChar) {
    this.token = token;
    this.tagType = tagType;
    this.endOfPeriod = endOfPeriod;
    this.subtitle = subtitle;
    this.startSubtitle = false;
    this.endSubtitle = false;
    if ((tagType.compareTo(Constants.PERSON) == 0) ||
        (tagType.compareTo(Constants.ORGANIZATION) == 0) ||
        (tagType.compareTo(Constants.LOCATION) == 0))
      this.predicted_probability = 1.0;
    else
      this.predicted_probability = -1.0;
    this.prevChar = prevChar;
  } // constructor: TokenNode

  public void setTokenNode(String token, String tagType, boolean endOfPeriod, boolean subtitle) {
    this.token = token;
    this.tagType = tagType;
    this.endOfPeriod = endOfPeriod;
    this.subtitle = subtitle;
  } // setTokenNode

  public String getToken() {
    return this.token;
  } // getToken

  public String getTagType() {
    return this.tagType;
  } // getTagType

  public boolean getEndOfPeriod() {
    return this.endOfPeriod;
  } // getEndOfPeriod

  public boolean isSubtitle() {
    return this.subtitle;
  } // isSubtitle

  public boolean isStartSubtitle() {
    return this.startSubtitle;
  } // isStartSubtitle

  public boolean isEndSubtitle() {
    return this.endSubtitle;
  } // isEndSubtitle

  public double getPredictedProbability() {
    return this.predicted_probability;
  } // getPredictedProbability

  public String getPrevChar() {
    return this.prevChar;
  } // getPrevChar

  public void setToken(String token) {
    this.token = token;
  } // setToken

  public void setTagType(String type) {
    this.tagType = type;
  } // setTagType

  public void setEndOfPeriod(boolean endperiod) {
    this.endOfPeriod = endperiod;
  } // setEndOfPeriod

  public void setSubtitle(boolean subtitle) {
    this.subtitle = subtitle;
  } // setSubtitle

  public void setStartSubtitle(boolean start) {
    this.startSubtitle = start;
  } // setStartSubtitle

  public void setEndSubtitle(boolean end) {
    this.endSubtitle = end;
  } // setEndSubtitle

  public void setPredictedProbability(double prob) {
    this.predicted_probability = prob;
  } // setPredictedProbability

  public void setPrevChar(String pch) {
    this.prevChar = pch;
  } // setPrevChar

} // end of class TokenNode
