/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.core;

import ipl.exceptions.XmlParseErrorException;
import ipl.util.Constants;

import java.util.StringTokenizer;
import java.util.ArrayList;

/**
 * <p>Description: Creates the parse tree.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class XmlParseTree {
    
    private ParseTreeNode root;
    
    public XmlParseTree(String xmlText, String fileName) throws XmlParseErrorException {
        
        root = null;
        ParseTreeNode curr = null, temp;
        int start = 0, end, startOfText = 0, endOfText;
        String tag, type;
        String prevChar = "";
        
        while ( (start = xmlText.indexOf("<", start)) != -1) {
            if ((!xmlText.startsWith(Constants.PARAGRAPH_START_TAG, start)) &&
                    (!xmlText.startsWith(Constants.PARAGRAPH_END_TAG, start))) {
                end = xmlText.indexOf(">", start + 1);
                endOfText = start;
                tag = xmlText.substring(start + 1, end);
                type = Constants.NONE;
                
                // case: end of tag
                if (tag.charAt(0) == '/') {
                    if (root == null) {
                        throw new XmlParseErrorException("PARSE ERROR1");
                        
                    } // if
                    // Check for valid tree
                    tag = tag.substring(1);
                    if (curr.getTag().compareTo(tag) != 0) {
                        throw new XmlParseErrorException("PARSE ERROR2");
                        
                    } // if
                    
                    //   .     
                    curr.setNextText(xmlText.substring(startOfText, endOfText));
                    curr.setNextPrevChar(prevChar);
                    curr = curr.getParent();
                } // if
                // case: start of tag
                else {
                    // get tag, type and confs
                    StringTokenizer st = new StringTokenizer(tag, " ");
                    tag = st.nextToken();
                    ArrayList<Double> confs = new ArrayList<Double>();
                    while (st.hasMoreTokens()) {
                        String str = st.nextToken();
                        if (str.equals("TYPE=\"" + Constants.PERSON + "\"")) {
                            type = Constants.PERSON;
                        } // if
                        else if (str.equals("TYPE=\"" + Constants.ORGANIZATION + "\"")) {
                            type = Constants.ORGANIZATION;
                        } // else if
                        else if (str.equals("TYPE=\"" + Constants.LOCATION + "\"")) {
                            type = Constants.LOCATION;
                        } // else if
                        else if (str.equals("TYPE=\"" + Constants.MONEY + "\"")) {
                            type = Constants.MONEY;
                        } // else if
                        else if (str.equals("TYPE=\"" + Constants.PERCENT + "\"")) {
                            type = Constants.PERCENT;
                        } // else if
                        else if (str.equals("TYPE=\"" + Constants.DATE + "\"")) {
                            type = Constants.DATE;
                        } // else if
                        else if (str.equals("TYPE=\"" + Constants.TIME + "\"")) {
                            type = Constants.TIME;
                        } // else if
                        else if (str.startsWith("CONF")) {
                            int s = str.indexOf("\"");
                            int e = str.indexOf("\"", s + 1);
                            confs.add(Integer.parseInt(str.substring(4, 5)),
                                    Double.parseDouble(str.substring(s + 1, e)));
                        } // else if
                        else {
                            throw new XmlParseErrorException("PARSE ERROR3 \n" +
                                    "Unknown tag: \"" + str + " \" found in file " + fileName + "\n");
                            
                            
                        } // else
                    } // while
                    
                    //   ,    
                    if (root == null) {
                        root = new ParseTreeNode(tag, type, null, confs);
                        curr = root;
                    } // if
                    else {
                        temp = new ParseTreeNode(tag, type, curr, confs);
                        curr.setNextPointer(temp);
                        curr.setNextText(xmlText.substring(startOfText, endOfText));
                        curr.setNextPrevChar(prevChar);
                        curr = temp;
                    } // else
                } // else
                startOfText = end + 1;
            } // if
            if (start == 0) {
                prevChar = "";
            } // if
            else {
                if ( (xmlText.charAt(start - 1) == '\n') ||
                        (xmlText.charAt(start - 1) == ' ')) {
                    prevChar = Character.toString(xmlText.charAt(start - 1));
                } // if
                else {
                    prevChar = "";
                } // else
            } // else
            start++;
        } // while
        
        
        
        
    } // constructor: XmlParseTree
    
    protected ParseTreeNode getRoot() {
        return (find(root, Constants.ARTICLE));
    } // getBody
    
    protected ParseTreeNode find(ParseTreeNode curr, String strTag) {
        
        if (curr.getTag().equalsIgnoreCase(strTag)) {
            return curr;
        } // if
        else {
            ParseTreeNode temp;
            int i = 0;
            while (i < curr.getPointersSize()) {
                if ((temp = find(curr.getPointer(i), strTag)) != null) {
                    return temp;
                } // if
                i++;
            } // while
            return null;
        } // else
    } // find
    
} // end of class XmlParseTree
