/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.gui.active_learning;


import ipl.nerc.Trainer;
import ipl.util.Constants;
import ipl.nerc.active_learning.ActiveLearning;
import ipl.nerc.active_learning.ActiveLearningFP;
import ipl.nerc.active_learning.ActiveLearningSP;
import ipl.nerc.active_learning.PrepareSP;

import ipl.nerc.lists.ListCreator;
import ipl.nerc.lists.ListManager;
import ipl.nerc.lists.ListNode;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import ipl.gui.main.Nerc;
import ipl.splitter.SentenceSplitter;
import java.util.GregorianCalendar;
import ipl.nerc.active_learning.data.AmbiguousVector;
import ipl.nerc.active_learning.data.PoolHandler;
import ipl.nerc.active_learning.data.SimilarityVector;
import ipl.nerc.Tester;
import ipl.nerc.preprocessing.ResultFilesProcessor;
import ipl.nerc.regex.timex.TimexMatcher;
import java.awt.Toolkit;
import java.net.MalformedURLException;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;

public class Form extends javax.swing.JFrame {
    
    
    private String currPass;
    /**
     * Dialog boxes for messages to user
     */
    private javax.swing.JOptionPane dialog;
    
    /**
     * Redirected output steam for System.resultsOutput
     */
    private PrintStream sysDefOut;
    
    ////////////////////////////////////////////////////////////////////////////
    /**
     * The last Stream used for saving experimental results
     */
    private FileOutputStream resultStream ;
    
    /**
     * The last PrintStream used for saving experimental results
     */
    private PrintStream resultsOutput ;
    
    /**
     * The last File where experimental results where saved
     */
    private File lastFile2KeepResults;
    ////////////////////////////////////////////////////////////////////////////
    
    
    /**
     * Number of training batch for each category after last training
     */
    private int personTrainVecs = 0, organTrainVecs = 0;
    
    private final static String
            PER_DECISION = Constants.PERSON,
            ORG_DECISION = Constants.ORGANIZATION,
            NONE_DECISION = Constants.NONE ;
    
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Sure decisions of previous pass classifier. To be used for second pass.
     * Uses category + "#" + name of file (eg ORGANIZATION#nea@8765n=7a.xml) as a key to store / retrieve
     * a list of surely classified tokens in this file for this category .
     * Decisions with cannonical posibility grater than a margin (eg 0.8) are considered
     * as sure, because previous pass classifier is sure about classifing those tokens.
     * The list of surelly classified tokens, is an array of String objects.
     *
     */
    private Hashtable<String, String[]> sureFPDec;
    
    /*
     * All decisions of previous pass classifier. To be used for second pass.
     * Uses category + "#" + name of file + "#" + position of token in text
     * (eg PERSON#nea@8765n=7a.xml#143)) as a key to store / retrieve
     * all cannonical posibilities assigned by previous pass classifier about each token for this category and each file.
     */
    private Hashtable<String, String> allDec;
    
    /**
     * create a Trainer object for performing training */
    private Trainer trainer;
    
    
    /**Matches date and time patterns */
    private TimexMatcher matcher;
    
    /**Separatior for periods in texts */
    private SentenceSplitter separator;
    
    ////////////////////////////////////////////////////////////////////////////
    /**Object for performing active learning */
    private ActiveLearning actlearning;
    
    /**Feature-Vectors for ambiguous to tokens to be classified by svm  */
    private AmbiguousVector[] batch;
    
    private int countPerAns = 0, countOrgAns = 0;
    
    /**The next ambiguous vector AmbiguousVectorNode[] */
    private int current_posInVectors;
    
    private String usedVectors, perAmbVectors, orgAmbVectors, vectorsInfoFP, vectorsInfoSP;
    
    private RandomAccessFile usedVectorsStream, perAmbVectorsStream, orgAmbVectorsStream, vectorsInfoFPStream, vectorsInfoSPStream;
    
    /**Array of Hashtables for words in lists. Each hashtable stands for a list of words*/
    private Hashtable<String, ListNode> lists[];
    
    /**
     * Size of batch of ambiguous batch to be classified by user
     */
    private int batchSize = 400;
    
    ////////////////////////////////////////////////////////////////////////////
    
    ////////////////////////////////////////////////////////////////////////////
    /**The main gui form of NERC*/
    private Nerc mainFrm;
    
    /**Frame for parameters*/
    private ParamsForm paramsForm;
    // paths loaded from parameters form
    private String
            /**lists directory path loaded from parameters form*/
            LISTS_DIRECTORY_LOADED_PATH,
            
            /**Constants.FIRST_PASS PERSON classifier model path loaded from parameters form*/
            FP_PER_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**Constants.FIRST_PASS ORGANIZATION classifier model path loaded from parameters form*/
            FP_ORG_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**SP ORGANIZATION classifier model path loaded from parameters form*/
            SP_PER_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**SP ORGANIZATION  classifier model path loaded from parameters form*/
            SP_ORG_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**Period separator classifier model path loaded from parameters form*/
            SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
            /**path to directory with active learning training files*/
            ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH ,
            /**test files directory path loaded from parameters form*/
            TEST_DIRECTORY_LOADED_PATH,
            /**Path to file with first pass used vectors from previous experiments*/
            FP_USED_VECTORS_FILE_LOADED_PATH,
            /**Path to file with second pass used vectors from previous experiments*/
            SP_USED_VECTORS_FILE_LOADED_PATH ,
            
            /**path to file with first pass ambiguous vectors */
            FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH ,
            /**path to file with first pass ambiguous vectors */
            FP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH ,
            /**path to file with second pass ambiguous vectors */
            SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH,
            /**path to file with second pass ambiguous vectors */
            SP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH,
            /**path to file with information about vectors*/
            FP_VECTORS_INFO_FILE_LOADED_PATH,
            SP_VECTORS_INFO_FILE_LOADED_PATH ;
    
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Constructes an ActiveLearningFrame. The File menu is visible but the back
     * button (backBut) is invisible to the user .
     *
     *
     * @param myClNaitiveTrainerr Object to be used
     */
    private Form(){
        
        
        //!!!!!!!!!!!!!!!!!!!!!!1!!!!!caution:!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        // follow these series of initialization
        // else a NullPointerException may be thrown
        // frame gui components and dimensions fixing
        
        // INITIALIZING GUI COMPONENTS OF FRAME
        initComponents();
        
        currPass = Constants.FIRST_PASS;
        paramsForm = new ParamsForm(this);
        
        trainer = new Trainer();
        
        loadParams(paramsForm);
        //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        ////////////////////////////////////////////////////////////////////////
        
        sysDefOut = System.out;
        fileM.setVisible(true);
        editM.setVisible(true);
        backBut.setVisible(false);
        
        current_posInVectors = -1;
        
        
    }//end of constructor
    /**
     * Constructor used when creating an object of this class from an existing
     * Nerc object. In this case the "file" menu from menu bar is
     * invisible to the user .
     *
     *
     *
     * @param mainFrm The Nerc object that creates this instance of ActLeForm
     */
    public Form(Nerc mainFrmIn){
        
        this();
        mainFrm = mainFrmIn;
        
        // in order to be able to see main frame again
        backBut.setVisible(true);
        
    }//constructor
    
    /**
     * Initializing graphical user interface components.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        FirstSecondPassButtonGrp = new javax.swing.ButtonGroup();
        passToolbarGrp = new javax.swing.ButtonGroup();
        methodSelectionGRP = new javax.swing.ButtonGroup();
        formPnl = new javax.swing.JPanel();
        northPanel = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jPanel6 = new javax.swing.JPanel();
        fpasradTool = new javax.swing.JRadioButton();
        spasradTool = new javax.swing.JRadioButton();
        initBut = new javax.swing.JButton();
        activeLearnPnl = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        neiborhoodTxt = new javax.swing.JTextPane();
        northLeftPnl = new javax.swing.JPanel();
        perBut = new javax.swing.JButton();
        orgBut = new javax.swing.JButton();
        noneBut = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        undoBut = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        paperLab = new javax.swing.JLabel();
        wordToClassifyLab = new javax.swing.JLabel();
        tokenPosLab = new javax.swing.JLabel();
        bottomPnl = new javax.swing.JPanel();
        backBut = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        startBut = new javax.swing.JButton();
        retrainBut = new javax.swing.JButton();
        testBut = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileM = new javax.swing.JMenu();
        backMI = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        editM = new javax.swing.JMenu();
        undoMI = new javax.swing.JMenuItem();
        processM = new javax.swing.JMenu();
        procResultsMI = new javax.swing.JMenuItem();
        procStatsMI = new javax.swing.JMenuItem();
        procMatrMI = new javax.swing.JMenuItem();
        optMenu = new javax.swing.JMenu();
        paramM = new javax.swing.JMenu();
        changeParamMI = new javax.swing.JMenuItem();
        initialize = new javax.swing.JMenu();
        initFPMU = new javax.swing.JMenu();
        fullListsFPMI = new javax.swing.JMenuItem();
        fpstartTrainMI = new javax.swing.JMenuItem();
        initSPMU = new javax.swing.JMenu();
        fullListsSPMI = new javax.swing.JMenuItem();
        spstartTrainMI = new javax.swing.JMenuItem();
        decisListsMI = new javax.swing.JMenuItem();
        experM = new javax.swing.JMenu();
        continusExper = new javax.swing.JCheckBoxMenuItem();
        passM = new javax.swing.JMenu();
        fpasRMI = new javax.swing.JRadioButtonMenuItem();
        spasRMI = new javax.swing.JRadioButtonMenuItem();
        saveListsCh = new javax.swing.JCheckBoxMenuItem();
        ActLeM = new javax.swing.JMenu();
        retrainMI = new javax.swing.JMenuItem();
        testMI = new javax.swing.JMenuItem();
        classifyM = new javax.swing.JMenu();
        perMI = new javax.swing.JMenuItem();
        orgMI = new javax.swing.JMenuItem();
        noneMI = new javax.swing.JMenuItem();
        batchM = new javax.swing.JMenu();
        findBatchMI = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Actice Learning");
        setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        formPnl.setLayout(new java.awt.BorderLayout());

        formPnl.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        formPnl.setToolTipText("Back to initial NERC form");
        northPanel.setLayout(new java.awt.BorderLayout());

        northPanel.setToolTipText("Messages Boxes View Area");
        jPanel3.setLayout(new java.awt.GridBagLayout());

        jPanel6.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 5));

        passToolbarGrp.add(fpasradTool);
        fpasradTool.setSelected(true);
        fpasradTool.setText("First Pass");
        fpasradTool.setToolTipText("First Pass");
        fpasradTool.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fpasradToolActionPerformed(evt);
            }
        });

        jPanel6.add(fpasradTool);

        passToolbarGrp.add(spasradTool);
        spasradTool.setText("Second Pass");
        spasradTool.setToolTipText("Second Pass ");
        spasradTool.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spasradToolActionPerformed(evt);
            }
        });

        jPanel6.add(spasradTool);

        initBut.setMnemonic('i');
        initBut.setText("Initialize files");
        initBut.setToolTipText("Initialize all files needed before starting active learning - must be performed only once before the first experiment of each pass");
        initBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(204, 0, 51), 2)));
        initBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                initButActionPerformed(evt);
            }
        });

        jPanel6.add(initBut);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 0.2;
        gridBagConstraints.weighty = 0.2;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel3.add(jPanel6, gridBagConstraints);

        northPanel.add(jPanel3, java.awt.BorderLayout.NORTH);

        activeLearnPnl.setLayout(new java.awt.BorderLayout());

        activeLearnPnl.setBorder(javax.swing.BorderFactory.createMatteBorder(1, 1, 2, 3, new java.awt.Color(0, 0, 0)));
        jPanel4.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setFocusable(false);
        neiborhoodTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        neiborhoodTxt.setToolTipText("neighbourhood (tokens around the token, currently under examination)");
        jScrollPane1.setViewportView(neiborhoodTxt);

        jPanel4.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        northLeftPnl.setLayout(new java.awt.GridBagLayout());

        perBut.setMnemonic('p');
        perBut.setText("Person (F7)");
        perBut.setEnabled(false);
        perBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                perButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(6, 6, 6, 6);
        northLeftPnl.add(perBut, gridBagConstraints);

        orgBut.setMnemonic('o');
        orgBut.setText("Organization (F8)");
        orgBut.setEnabled(false);
        orgBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                orgButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(6, 6, 6, 6);
        northLeftPnl.add(orgBut, gridBagConstraints);

        noneBut.setMnemonic('n');
        noneBut.setText("None (F9)");
        noneBut.setEnabled(false);
        noneBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noneButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(6, 6, 6, 6);
        northLeftPnl.add(noneBut, gridBagConstraints);

        jPanel2.setLayout(new java.awt.GridBagLayout());

        undoBut.setFont(new java.awt.Font("Tahoma", 1, 11));
        undoBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/wrong.gif")));
        undoBut.setText("Undo");
        undoBut.setToolTipText("Undo");
        undoBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(204, 0, 0)), new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        undoBut.setEnabled(false);
        undoBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        undoBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        undoBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undoButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHEAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(3, 21, 8, 8);
        jPanel2.add(undoBut, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 3;
        gridBagConstraints.ipady = 8;
        northLeftPnl.add(jPanel2, gridBagConstraints);

        jPanel4.add(northLeftPnl, java.awt.BorderLayout.EAST);

        activeLearnPnl.add(jPanel4, java.awt.BorderLayout.CENTER);

        jPanel5.setLayout(new java.awt.GridBagLayout());

        paperLab.setFont(new java.awt.Font("Courier New", 1, 12));
        paperLab.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        paperLab.setText("paper");
        paperLab.setToolTipText("Paper file ");
        paperLab.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        paperLab.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        paperLab.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                paperLabMouseEntered(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 302;
        gridBagConstraints.ipady = 6;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel5.add(paperLab, gridBagConstraints);

        wordToClassifyLab.setBackground(new java.awt.Color(0, 0, 0));
        wordToClassifyLab.setFont(new java.awt.Font("Comic Sans MS", 1, 12));
        wordToClassifyLab.setForeground(new java.awt.Color(204, 51, 0));
        wordToClassifyLab.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        wordToClassifyLab.setText("Token to classify");
        wordToClassifyLab.setToolTipText("Current ambiguous token");
        wordToClassifyLab.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(255, 0, 51), 3));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipadx = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 4, 3, 3);
        jPanel5.add(wordToClassifyLab, gridBagConstraints);

        tokenPosLab.setFont(new java.awt.Font("Courier New", 1, 12));
        tokenPosLab.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        tokenPosLab.setText("position");
        tokenPosLab.setToolTipText("Position in neighbourhood (phrase shown in test area)");
        tokenPosLab.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        tokenPosLab.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipadx = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 22, 3, 7);
        jPanel5.add(tokenPosLab, gridBagConstraints);

        activeLearnPnl.add(jPanel5, java.awt.BorderLayout.NORTH);

        northPanel.add(activeLearnPnl, java.awt.BorderLayout.CENTER);

        formPnl.add(northPanel, java.awt.BorderLayout.CENTER);

        bottomPnl.setLayout(new java.awt.GridBagLayout());

        backBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/back_05.jpg")));
        backBut.setText("Back");
        backBut.setToolTipText("main form");
        backBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0))), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        backBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(13, 0, 0, 0);
        bottomPnl.add(backBut, gridBagConstraints);

        jPanel1.setLayout(new java.awt.GridBagLayout());

        startBut.setBackground(new java.awt.Color(255, 255, 255));
        startBut.setFont(new java.awt.Font("Sylfaen", 1, 12));
        startBut.setForeground(new java.awt.Color(0, 102, 0));
        startBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/find.jpg")));
        startBut.setText("Get next batch");
        startBut.setToolTipText("Find batch of training vectors");
        startBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 102, 0), 2)));
        startBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        startBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        startBut.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                startButMouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                startButMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                startButMouseReleased(evt);
            }
        });
        startBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        jPanel1.add(startBut, gridBagConstraints);

        retrainBut.setFont(new java.awt.Font("Sylfaen", 1, 12));
        retrainBut.setForeground(new java.awt.Color(153, 0, 0));
        retrainBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/train_icon.JPG")));
        retrainBut.setText("Retrain");
        retrainBut.setToolTipText("Retrain SVM classifiers  (see command line for results)");
        retrainBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(204, 0, 0), 2)));
        retrainBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        retrainBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        retrainBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                retrainButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.5;
        jPanel1.add(retrainBut, gridBagConstraints);

        testBut.setFont(new java.awt.Font("Sylfaen", 1, 12));
        testBut.setForeground(new java.awt.Color(0, 0, 102));
        testBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/test_01.GIF")));
        testBut.setText("Perform testing");
        testBut.setToolTipText("Test (see command line for results)");
        testBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 102), 2)));
        testBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        testBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        testBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.5;
        jPanel1.add(testBut, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(11, 0, 0, 0);
        bottomPnl.add(jPanel1, gridBagConstraints);

        formPnl.add(bottomPnl, java.awt.BorderLayout.SOUTH);

        getContentPane().add(formPnl, java.awt.BorderLayout.CENTER);

        fileM.setMnemonic('f');
        fileM.setText("File");
        backMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, java.awt.event.InputEvent.CTRL_MASK));
        backMI.setMnemonic('b');
        backMI.setText("Back");
        backMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backMIActionPerformed(evt);
            }
        });

        fileM.add(backMI);

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        exitMenuItem.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        exitMenuItem.setForeground(new java.awt.Color(204, 51, 0));
        exitMenuItem.setMnemonic('e');
        exitMenuItem.setText("Exit");
        exitMenuItem.setToolTipText("terminates Application");
        exitMenuItem.setBorderPainted(true);
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileM.add(exitMenuItem);

        menuBar.add(fileM);

        editM.setMnemonic('d');
        editM.setText("Edit");
        undoMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, java.awt.event.InputEvent.CTRL_MASK));
        undoMI.setText("Undo decision");
        undoMI.setToolTipText("Undo last classification decision");
        undoMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undoMIActionPerformed(evt);
            }
        });

        editM.add(undoMI);

        processM.setText("Results");
        procResultsMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        procResultsMI.setFont(new java.awt.Font("Courier New", 1, 12));
        procResultsMI.setMnemonic('r');
        procResultsMI.setText("Results");
        procResultsMI.setToolTipText("Processes and gathers data in files with experimental results");
        procResultsMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                procResultsMIActionPerformed(evt);
            }
        });

        processM.add(procResultsMI);

        procStatsMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        procStatsMI.setFont(new java.awt.Font("Courier New", 1, 12));
        procStatsMI.setMnemonic('p');
        procStatsMI.setText("Statistics");
        procStatsMI.setToolTipText("Proccesses and gathers experimental statistics");
        procStatsMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                procStatsMIActionPerformed(evt);
            }
        });

        processM.add(procStatsMI);

        procMatrMI.setText("Matrixes");
        procMatrMI.setToolTipText("Processes and gathers data of matrixes in files with experimental results");
        procMatrMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                procMatrMIActionPerformed(evt);
            }
        });

        processM.add(procMatrMI);

        editM.add(processM);

        menuBar.add(editM);

        optMenu.setMnemonic('p');
        optMenu.setText("Options");
        optMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optMenuActionPerformed(evt);
            }
        });

        paramM.setText("Parameters");
        changeParamMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        changeParamMI.setText("Change parameters");
        changeParamMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeParamMIActionPerformed(evt);
            }
        });

        paramM.add(changeParamMI);

        initialize.setText("Initialize");
        initialize.setToolTipText("initialize parameters");
        initFPMU.setText("First Pass");
        initFPMU.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                initFPMUActionPerformed(evt);
            }
        });

        fullListsFPMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.SHIFT_MASK));
        fullListsFPMI.setText("Full lists");
        fullListsFPMI.setToolTipText("Creates full list files for first pass");
        fullListsFPMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fullListsFPMIActionPerformed(evt);
            }
        });

        initFPMU.add(fullListsFPMI);

        fpstartTrainMI.setText("Starting vectors");
        fpstartTrainMI.setToolTipText("Creates startTrain files which contain the initial training vectors of the initial classifier used before active learning");
        fpstartTrainMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fpstartTrainMIActionPerformed(evt);
            }
        });

        initFPMU.add(fpstartTrainMI);

        initialize.add(initFPMU);

        initSPMU.setText("Second Pass");
        initSPMU.setToolTipText("");
        fullListsSPMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.SHIFT_MASK));
        fullListsSPMI.setText("Full lists");
        fullListsSPMI.setToolTipText("Creates full list files for second pass");
        fullListsSPMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fullListsSPMIActionPerformed(evt);
            }
        });

        initSPMU.add(fullListsSPMI);

        spstartTrainMI.setText("Starting vectors");
        spstartTrainMI.setToolTipText("Creates startTrain files which contain the initial training vectors of the initial classifier used before active learning");
        spstartTrainMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spstartTrainMIActionPerformed(evt);
            }
        });

        initSPMU.add(spstartTrainMI);

        decisListsMI.setText("FP Decision Lists");
        decisListsMI.setToolTipText("Create lists of FP classifier decisions over training pool");
        decisListsMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decisListsMIActionPerformed(evt);
            }
        });

        initSPMU.add(decisListsMI);

        initialize.add(initSPMU);

        paramM.add(initialize);

        optMenu.add(paramM);

        experM.setMnemonic('x');
        experM.setText("Experiment");
        continusExper.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F10, java.awt.event.InputEvent.CTRL_MASK));
        continusExper.setText("Non-stop mode");
        continusExper.setToolTipText("Continues expreriments  - Stops only for user to classify vectrs in batch. See results saved");
        continusExper.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                continusExperActionPerformed(evt);
            }
        });

        experM.add(continusExper);

        passM.setText("Choose Pass");
        FirstSecondPassButtonGrp.add(fpasRMI);
        fpasRMI.setSelected(true);
        fpasRMI.setText("First pass");
        fpasRMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fpasRMIActionPerformed(evt);
            }
        });

        passM.add(fpasRMI);

        FirstSecondPassButtonGrp.add(spasRMI);
        spasRMI.setText("Second Pass");
        spasRMI.setToolTipText("");
        spasRMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spasRMIActionPerformed(evt);
            }
        });

        passM.add(spasRMI);

        experM.add(passM);

        saveListsCh.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.ALT_MASK));
        saveListsCh.setSelected(true);
        saveListsCh.setText("Save Lists");
        saveListsCh.setToolTipText("Saves lists after classification of vectors by user");
        saveListsCh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveListsChActionPerformed(evt);
            }
        });

        experM.add(saveListsCh);

        optMenu.add(experM);

        menuBar.add(optMenu);

        ActLeM.setMnemonic('a');
        ActLeM.setText("Active Learning");
        retrainMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.CTRL_MASK));
        retrainMI.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        retrainMI.setText("Retrain classifier");
        retrainMI.setToolTipText("Click to retrain classifier for first or second  pass (see Parameters|Choose pass)");
        retrainMI.setBorderPainted(true);
        retrainMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                retrainMIActionPerformed(evt);
            }
        });

        ActLeM.add(retrainMI);

        testMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
        testMI.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        testMI.setText("Test Classifier");
        testMI.setToolTipText("Click to retrain classifier for first or second  pass (see Parameters|Choose pass)");
        testMI.setBorderPainted(true);
        testMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testMIActionPerformed(evt);
            }
        });

        ActLeM.add(testMI);

        classifyM.setText("Classify as");
        perMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F7, 0));
        perMI.setText("PERSON");
        perMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                perMIActionPerformed(evt);
            }
        });

        classifyM.add(perMI);

        orgMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F8, 0));
        orgMI.setText("Organization");
        orgMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                orgMIActionPerformed(evt);
            }
        });

        classifyM.add(orgMI);

        noneMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F9, 0));
        noneMI.setText("None");
        noneMI.setToolTipText("Negative decision for both categories");
        noneMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noneMIActionPerformed(evt);
            }
        });

        classifyM.add(noneMI);

        ActLeM.add(classifyM);

        batchM.setText("Batch");
        findBatchMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        findBatchMI.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        findBatchMI.setText("Find batch");
        findBatchMI.setToolTipText("Starts Active Learning by computing batch according to method specified by user");
        findBatchMI.setBorderPainted(true);
        findBatchMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                findBatchMIActionPerformed(evt);
            }
        });

        batchM.add(findBatchMI);

        ActLeM.add(batchM);

        menuBar.add(ActLeM);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-632)/2, (screenSize.height-460)/2, 632, 460);
    }// </editor-fold>//GEN-END:initComponents
    
    private void initButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_initButActionPerformed
        
        JOptionPane jp = new JOptionPane();
        
        String[] passes = {Constants.FIRST_PASS, Constants.SECOND_PASS, "Cancel"};
        int ch = jp.showOptionDialog(
                this, "Choose classifiers pass:", "PASS",
                JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, passes, passes[1]);
        if(ch == 2) {
            return ;
        }
        
        if(ch == 0 ){// first pass initialization
            fullListsFPMI.doClick();
            fpstartTrainMI.doClick();
        }else{// second pass initialization
            fullListsSPMI.doClick();
            spstartTrainMI.doClick();
            decisListsMI.doClick();
        }
        
    }//GEN-LAST:event_initButActionPerformed
    
    private void optMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optMenuActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_optMenuActionPerformed
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        
        ImageIcon questionMark = new ImageIcon(getClass().getResource(Constants.IMA_WARN_01));
        
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
        int ans = JOptionPane.showConfirmDialog(null,
                "Are you sure you want to exit?",
                "Exit?", JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE, questionMark);
        if (JOptionPane.YES_OPTION == ans){
            System.exit(0);
        }
        
        
        
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    private void backMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backMIActionPerformed
        backBut.doClick();
    }//GEN-LAST:event_backMIActionPerformed
    
    private void initFPMUActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_initFPMUActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_initFPMUActionPerformed
    
    private void paperLabMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_paperLabMouseEntered
        
        paperLab.setToolTipText(paperLab.getText());
        
    }//GEN-LAST:event_paperLabMouseEntered
    
    private void procMatrMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_procMatrMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH);
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of result files");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir="";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            // choose category
            String []catoptions = new String[2];
            catoptions[0] = new String(Constants.PERSON);
            catoptions[1] = new String(Constants.ORGANIZATION);
            int userChoseCat =
                    dialog.showOptionDialog(
                    null, "Choose category of results to process",
                    "Category processing selection", dialog.OK_CANCEL_OPTION ,
                    dialog.QUESTION_MESSAGE, null, catoptions, catoptions[0]);
            
            
            if (userChoseCat == dialog.CLOSED_OPTION){
                return ;
            }
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractMatrix(dir, catoptions[userChoseCat], dir/*Save*/);
            
        }catch(Exception e){
            e.printStackTrace();
        }
        
    }//GEN-LAST:event_procMatrMIActionPerformed
    
    private void saveListsChActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveListsChActionPerformed
        
        
    }//GEN-LAST:event_saveListsChActionPerformed
    
    private void spstartTrainMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spstartTrainMIActionPerformed
        
        if(!createStartTRAINx(Constants.PERSON, Constants.SECOND_PASS)){
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
            String message = "Could not load initial trainging vectors file for category " + Constants.PERSON + ".";
            JOptionPane.showMessageDialog(this, message, "Warning!", JOptionPane.WARNING_MESSAGE,
                    new ImageIcon(getClass().getResource(Constants.IMA_WARN_01)));
            
        }else{
            
            System.out.println("Starting vectors for PERSON ok");
            
        }
        if (!createStartTRAINx(Constants.ORGANIZATION, Constants.SECOND_PASS)){
            
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
            String message = "Could not load initial trainging vectors file, for category " + Constants.ORGANIZATION + ".";
            JOptionPane.showMessageDialog(this, message, "Warning!", JOptionPane.WARNING_MESSAGE,
                    new ImageIcon(getClass().getResource(Constants.IMA_WARN_01)));
            
        }else{
            
            System.out.println("Starting vectors for ORGANIZATION ok");
            
        }
        
    }//GEN-LAST:event_spstartTrainMIActionPerformed
    
    private void fpstartTrainMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpstartTrainMIActionPerformed
        
        
        if(!createStartTRAINx(Constants.PERSON, Constants.FIRST_PASS)){
            
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
            String message = "Could not load initial trainging vectors file for category " + Constants.PERSON + ".";
            JOptionPane.showMessageDialog(this, message, "Warning!", JOptionPane.WARNING_MESSAGE,
                    new ImageIcon(getClass().getResource(Constants.IMA_WARN_01)));
        }else{
            
            System.out.println("Starting vectors for PERSON ok");
            
        }
        
        if (!createStartTRAINx(Constants.ORGANIZATION, Constants.FIRST_PASS)){
            
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
            String message = "Could not load initial trainging vectors file, for category " + Constants.ORGANIZATION + ".";
            JOptionPane.showMessageDialog(this, message, "Warning!", JOptionPane.WARNING_MESSAGE, new ImageIcon(getClass().getResource(Constants.IMA_WARN_01)));
            
        }else{
            
            System.out.println("Starting vectors for ORGANIZATION ok");
            
        }
        
        
    }//GEN-LAST:event_fpstartTrainMIActionPerformed
    
    private void startButMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startButMouseReleased
        
        startButPopUp(evt);
        
    }//GEN-LAST:event_startButMouseReleased
    
    private void startButMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startButMousePressed
        
        startButPopUp(evt);
        
    }//GEN-LAST:event_startButMousePressed
    
    
    private void startButPopUp(MouseEvent evt){
        
        if(evt.isPopupTrigger()){
            
            JPopupMenu popM = new JPopupMenu("Batch options");
            
            JMenuItem saveBatch = new JMenuItem("Save current batch");
            popM.add(saveBatch);
            
            JMenuItem loadBatch = new JMenuItem("Load former batch");
            popM.add(loadBatch);
            
            JMenuItem sizeBatch = new JMenuItem("Change batch size");
            popM.add(sizeBatch);
            
            popM.show(startBut, evt.getX(), evt.getY());
            
            ////////////////////////////////////////////////////////////////////////
            /////////////////////////// saving batch ///////////////////////////////
            saveBatch.addActionListener(new ActionListener() {
                
                public void actionPerformed(ActionEvent e) {
                    
                    if(batch == null){
                        
                        JOptionPane.showMessageDialog(null, "No batch to save", "Failed", JOptionPane.ERROR_MESSAGE);
                        return ;
                        
                    }
                    JFileChooser jCh = new JFileChooser();
                    jCh.setCurrentDirectory(new File("../files"));
                    jCh.setFileSelectionMode(jCh.FILES_ONLY);
                    int ans = jCh.showSaveDialog(Form.this);
                    
                    if (ans == jCh.APPROVE_OPTION){
                        
                        ObjectOutputStream oos;
                        try {
                            
                            oos = new ObjectOutputStream(
                                    new FileOutputStream(
                                    jCh.getSelectedFile().getCanonicalPath() + ".obj"));
                            oos.writeObject(Form.this.batch);
                            oos.close();
                            
                        } catch (FileNotFoundException ex) {
                            ex.printStackTrace();
                        } catch (IOException ex) {
                            ex.printStackTrace();
                        }
                        
                    }
                    
                }
            }
            );
            
            
            
            
            
            //////////////////////////////////////////////////////////////////////
            ////////////////////////// loading batch /////////////////////////////
            
            loadBatch.addActionListener(new BatchLoaderListener());
            
            
            //////////////// changing batch size //////////////////////////////
            
            sizeBatch.addActionListener(new ActionListener() {
                
                public void actionPerformed(ActionEvent e) {
                    
                    String size = JOptionPane.showInputDialog("Enter new size of batch: ", paramsForm.batchSizeTxt.getText());
                    
                    int sizeI;
                    try {
                        sizeI = Integer.parseInt(size);
                        
                    } catch (NumberFormatException ex) {
                        
                        JOptionPane.showMessageDialog(null, "Batch size remained the same, try again", "Wrong argument", JOptionPane.ERROR_MESSAGE);
                        return ;
                        
                    }
                    
                    paramsForm.batchSizeTxt.setText(size);
                    loadParams(paramsForm);
                    
                }
                
            });
            
            
            
        }
        
    }
    
    private void startButMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startButMouseClicked
        
        
    }//GEN-LAST:event_startButMouseClicked
    
    /**
     * Creating sure decisions lists and all decisions lists by previous pass classifiers
     */
    private void decisListsMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decisListsMIActionPerformed
        
        PrepareSP prepSP = new PrepareSP(Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
        Runnable run = prepSP.getPrevPassDecionListsThread(
                Constants.TIMEX_PATERNS_FILE_PATH, SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
                FP_PER_CLASSIFIER_FILE_LOADED_PATH, FP_ORG_CLASSIFIER_FILE_LOADED_PATH);
        
        Thread thr = new Thread(run);
        thr.start();
        
    }//GEN-LAST:event_decisListsMIActionPerformed
    
    
    private void fullListsSPMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fullListsSPMIActionPerformed
        
        // files which were used for training the initial classifiers with passive learning
        File[] filesSel = choseFilesSet("Chose text files used for training the initial classifiers", "ok");
        if (filesSel == null){
            return ;
        }
        initializeFullLists(Constants.PERSON, Constants.SECOND_PASS, filesSel);
        initializeFullLists(Constants.ORGANIZATION, Constants.SECOND_PASS, filesSel);
        
    }//GEN-LAST:event_fullListsSPMIActionPerformed
    
    private void continusExperActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_continusExperActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_continusExperActionPerformed
    
    private void procStatsMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_procStatsMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH + "/stats");
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of stats");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir = "";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        
        /*ch.setDialogTitle("Choose directory to saved processed stats");
        ch.setApproveButtonText("Save");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('s');
        ans = ch.showOpenDialog(null);
         
        String dirSave = dir;//= "";
        if (ans == ch.APPROVE_OPTION){
         
            dirSave = ch.getSelectedFile().toString();
         
        }else return ;
         */
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractStats(dir, dir/*Save*/);
            
        }catch(Exception e){e.printStackTrace();}
        
        
    }//GEN-LAST:event_procStatsMIActionPerformed
    
    private void procResultsMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_procResultsMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH);
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of result files");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir="";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        /*
        ch.setDialogTitle("Choose directory to saved process result files");
        ch.setApproveButtonText("Save");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('s');
        ans = ch.showOpenDialog(null);
        String dirSave = "";
         
        if (ans == ch.APPROVE_OPTION){
         
            dirSave = ch.getSelectedFile().toString();
         
        }else return ;
         */
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            // choose category
            String []catoptions = new String[2];
            catoptions[0] = new String(Constants.PERSON);
            catoptions[1] = new String(Constants.ORGANIZATION);
            int userChoseCat =
                    dialog.showOptionDialog(
                    null, "Choose category of results to process",
                    "Category processing selection", dialog.OK_CANCEL_OPTION ,
                    dialog.QUESTION_MESSAGE, null, catoptions, catoptions[0]);
            
            
            if (userChoseCat == dialog.CLOSED_OPTION){
                return ;
            }
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractResults(dir, catoptions[userChoseCat], dir/*Save*/);
            
        }catch(Exception e){
            e.printStackTrace();
        }
        
    }//GEN-LAST:event_procResultsMIActionPerformed
    
    private void undoMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_undoMIActionPerformed
        
        undoBut.doClick();
        
    }//GEN-LAST:event_undoMIActionPerformed
    
    private void fpasradToolActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpasradToolActionPerformed
        
        currPass = Constants.FIRST_PASS;
        // paramsForm.firstPass();
        fpasRMI.setSelected(true);
        fpasradTool.setSelected(true);
        
    }//GEN-LAST:event_fpasradToolActionPerformed
    
    private void spasradToolActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spasradToolActionPerformed
        
        
        currPass = Constants.SECOND_PASS;
        // paramsForm.secondPass();
        spasRMI.setSelected(true);
        spasradTool.setSelected(true);
        
    }//GEN-LAST:event_spasradToolActionPerformed
    
    private void startButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CLICK_01_FILE)).play();
        
        // load params in each experiment. They are most likely to change eg used vectors txt files
        loadParams(paramsForm);
        
        if ( batch != null){
            
            int ans = JOptionPane.showConfirmDialog(null, "There is already a batch computed. Do you wish to compute another one?",
                    "Batch already exists!", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (ans == JOptionPane.NO_OPTION) return ;
            
        }
        try {
            
            // redirecting deafault output to lastFile2KeepResults.
            // the old default output stream
            lastFile2KeepResults = createResultFile();
            resultStream = new FileOutputStream(lastFile2KeepResults);
            resultsOutput = new PrintStream(resultStream);
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (Exception exep) {
            exep.printStackTrace();
        }
        
        long start, end;
        start = System.currentTimeMillis();
        
        try {
            
            // first pass
            if (currPass.equals(Constants.FIRST_PASS)) {
                
                actlearning = new ActiveLearningFP(
                        Constants.TIMEX_PATERNS_FILE_PATH,
                        SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
                        LISTS_DIRECTORY_LOADED_PATH, batchSize);
                
                wordToClassifyLab.setText("Finding ambigious vectors...");
                System.out.println("Finding ambigious vectors...");
                
                // the batch containing the ambiguous batch found by calling method
                // "actlearning.findPerOrgAmbiguousVectors(...)"
                long startBatch = System.currentTimeMillis();
                batch = actlearning.findAndGetBatch(
                        FP_USED_VECTORS_FILE_LOADED_PATH,
                        ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH,
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        Constants.PER_ORG, ActiveLearning.INFORMATIVENESS, true);
                resultsOutput.print("Time for computing batch: " + Constants.toDigitalTimeString(startBatch) + "\n");
                System.out.print("Time for computing batch: " + Constants.toDigitalTimeString(startBatch) + "\n");
                
                // in case it was not possible to fill fully the batch
                batchSize = batch.length;
                
                String fileName = lastFile2KeepResults.getName();
                fileName = fileName.substring(0, fileName.indexOf('.'));
                // saving statistics for batch
                saveStatistics(lastFile2KeepResults.getParent() + "/batch_stats", fileName + ".sta");
                
                if(batch.length == 0) {
                    
                    JOptionPane.showMessageDialog(
                            null, "Batch Empty - No more Vectors available in pool. \n " +
                            "System will exit","Pool is Empty", JOptionPane.INFORMATION_MESSAGE);
                    return;
                    
                }
                
                System.out.println("batch filled, batch size = " + batch.length);
                wordToClassifyLab.setText("batch filled, batch size = " + batch.length);
                
            } // if
            
            else if (currPass.equals(Constants.SECOND_PASS)){
                
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                /////////////////////////////////////////// loading from first pass ///////////////////////////////////////////////
                
                ////////////////////////////////////////////////////////////////
                /////// reading lists of First pass decisions to be used ///////
                ////////////// for computing Second Pass features //////////////
                // currently used active learning files
                File currActLeFiles =
                        new File(ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH);
                
                File dirOfFPlists = new File(Constants.ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR);
                
                // reading all decisions of FP classifiers for currently used subpool of active learning files
                
                System.out.println("Loading from " + Constants.ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR);
                
                File allDecisionsF = new File(dirOfFPlists, Constants.ACTIVE_LEARNING_FP_ALL_DECISION_LISTS_FILENAME);
                FileInputStream in = new FileInputStream(allDecisionsF);
                ObjectInputStream obIn = new ObjectInputStream(in);
                allDec = (Hashtable<String, String>) obIn.readObject();
                obIn.close();
                
                // reading lists of sure classified tokens by FP classifiers, for currently used subpool of active learning files
                File sureDecisionsF = new File(dirOfFPlists, Constants.ACTIVE_LEARNING_FP_SURE_DECISIONS_FILENAME);
                in = new FileInputStream(sureDecisionsF);
                obIn = new ObjectInputStream(in);
                sureFPDec = (Hashtable<String, String[]>) obIn.readObject();
                obIn.close();
                
                System.out.println(sureDecisionsF.getCanonicalPath() + " Loaded ");
                System.out.println(allDecisionsF.getCanonicalPath() + " Loaded ");
                ///================== FP LISTS LOADED ==================================
                
                
                /////////////////////////////////////////// loading from first pass ///////////////////////////////////////////////
                //=================================================================================================================
                
                actlearning = new ActiveLearningSP(
                        allDec, sureFPDec, new File(FP_USED_VECTORS_FILE_LOADED_PATH),
                        Constants.TIMEX_PATERNS_FILE_PATH,
                        SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
                        LISTS_DIRECTORY_LOADED_PATH, batchSize);
                
                wordToClassifyLab.setText("Finding ambigious vectors...");
                System.out.println("Finding ambigious vectors...");
                
                // the batch containing the ambiguous batch found by calling method
                // "actlearning.findPerOrgAmbiguousVectors(...)"
                long startBatch = System.currentTimeMillis();
                batch = actlearning.findAndGetBatch(
                        SP_USED_VECTORS_FILE_LOADED_PATH, ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH,
                        SP_PER_CLASSIFIER_FILE_LOADED_PATH, SP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        Constants.PER_ORG, ActiveLearning.INFORMATIVENESS, true);
                
                resultsOutput.print("Time for computing batch: " + Constants.toDigitalTimeString(startBatch));
                System.out.println("Time for computing batch: " + Constants.toDigitalTimeString(startBatch));
                
                // in case it was not possible to fill fully the batch
                batchSize = batch.length;
                
                String fileName = lastFile2KeepResults.getName();
                fileName = fileName.substring(0, fileName.indexOf('.'));
                // saving statistics for batch
                long statStart = System.currentTimeMillis();
                saveStatistics(lastFile2KeepResults.getParent() + "/batch_stats", fileName + ".sta");
                
                if(batch.length == 0) {
                    
                    JOptionPane.showMessageDialog(
                            null, "Batch Empty - No more Vectors available in pool. \n " +
                            "System will exit","Pool is Empty", JOptionPane.INFORMATION_MESSAGE);
                    return;
                    
                }
                
                System.out.println("batch filled, batch size = " + batch.length);
                wordToClassifyLab.setText("batch filled, batch size = " + batch.length);
                
            } else {
                
                try {
                    
                    throw new Exception("Wrong Pass, only first and second passes supported");
                    
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                
            }
            
        } // try
        
        catch (Exception ex) {
            ex.printStackTrace();
        } // catch
        
        
        current_posInVectors = 0;
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////opening streams/////////////////////////////
        start = System.currentTimeMillis();
        // loading Lists
        loadFullLists();
        System.out.println("Lists loaded. Time needed: " + Constants.toDigitalTimeString(start) );
        
        ////enabling buttons and menu items/////
        undoBut.setEnabled(true);
        
        // startBut.setEnabled(false);
        // findBatchMI.setEnabled(false);
        
        passM.setEnabled(false);
        fpasradTool.setEnabled(false);
        spasradTool.setEnabled(false);
        
        System.out.println(
                "\nStart finished . . . ready to classify vectors in batch\n");
        
        perBut.setEnabled(true);
        orgBut.setEnabled(true);
        noneBut.setEnabled(true);
        
        perMI.setEnabled(true);
        orgMI.setEnabled(true);
        noneMI.setEnabled(true);
        
        retrainBut.setEnabled(true);
        
        ///////////////setting textfields for informing user////////////////////
        showInformation(System.out, -1, false);
        Toolkit.getDefaultToolkit().beep();
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_NOTIFY_FILE)).play();
        
    }//GEN-LAST:event_startButActionPerformed
    
    private void retrainButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_retrainButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CLICK_01_FILE)).play();
        
        if(this.batch == null){
            
            JFileChooser jCh = new JFileChooser();
            jCh.setCurrentDirectory(new File(Constants.RESULTS_DIRECTORY_PATH));
            jCh.setFileSelectionMode(jCh.FILES_ONLY);
            jCh.setDialogTitle("Choose file to save results");
            int ans = jCh.showSaveDialog(Form.this);
            if (ans == jCh.CANCEL_OPTION){
                return;
            }
            
            String fileName = (lastFile2KeepResults = jCh.getSelectedFile()).getName();
            
            try {
                
                resultsOutput = new PrintStream((resultStream = new FileOutputStream(lastFile2KeepResults)));
                personTrainVecs = organTrainVecs = -1;//unknown
                JOptionPane.showMessageDialog(this, "Retraining classifers without any batch loaded. \n" +
                        "See testing results in file:\n" + lastFile2KeepResults,
                        "Training Classifiers", JOptionPane.WARNING_MESSAGE);
                
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            } catch (IOException exep) {
                exep.printStackTrace();
            }
        }// if batch == null
        
        // we don't need it any more. Must be null in order to know that a new one is not recomputed
        batch = null;
        
        // time counting variables
        long start = System.currentTimeMillis();
        
        if (currPass.equals(Constants.FIRST_PASS)){
            
            System.out.println("Training First Pass SVMs for categories Person and Organization . . . ");
            
            System.out.println("Training PERSON SVM for First Pass...");
            // resultsOutput.println(Constants.PERSON+":");
            this.personTrainVecs =
                    trainer.trainALFP(
                    Constants.PERSON,
                    FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                    FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH,
                    resultsOutput);
            
            System.out.println("Training ORGANIZATION SVM for First Pass...");
            // resultsOutput.println(Constants.ORGANIZATION+":");
            this.organTrainVecs =
                    trainer.trainALFP(
                    Constants.ORGANIZATION,
                    FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                    FP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH,
                    resultsOutput);
            
        }
        
        else{///////////////// second pass active learning ////////////////////////////
            
            
            System.out.println("Training Second Pass SVMs for categories Person and Organization . . . ");
            
            // does not matter witch classifier is used
            int trainVecs[] =
                    trainer.trainALSP(
                    Constants.PER_ORG,
                    SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH, SP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH,
                    SP_PER_CLASSIFIER_FILE_LOADED_PATH, SP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                    resultsOutput);
            
            this.personTrainVecs = trainVecs[0];
            this.organTrainVecs = trainVecs[1];
            
        }//else Secondpass
        
        // retrainBut.setEnabled(false);
        retrainMI.setEnabled(false);
        
        // testBut.setEnabled(true);
        testMI.setEnabled(true);
        
        System.out.println("\nRetraining finished . . . ready to test\n");
        actlearning = null;
        System.gc();
        batch = null;
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_NOTIFY_FILE)).play();
        
    }//GEN-LAST:event_retrainButActionPerformed
    
    private void testButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_testButActionPerformed
        
        retrainBut.setEnabled(false);
        retrainMI.setEnabled(false);
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CLICK_01_FILE)).play();
        
        if(resultsOutput == null){
            
            
            JFileChooser jCh = new JFileChooser();
            jCh.setCurrentDirectory(new File(Constants.RESULTS_DIRECTORY_PATH));
            jCh.setFileSelectionMode(jCh.FILES_ONLY);
            jCh.setDialogTitle("Choose file to save results");
            int ans = jCh.showSaveDialog(Form.this);
            if (ans == jCh.CANCEL_OPTION){
                return;
            }
            
            String fileName = (lastFile2KeepResults = jCh.getSelectedFile()).getName();
            try {
                resultStream = new FileOutputStream(lastFile2KeepResults);
                resultsOutput = new PrintStream(resultStream);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            personTrainVecs = organTrainVecs = -1;//unknown
            
            
            
        }
        
        String perOutFileName = Constants.PERSON_ENTITIES_FILE_PATH;
        String orgOutFileName = Constants.ORGANIZATION_ENTITIES_FILE_PATH;
        
        ListManager fpPerListsMangr = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
        ListManager fpOrgListsMangr = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
        TimexMatcher dTRM = new TimexMatcher(Constants.TIMEX_PATERNS_FILE_PATH);
        SentenceSplitter pSep = new SentenceSplitter(SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH);
        
        if (currPass.equals(Constants.FIRST_PASS)){
            
            try{
                
                Tester.testFP(
                        dTRM, pSep,
                        TEST_DIRECTORY_LOADED_PATH,
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        fpPerListsMangr, perOutFileName,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        fpOrgListsMangr, orgOutFileName,
                        this.personTrainVecs, this.organTrainVecs,
                        this.resultsOutput);
                
            } catch(Exception e){
                
                e.printStackTrace();
            }
            ///////////////////////////////////////////////////////////////////////////////////////////////////
        } else{//////////////////////////////////// Second pass //////////////////////////////////////////////////
            ///////////////////////////////////////////////////////////////////////////////////////////////////
            ListManager spPerListsMangr = ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, LISTS_DIRECTORY_LOADED_PATH);
            ListManager spOrgListsMangr = ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, LISTS_DIRECTORY_LOADED_PATH);
            
            Tester.testSP(
                    dTRM, pSep,
                    TEST_DIRECTORY_LOADED_PATH,
                    FP_PER_CLASSIFIER_FILE_LOADED_PATH, fpPerListsMangr, SP_PER_CLASSIFIER_FILE_LOADED_PATH,
                    spPerListsMangr, perOutFileName,
                    FP_ORG_CLASSIFIER_FILE_LOADED_PATH, fpOrgListsMangr, SP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                    spOrgListsMangr, orgOutFileName,
                    
                    this.personTrainVecs, this.organTrainVecs,
                    this.resultsOutput);
            
        }//else Secondpass
        
        
        // startBut.setEnabled(true);
        // findBatchMI.setEnabled(true);
        
        // testBut.setEnabled(false);
        testMI.setEnabled(false);
        
        passM.setEnabled(true);
        fpasradTool.setEnabled(true);
        spasradTool.setEnabled(false);
        
        undoBut.setEnabled(false);
        
        spasradTool.setEnabled(true);
        spasRMI.setEnabled(false);
        fpasradTool.setEnabled(true);
        fpasRMI.setEnabled(false);
        
        System.out.println("\nTest finished . . .\n");
        resultsOutput.close();
        resultsOutput = null;
        
        
        
        retrainBut.setEnabled(false);
        retrainMI.setEnabled(false);
        
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_PROCESS_END_FILE)).play();
        
    }//GEN-LAST:event_testButActionPerformed
    
    private void perButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_perButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CATEG_FILE)).play();
        
        
        batch[current_posInVectors].setTagType(PER_DECISION);
        countPerAns++;
        showInformation(System.out, current_posInVectors, false);
        
        current_posInVectors ++;
        checkClassifiedAll(current_posInVectors);
        
    }//GEN-LAST:event_perButActionPerformed
    
    private void orgButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_orgButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CATEG_FILE)).play();
        
        
        batch[current_posInVectors].setTagType(ORG_DECISION);
        countOrgAns++;
        showInformation(System.out, current_posInVectors, false);
        
        current_posInVectors++;
        checkClassifiedAll(current_posInVectors);
        
    }//GEN-LAST:event_orgButActionPerformed
    
    private void noneButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noneButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CATEG_FILE)).play();
        
        
        batch[current_posInVectors].setTagType(NONE_DECISION);
        
        showInformation(System.out, current_posInVectors, false);
        
        current_posInVectors++;
        checkClassifiedAll(current_posInVectors);
        
    }//GEN-LAST:event_noneButActionPerformed
    
    private void undoButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_undoButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_DISCONNECT_FILE)).play();
        
        // checking range of position in array
        --current_posInVectors;
        if(current_posInVectors < 0) current_posInVectors = 0;
        if(batch[current_posInVectors].getTagType().equalsIgnoreCase(PER_DECISION))countPerAns--;
        if(batch[current_posInVectors].getTagType().equalsIgnoreCase(ORG_DECISION))countOrgAns--;
        
        showInformation(System.out, current_posInVectors, true);
        
    }//GEN-LAST:event_undoButActionPerformed
    
    private void noneMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noneMIActionPerformed
        
        noneBut.doClick();
        
    }//GEN-LAST:event_noneMIActionPerformed
    
    private void orgMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_orgMIActionPerformed
        
        orgBut.doClick();
        
    }//GEN-LAST:event_orgMIActionPerformed
    
    private void perMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_perMIActionPerformed
        
        perBut.doClick();
        
    }//GEN-LAST:event_perMIActionPerformed
    
    /**
     * Use this method to initilize file startTrain for this
     * category and pass, as specified by user in GUI.
     */
    private void fullListsFPMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fullListsFPMIActionPerformed
        
        // files which were used for training the initial classifiers with passive learning
        File[] filesSel = choseFilesSet("Chose text files used for training the initial classifiers", "ok");
        if (filesSel == null){
            return ;
        }
        
        initializeFullLists(Constants.PERSON, Constants.FIRST_PASS, filesSel);
        initializeFullLists(Constants.ORGANIZATION, Constants.FIRST_PASS, filesSel);
        
    }//GEN-LAST:event_fullListsFPMIActionPerformed
    
    private void spasradTool2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spasradTool2ActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_spasradTool2ActionPerformed
    
    private void fpasradTool2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpasradTool2ActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_fpasradTool2ActionPerformed
    
    private void testMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_testMIActionPerformed
        testBut.doClick();
    }//GEN-LAST:event_testMIActionPerformed
    
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        
        backMI.doClick();
        
    }//GEN-LAST:event_formWindowClosing
    /**Back to main form*/
    private void backButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_BACK_FILE)).play();
        
        mainFrm.setVisible(true);
        mainFrm.setEnabled(true);
        System.setOut(sysDefOut);
        
        dispose();//killing this window
        
    }//GEN-LAST:event_backButActionPerformed
    /** Retraining Person-SVM an Organization-SVM*/
    private void retrainMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_retrainMIActionPerformed
        retrainBut.doClick();
    }//GEN-LAST:event_retrainMIActionPerformed
    
    private void findBatchMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_findBatchMIActionPerformed
        startBut.doClick();
    }//GEN-LAST:event_findBatchMIActionPerformed
    
    
    private void fpasRMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpasRMIActionPerformed
        fpasradTool.doClick();
    }//GEN-LAST:event_fpasRMIActionPerformed
    
    private void changeParamMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeParamMIActionPerformed
        
        paramsForm.setVisible(true);
        
    }//GEN-LAST:event_changeParamMIActionPerformed
    
    private void spasRMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spasRMIActionPerformed
        
        spasradTool.doClick();
        
    }//GEN-LAST:event_spasRMIActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu ActLeM;
    private javax.swing.ButtonGroup FirstSecondPassButtonGrp;
    private javax.swing.JPanel activeLearnPnl;
    private javax.swing.JButton backBut;
    private javax.swing.JMenuItem backMI;
    private javax.swing.JMenu batchM;
    private javax.swing.JPanel bottomPnl;
    private javax.swing.JMenuItem changeParamMI;
    private javax.swing.JMenu classifyM;
    private javax.swing.JCheckBoxMenuItem continusExper;
    private javax.swing.JMenuItem decisListsMI;
    private javax.swing.JMenu editM;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu experM;
    private javax.swing.JMenu fileM;
    private javax.swing.JMenuItem findBatchMI;
    private javax.swing.JPanel formPnl;
    protected javax.swing.JRadioButtonMenuItem fpasRMI;
    protected javax.swing.JRadioButton fpasradTool;
    private javax.swing.JMenuItem fpstartTrainMI;
    private javax.swing.JMenuItem fullListsFPMI;
    private javax.swing.JMenuItem fullListsSPMI;
    private javax.swing.JButton initBut;
    private javax.swing.JMenu initFPMU;
    private javax.swing.JMenu initSPMU;
    private javax.swing.JMenu initialize;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.ButtonGroup methodSelectionGRP;
    private javax.swing.JTextPane neiborhoodTxt;
    private javax.swing.JButton noneBut;
    private javax.swing.JMenuItem noneMI;
    private javax.swing.JPanel northLeftPnl;
    private javax.swing.JPanel northPanel;
    private javax.swing.JMenu optMenu;
    private javax.swing.JButton orgBut;
    private javax.swing.JMenuItem orgMI;
    private javax.swing.JLabel paperLab;
    private javax.swing.JMenu paramM;
    private javax.swing.JMenu passM;
    private javax.swing.ButtonGroup passToolbarGrp;
    private javax.swing.JButton perBut;
    private javax.swing.JMenuItem perMI;
    private javax.swing.JMenuItem procMatrMI;
    private javax.swing.JMenuItem procResultsMI;
    private javax.swing.JMenuItem procStatsMI;
    private javax.swing.JMenu processM;
    private javax.swing.JButton retrainBut;
    private javax.swing.JMenuItem retrainMI;
    protected javax.swing.JCheckBoxMenuItem saveListsCh;
    protected javax.swing.JRadioButtonMenuItem spasRMI;
    protected javax.swing.JRadioButton spasradTool;
    private javax.swing.JMenuItem spstartTrainMI;
    private javax.swing.JButton startBut;
    private javax.swing.JButton testBut;
    private javax.swing.JMenuItem testMI;
    private javax.swing.JLabel tokenPosLab;
    private javax.swing.JButton undoBut;
    private javax.swing.JMenuItem undoMI;
    private javax.swing.JLabel wordToClassifyLab;
    // End of variables declaration//GEN-END:variables
    
    
    /**
     * Checks if user classified all vectors in batch.
     * Does eabling and disenabling buttons for procceding to training and testing
     */
    private void checkClassifiedAll(int currAmbVector){
        
        if(currAmbVector == batch.length){
            
            if( !askToSaveFiles(true) ) {
                return;
            }
            
            System.out.println("saving to classification files . . .");
            
            long now = System.currentTimeMillis();
            usedVectors = "";
            perAmbVectors = "";
            orgAmbVectors = "";
            vectorsInfoFP = "";
            vectorsInfoSP = "";
            for (int j = 0; j < batch.length; j++){
                
                updateFiles(j);
                
            }
            
            System.out.println("Classification files Created. Time needed: " + Constants.toTimeString(now));
            saveFiles(System.out);
            
            // retrainBut.setEnabled(true);
            retrainMI.setEnabled(true);
            
            perBut.setEnabled(false);
            orgBut.setEnabled(false);
            noneBut.setEnabled(false);
            
            perMI.setEnabled(false);
            orgMI.setEnabled(false);
            noneMI.setEnabled(false);
            
            System.out.println(
                    "\nClassification finished .  .  .\n" +
                    "Ready to retrain SVMs\n");
            
            if(continusExper.isSelected()){
                
                // retrainBut.setEnabled(true);
                retrainBut.doClick();
                // retrainBut.setEnabled(false);
                
                // testBut.setEnabled(true);
                testBut.doClick();
                // testBut.setEnabled(false);
                
                // startBut.setEnabled(true);
                startBut.doClick();
                // startBut.setEnabled(false);
                
            }
            
        }
        
    }
    
    /**
     * Creates and returns paths for the output files - The files of the experiments results
     *
     * @returns The path to the output-result files
     */
    private File createResultFile(){
        
        String catPass = "";
        if (currPass.equals(Constants.FIRST_PASS))
            catPass = "AL" + Constants.FIRST_PASS;
        else if (currPass.equals(Constants.SECOND_PASS))
            catPass = "AL" + Constants.SECOND_PASS;
        
        
        
        // date for naming directory of result files
        GregorianCalendar calendar = new GregorianCalendar();
        
        int month_int=
                1 + calendar.get(calendar.MONTH),
                day_int = calendar.get(calendar.DAY_OF_MONTH);
        
        String
                year = String.valueOf(calendar.get(calendar.YEAR)),
                month, day;
        
        if(month_int < 10)
            month = "0".concat(String.valueOf(month_int));
        else month = String.valueOf(month_int);
        
        if(day_int < 10) day = "0".concat(String.valueOf(day_int));
        else day = String.valueOf(day_int);
        
        
        String resultsDir = new String(Constants.RESULTS_DIRECTORY_PATH);
        
        File file2KeepResults = new File(resultsDir);
        
        file2KeepResults=new File(file2KeepResults, catPass);
        
        
        file2KeepResults = new File(
                file2KeepResults,
                year + "_" + month + "_" + day);
        
        file2KeepResults.mkdirs();
        
        // num of experiment files performed today (according to date-see above)
        String num = "01";
        
        try{
            
            //log for todays result files
            File log = new File(file2KeepResults.getParentFile() , "experiments.log");
            
            if (log.exists()){
                
                /////////////////////////reading from log///////////////////////
                FileReader logdat = new FileReader(log);
                char []cBuf = new char[(int)log.length()];
                logdat.read(cBuf);
                num = new String(cBuf);
                logdat.close();
                //====================reading from log==========================
                
                int numb=Integer.parseInt(num)+1;
                num=(numb<10)?"0"+num.valueOf(numb):num.valueOf(numb);
                
                FileWriter logWr=new FileWriter(log);
                logWr.write(num);
                logWr.flush();
                logWr.close();
                
            }
            
            else{// log does not exist
                
                FileWriter logWr=new FileWriter(log);
                logWr.write("01");
                logWr.flush();
                logWr.close();
                
            }
            
            String suffix = "per_org";
            // FULL PATH TO FILE TO KEEP RESULTS
            String  parentFile = file2KeepResults.getCanonicalPath();
            file2KeepResults = new File(
                    parentFile + "/" +
                    "expmnt_" + num + "." + suffix);
            
        } catch(IOException ex){
            
            ex.printStackTrace();
            
        }
        
        return file2KeepResults;
        
    }
    
    /**
     *
     */
    private boolean createStartTRAINx(String category, String pass){
        
        String toReturn = "";
        // file of starting train files
        /*File TRAINfile =  new File(Constants.FP_PERSON_TRAIN_VECTORS_FILE);
         
        JFileChooser jf = new JFileChooser(TRAINfile.getParent());
        jf.setDialogTitle("Choose file containing training vectors of " + category + " " + pass + " classifier");
         */
        String startTrainFile;
        String trainFile;
        if (category.equals(Constants.PERSON)){
            
            if(pass.equals(Constants.FIRST_PASS)){
                startTrainFile = Constants.FP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
                trainFile = Constants.FP_PERSON_TRAIN_VECTORS_FILE;
            }else{
                startTrainFile = Constants.SP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
                trainFile = Constants.SP_PERSON_TRAIN_VECTORS_FILE;
            }
            
        }else{//organization
            
            if(pass.equals(Constants.FIRST_PASS)){
                startTrainFile = Constants.FP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
                trainFile = Constants.FP_ORGANIZATION_TRAIN_VECTORS_FILE;
            }else{
                startTrainFile = Constants.SP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE;
                trainFile = Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE;
            }
            
            
        }
        
        /*int ans = jf.showOpenDialog(this);
        if (ans!=jf.APPROVE_OPTION) return false;
        TRAINfile = jf.getSelectedFile();
        TRAINfile.renameTo*/
        File TRAINFile = new File(trainFile);
        if(!TRAINFile.exists()) return false;
        TRAINFile.renameTo(new File(startTrainFile));
        
        return true;
        
    }//end of createStartTRAINx()
    
    
    /**
     * Asks user to specify  <B>files of passive learning training set</B>.
     * @return A File array containing the chosen training set (set of text files). Else, return null.
     */
    private File[] choseFilesSet(String message, String butLabel ){
        
        String[]ch = new String[3];
        JFileChooser jch = new JFileChooser(message);
        jch.setApproveButtonText(butLabel);
        jch.setDialogTitle(message);
        jch.setMultiSelectionEnabled(true);
        jch.setFileSelectionMode(jch.FILES_ONLY);
        jch.setCurrentDirectory(new File(Constants.FILES_DIR_PATH).getParentFile());
        
        
        if(jch.showOpenDialog(this) != jch.APPROVE_OPTION) return null;
        
        File[] filesSel = jch.getSelectedFiles();
        
        return filesSel;
        
    }
    
    private void initializeFullLists(String category, String pass, File[] filesSel){
        
        String outputDirName = Constants.FULL_LISTS_DIRECTORY_PATH;
        separator = new SentenceSplitter(SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH);
        
        
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 5, 20, 7, separator, outputDirName);
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 5, 20, 1, separator, outputDirName);
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 3, 4, 7, separator, outputDirName);
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 3, 4, 1, separator, outputDirName);
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 1, 2, 7, separator, outputDirName);
        ListCreator.getPreviousTokens(filesSel, category, pass, 0, 1, 2, 1, separator, outputDirName);
        
        System.out.println("\nFull lists for category " + category + " created\n");
        
    }
    
    /**
     * To be called after finding ambiguous batch for batch.
     * Saves files and informs user.
     */
    private boolean askToSaveFiles(boolean dontAsk){
        
        String[]ch = new String[2];
        ch[0] = "Save";
        ch[1] = "Undo";
        
        int choice;
        if(!dontAsk){
            choice =
                    
                    dialog.showOptionDialog(null,
                    "Do you want to save all changes to files, or do you want to undo last decision?",
                    "Classification - Retraining - Testing Finished!",dialog.YES_NO_OPTION,
                    dialog.QUESTION_MESSAGE, null, ch, ch[0]);
            
        }else {
            choice = 0;
        }
        
        if(choice == 0) {
            
            // retrainBut.setEnabled(true);
            retrainMI.setEnabled(true);
            
            // retrainBut.setEnabled(true);
            retrainMI.setEnabled(true);
            
            passM.setEnabled(false);
            
            return true;
            
        } else {
            
            undoBut.doClick();
            return false;
            
        }
        
    }
    
    /**
     * Informs user for classification decision of current vector, and shows
     * information about next vector to be classified.
     */
    private void showInformation(
            PrintStream out, int posOfVec, boolean undo) {
        
        if(posOfVec > -1 && !undo){
            
            if( posOfVec  >= batch.length ) return ;
            AmbiguousVector lastClassifiedVec = batch[posOfVec];
            String decision = lastClassifiedVec.getTagType();
            
            out.println(
                    "Classification of vector " + (posOfVec + 1) +
                    " out of " + batch.length + ". Token " +
                    lastClassifiedVec.getToken() + " classified as: decision " + decision
                    );
            
        }else if(undo){
            
            out.println("Decsion undone. Token at position " + (posOfVec + 1 ) +
                    " of batch, to be classified again");
            
        }
        
        ////////////////////////////////////////////////////////////////////////
        /////////// informing user for next vector to be classified ////////////
        AmbiguousVector next;
        
        if(posOfVec < 0) {
            // in case of first token in batch to be classified
            next = batch[0];
            
        }else if(!undo){
            
            if( posOfVec  >= (batch.length-1) ) return ;
            // show information about next token
            next = batch[posOfVec + 1];
            
        }else{
            
            if( posOfVec >= batch.length) return ;
            // show information about current token (token at posOfVec)
            next = batch[posOfVec];
            
        }
        
        wordToClassifyLab.setText(next.getToken());
        paperLab.setText(next.getFileName());
        String tmp = "";
        int l = next.getPhrase().length;
        
        int countChars = 0; // count characters to token under classification
        for (int i = 0; i < l; i++) {
            countChars += ( l < next.getPosInPhrase()) ? next.getPhrase()[i].length() + 1:0;
            tmp += next.getPhrase()[i] + " ";
        } // for
        
        // selecting - toning with red color the token of the current ambiguous vector
        int toneInd = tmp.indexOf(next.getToken(), countChars);
        neiborhoodTxt.setText(tmp);
        neiborhoodTxt.setSelectionColor(Color.RED);
        neiborhoodTxt.grabFocus();
        neiborhoodTxt.select(toneInd, toneInd + next.getToken().length());
        neiborhoodTxt.setEditable(false);
        tokenPosLab.setText( String.valueOf(next.getPosInPhrase() + 1 ));
        
    } // showInformation(System.resultsOutput)
    
    private static Hashtable<String, ListNode> loadList(String listfn) {
        
        Hashtable<String, ListNode> hashtable = new Hashtable<String, ListNode>();
        
        try {
            
            File f = new File(listfn);
            FileInputStream fis = new FileInputStream(f);
            byte[] buff = new byte[ (int) f.length()];
            fis.read(buff);
            fis.close();
            String list = new String(buff);
            list = ipl.nerc.preprocessing.TextPreprocessor.removeNoisySpaces(list);
            StringTokenizer strTok = new StringTokenizer(list, " \n");
            
            while (strTok.hasMoreTokens()) {
                
                String key = strTok.nextToken();
                int count = ipl.util.Auxiliary.atoi(strTok.nextToken());
                ListNode node = new ListNode(key, count);
                hashtable.put(key, node);
                
            } // while
            
        }catch (Exception ex) {
            ex.printStackTrace();
        } // catch
        
        return hashtable;
        
    } // loadList
    
    /**
     *  12         , 
     *         ,     
     *  
     */
    private void loadFullLists() {
        
        String listsDir = Constants.FULL_LISTS_DIRECTORY_PATH;
        
        String list0 = Constants.PERSON + "/lessOcc=0,minWordLen=5,maxWordLen=20," +
                "numPrevWords=7.txt";
        String list1 = Constants.PERSON + "/lessOcc=0,minWordLen=5,maxWordLen=20," +
                "numPrevWords=1.txt";
        String list2 = Constants.PERSON + "/lessOcc=0,minWordLen=3,maxWordLen=4," +
                "numPrevWords=7.txt";
        String list3 = Constants.PERSON + "/lessOcc=0,minWordLen=3,maxWordLen=4," +
                "numPrevWords=1.txt";
        String list4 = Constants.PERSON + "/lessOcc=0,minWordLen=1,maxWordLen=2," +
                "numPrevWords=7.txt";
        String list5 = Constants.PERSON + "/lessOcc=0,minWordLen=1,maxWordLen=2," +
                "numPrevWords=1.txt";
        String list6 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=5,maxWordLen=20," +
                "numPrevWords=7.txt";
        String list7 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=5,maxWordLen=20," +
                "numPrevWords=1.txt";
        String list8 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=3,maxWordLen=4," +
                "numPrevWords=7.txt";
        String list9 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=3,maxWordLen=4," +
                "numPrevWords=1.txt";
        String list10 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=1,maxWordLen=2," +
                "numPrevWords=7.txt";
        String list11 = Constants.ORGANIZATION + "/lessOcc=0,minWordLen=1,maxWordLen=2," +
                "numPrevWords=1.txt";
        
        lists = new<String, ListNode> Hashtable[12];
        
        String pass = currPass;
        
        lists[0] = loadList(listsDir + "/" + pass + "/" +  list0);
        lists[1] = loadList(listsDir + "/" +  pass + "/" + list1);
        lists[2] = loadList(listsDir + "/" +  pass + "/" + list2);
        lists[3] = loadList(listsDir + "/" +  pass + "/" + list3);
        lists[4] = loadList(listsDir + "/" +  pass + "/" + list4);
        lists[5] = loadList(listsDir + "/" +  pass + "/" + list5);
        lists[6] = loadList(listsDir + "/" +  pass + "/" + list6);
        lists[7] = loadList(listsDir + "/" +  pass + "/" + list7);
        lists[8] = loadList(listsDir + "/" +  pass + "/" + list8);
        lists[9] = loadList(listsDir + "/" +  pass + "/" + list9);
        lists[10] = loadList(listsDir + "/" +  pass + "/" + list10);
        lists[11] = loadList(listsDir + "/" +  pass + "/" + list11);
        
        
    } // loadFullLists
    
    public void loadParams(){
        
        this.loadParams(this.paramsForm);
        
    }
    /**
     * Loads all parameters as defined in the parameters form (field paramsFrm)
     */
    public  void loadParams( ParamsForm paramsFormIn){
        
        // loadbatch size
        batchSize = paramsFormIn.batchSize();
        // used vecs
        FP_USED_VECTORS_FILE_LOADED_PATH = paramsFormIn.FPusedVectorsFileNameTxt.getText();
        SP_USED_VECTORS_FILE_LOADED_PATH = paramsFormIn.SPusedVectorsFileNameTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        // ambiguous vecs
        FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH =
                paramsFormIn.FPPERambiguousVectorsFileNameTxt.getText();
        SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH =
                paramsFormIn.SPPERambiguousVectorsFileNameTxt.getText();
        
        FP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH =
                paramsFormIn.FPORGambiguousVectorsFileNameTxt.getText();
        SP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH =
                paramsFormIn.SPORGambiguousVectorsFileNameTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        // info path
        FP_VECTORS_INFO_FILE_LOADED_PATH =
                paramsFormIn.FPvectorsInformationsFileNameTxt.getText();
        SP_VECTORS_INFO_FILE_LOADED_PATH =
                paramsFormIn.SPvectorsInformationsFileNameTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        // lists
        LISTS_DIRECTORY_LOADED_PATH =
                paramsFormIn.listsDirectoryTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        // active learning files
        ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH =
                paramsFormIn.activeLearningFilesDirectoryTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        //classifiers
        SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH =
                paramsFormIn.sentenceSplitterModelFileNameTxt.getText();
        trainer.setSeparator(SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH);
        trainer.setSeparator(SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH);
        // fp
        FP_PER_CLASSIFIER_FILE_LOADED_PATH =
                paramsFormIn.FPPerClassifierFileNameTxt.getText();
        FP_ORG_CLASSIFIER_FILE_LOADED_PATH =
                paramsFormIn.FPOrgClassifierFileNameTxt.getText();
        // sp
        SP_PER_CLASSIFIER_FILE_LOADED_PATH =
                paramsFormIn.SPPerClassifierFileNameTxt.getText();
        SP_ORG_CLASSIFIER_FILE_LOADED_PATH =
                paramsFormIn.SPOrgClassifierFileNameTxt.getText();
        ////////////////////////////////////////////////////////////////////////
        
        trainer.setMatcher(Constants.TIMEX_PATERNS_FILE_PATH);
        
        //test files path
        TEST_DIRECTORY_LOADED_PATH =
                paramsFormIn.testFilesDirectoryTxt.getText();
        
        
    }
    
    /**
     * There are 6 lists for each of two categories person and organization.
     * Each list differs from the other by the length of its words (params:
     * minWordLength,maxWordLength) and the distance of its word(keys) from
     * the token being categorized as person or organization (numOfPreviousWords)
     *
     * @param list the hashtable list to be updated
     * @param key Word to increase number of found instances in list(
     *          or to be added in the list if not allready in list)
     */
    private void updateList(
            Hashtable<String, ListNode> list, String key,
            int distance,int minWordLength, int maxWordLength,
            int numOfPreviousWords) {
        
        // if the length of the word(key) is proper for this list
        if ((key.length() >= minWordLength) &&
                (key.length() <= maxWordLength) &&
                (distance <= numOfPreviousWords)) {
            // key-word already in list
            if (list.containsKey(key)) {
                list.get(key).increase();
            } // if
            // if not word-key in list, then add it
            else {
                ListNode node = new ListNode(key);
                list.put(key,node);
            } // else
        } // if
    } // updateList
    
    /**
     * Updates all files including lists and paths, according to the decision of
     * user for the currently examined vector ambiguousVector in the batch.
     *
     * @param decision
     *      A String indicating the user decision wicth can be one
     *      the following:
     *          - this.PER_DECISION
     *          - this.ORG_DECISION
     *          - this.NONE_DECISION
     */
    private void updateFiles(int posOfVector) {
        
        AmbiguousVector ambiguousVector = batch[posOfVector];
        
        String decision = ambiguousVector.getTagType();
        
        // updating used vectors
        usedVectors += ambiguousVector.getFileName() + " " + ambiguousVector.getPosition() + "\n";
        
        ////////////////////////////////////////////////////////////////////////
        ///////////converting table of features to a string representing////////
        /////////////////////features stored in table///////////////////////////
        StringBuffer perFeatVecBuf = new StringBuffer("");
        StringBuffer orgFeatVecBuf = new StringBuffer("");
        String[] arrFeats = ambiguousVector.getFeatureVector();
        
        // string representation of vector of features for each category
        String perFeaturesStr ;
        String orgFeaturesStr ;
        
        
        // JOptionPane.showMessageDialog(null, "counted features: " + arrFeats.length);
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////////// first pass //////////////////////////////
        ////////////////////////////////////////////////////////////////////////
        if(currPass.equals(Constants.FIRST_PASS)){
            
            // in case features are not computed by not appling safe rules for person
            if(ambiguousVector.isPersonSafeRulesApplied()){
                
                for(int i = 1; i < Constants.PER_FEAT_NUM_FP + 1; i++){
                    
                    perFeatVecBuf.append("\t" + i + ":" + arrFeats[i]);
                    
                }// for
                
            }// if
            
            // in case features are not computed by not appling safe rules for organization
            if(ambiguousVector.isOrganizationSafeRulesApplied()){
                
                int length = Constants.PER_FEAT_NUM_FP + Constants.ORG_FEAT_NUM_FP + 2;
                for(int i = Constants.PER_FEAT_NUM_FP + 2; i < arrFeats.length; i++){
                    
                    int j = i - Constants.PER_FEAT_NUM_FP -1;
                    orgFeatVecBuf.append("\t" + j + ":" + arrFeats[i]);
                    
                }// for
                
            }// if
            
            
            // string representation of vector of features for each category
            perFeaturesStr = perFeatVecBuf.toString();
            orgFeaturesStr = orgFeatVecBuf.toString();
            //////////////////////////////////////////////////////////////////////
            /////////////////////////batch information////////////////////////////
            String temp = "";
            temp += decision + perFeaturesStr + "," + orgFeaturesStr + "\n";
            temp += ambiguousVector.getToken() + "\n";
            int length = ambiguousVector.getPhrase().length;
            
            for (int i = 0; i < length; i++) {
                
                temp += ambiguousVector.getPhrase()[i] + " ";
                
            } // for
            
            temp += "\n" + ambiguousVector.getPosInPhrase() + "\n";
            temp += ambiguousVector.getPosition() + "\n";
            temp += ambiguousVector.getFileName() + "\n";
            vectorsInfoFP += temp;
            
            
        }
        
        
        ///////////////////////////////////////////////////////////////////////
        ///////////////////////// Second Pass /////////////////////////////////
        ///////////////////////////////////////////////////////////////////////
        else{
            // in case features are not computed by not appling safe rules for person
            if(ambiguousVector.isPersonSafeRulesApplied()){
                
                for(int i = 1; i < Constants.PER_FEAT_NUM_SP + 1; i++){
                    
                    perFeatVecBuf.append("\t" + i + ":" + arrFeats[i]);
                    
                }// for
                
            }// if
            
            // in case features are not computed by not appling safe rules for organization
            if(ambiguousVector.isOrganizationSafeRulesApplied()){
                
                int length = Constants.PER_FEAT_NUM_SP + Constants.ORG_FEAT_NUM_SP + 2;
                for(int i = Constants.PER_FEAT_NUM_SP + 2; i < arrFeats.length; i++){
                    
                    int j = i - Constants.PER_FEAT_NUM_SP -1;
                    orgFeatVecBuf.append("\t" + j + ":" + arrFeats[i]);
                    
                }// for
                
            }// if
            
            
            // string representation of vector of features for each category
            perFeaturesStr = perFeatVecBuf.toString();
            orgFeaturesStr = orgFeatVecBuf.toString();
            
            //////////////////////////////////////////////////////////////////////
            /////////////////////////batch information////////////////////////////
            
            String temp = "";
            temp += decision + perFeaturesStr + "," + orgFeaturesStr + "\n";
            temp += ambiguousVector.getToken() + "\n";
            int length = ambiguousVector.getPhrase().length;
            
            for (int i = 0; i < length; i++) {
                
                temp += ambiguousVector.getPhrase()[i] + " ";
                
            } // for
            
            temp += "\n" + ambiguousVector.getPosInPhrase() + "\n";
            temp += ambiguousVector.getPosition() + "\n";
            temp += ambiguousVector.getFileName() + "\n";
            vectorsInfoSP += temp;
            
        }// second pass
        
        
        //=========converting table of features to a string representing========
        //===================features stored in table===========================
        //======================================================================
        
        
        // update lists if user says it is of  Constants.PER_ORG person
        if (decision.compareTo(PER_DECISION) == 0) {
            
            // adding to ambiguous batch, decision and featutres
            if(ambiguousVector.isPersonSafeRulesApplied()){
                perAmbVectors += "1" + perFeaturesStr + "\n";
            }
            if(ambiguousVector.isOrganizationSafeRulesApplied()){
                orgAmbVectors += "-1" + orgFeaturesStr + "\n";
            }
            
            int l = ambiguousVector.getPreviousTokens().length;
            for (int i = 0; i < l; i++) {
                
                String key = ambiguousVector.getPreviousTokens()[i];
                int dist = i + 1;
                
                updateList(lists[0], key, dist, 5, 20, 7);
                updateList(lists[1], key, dist, 5, 20, 1);
                updateList(lists[2], key, dist, 3, 4, 7);
                updateList(lists[3], key, dist, 3, 4, 1);
                updateList(lists[4], key, dist, 1, 2, 7);
                updateList(lists[5], key, dist, 1, 2, 1);
                
            } // for
        } // if
        
        
        // update lists if user says it is of  Constants.PER_ORG organization
        else if (decision.compareTo(ORG_DECISION) == 0) {
            
            // adding to ambiguous vectors, decision and featutres
            if(ambiguousVector.isPersonSafeRulesApplied()){
                perAmbVectors += "-1" + perFeaturesStr + "\n";
            }
            if(ambiguousVector.isOrganizationSafeRulesApplied()){
                orgAmbVectors += "1" + orgFeaturesStr + "\n";
            }
            
            int l = ambiguousVector.getPreviousTokens().length;
            for (int i = 0; i < l; i++) {
                
                String key = ambiguousVector.getPreviousTokens()[i];
                int dist = i + 1;
                updateList(lists[6], key, dist, 5, 20, 7);
                updateList(lists[7], key, dist, 5, 20, 1);
                updateList(lists[8], key, dist, 3, 4, 7);
                updateList(lists[9], key, dist, 3, 4, 1);
                updateList(lists[10], key, dist, 1, 2, 7);
                updateList(lists[11], key, dist, 1, 2, 1);
                
            } // for
            
        } // if
        
        else{// none
            
            // adding to ambiguous vectors, decision and featutres
            if(ambiguousVector.isPersonSafeRulesApplied()){
                
                perAmbVectors += "-1" + perFeaturesStr + "\n";
                
            }
            if(ambiguousVector.isOrganizationSafeRulesApplied()){
                
                orgAmbVectors += "-1" + orgFeaturesStr + "\n";
                
            }
            
        }
        
    } // updateFiles
    
    /**
     * Saves all files  (lists, ambiguous feature vectors etc).
     * To be called before calling training methods (in order files containing
     * feature vectors to be saved in paths used as parameters to training methods);
     */
    private void saveFiles(PrintStream output) {
        
        output.println("Saving files . . . ");
        long start = System.currentTimeMillis();
        
        try {
            
            ////////////////////////////////////////////////////////////////////
            //////////////////////reading used batch//////////////////////////
            if (currPass.equals(Constants.FIRST_PASS)) {
                
                long savT = System.currentTimeMillis();
                usedVectorsStream = new RandomAccessFile(FP_USED_VECTORS_FILE_LOADED_PATH, "rw");
                System.out.println("Saving used vectors in path:\n" + FP_USED_VECTORS_FILE_LOADED_PATH);
                // go to end of used vectors stream, so append new records at this point of file
                usedVectorsStream.seek(usedVectorsStream.length());
                usedVectorsStream.writeBytes(usedVectors);
                usedVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                new File(FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH).getParentFile().mkdirs();
                perAmbVectorsStream = new RandomAccessFile(FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH, "rw");
                savT = System.currentTimeMillis();
                System.out.println("Saving Person Ambiguous vectors in path:\n" + FP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH);
                // person
                // go to end of perAmbVectorsStream, so append new records at this point of file
                perAmbVectorsStream.seek(perAmbVectorsStream.length());
                perAmbVectorsStream.writeBytes(perAmbVectors);
                perAmbVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                orgAmbVectorsStream = new RandomAccessFile(FP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH, "rw");
                savT = System.currentTimeMillis();
                System.out.println("Saving Organization Ambiguous vectors in path:\n" + FP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH);
                // organization
                // go to end of orgAmbVectorsStream, so append new records at this point of file
                orgAmbVectorsStream.seek(orgAmbVectorsStream.length());
                orgAmbVectorsStream.writeBytes(orgAmbVectors);
                orgAmbVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                savT = System.currentTimeMillis();
                System.out.println("Saving vectors information in  path:\n" + FP_VECTORS_INFO_FILE_LOADED_PATH);
                vectorsInfoFPStream = new RandomAccessFile(FP_VECTORS_INFO_FILE_LOADED_PATH, "rw");
                vectorsInfoFPStream.seek(vectorsInfoFPStream.length());
                vectorsInfoFPStream.writeBytes(vectorsInfoFP);
                vectorsInfoFPStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
            } // if
            else {
                
                
                long savT = System.currentTimeMillis();
                usedVectorsStream = new RandomAccessFile(SP_USED_VECTORS_FILE_LOADED_PATH, "rw");
                System.out.println("Saving used vectors in path:\n" + SP_USED_VECTORS_FILE_LOADED_PATH);
                // go to end of used vectors stream, so append new records at this point of file
                usedVectorsStream.seek(usedVectorsStream.length());
                usedVectorsStream.writeBytes(usedVectors);
                usedVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                new File(SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH).getParentFile().mkdirs();
                perAmbVectorsStream = new RandomAccessFile(SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH, "rw");
                savT = System.currentTimeMillis();
                System.out.println("Saving Person Ambiguous vectors in path:\n" + SP_PER_AMBIGIOUS_VECTORS_FILE_LOADED_PATH);
                // person
                // go to end of perAmbVectorsStream, so append new records at this point of file
                perAmbVectorsStream.seek(perAmbVectorsStream.length());
                perAmbVectorsStream.writeBytes(perAmbVectors);
                perAmbVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                orgAmbVectorsStream = new RandomAccessFile(SP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH, "rw");
                savT = System.currentTimeMillis();
                System.out.println("Saving Organization Ambiguous vectors in path:\n" + SP_ORG_AMBIGIOUS_VECTORS_FILE_LOADED_PATH);
                // organization
                // go to end of orgAmbVectorsStream, so append new records at this point of file
                orgAmbVectorsStream.seek(orgAmbVectorsStream.length());
                orgAmbVectorsStream.writeBytes(orgAmbVectors);
                orgAmbVectorsStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                savT = System.currentTimeMillis();
                System.out.println("Saving vectors information in  path:\n" + SP_VECTORS_INFO_FILE_LOADED_PATH);
                vectorsInfoSPStream = new RandomAccessFile(SP_VECTORS_INFO_FILE_LOADED_PATH, "rw");
                vectorsInfoSPStream.seek(vectorsInfoSPStream.length());
                vectorsInfoSPStream.writeBytes(vectorsInfoSP);
                vectorsInfoSPStream.close();
                System.out.println(". . . Saved, Time needed: " + Constants.toTimeString(savT));
                
                
            } // else
            
            System.out.println("\n" + countPerAns + " ambiguous vectors classified as persons, " + countOrgAns +
                    " classified as organizations and " + (batchSize - countPerAns - countOrgAns) + " as none.");
            this.resultsOutput.print("\n" + countPerAns + " ambiguous vectors classified as persons, " + countOrgAns +
                    " classified as organizations and " + (batchSize - countPerAns - countOrgAns) + " as none.\n");
            
            countPerAns = countOrgAns = 0;
            
        } catch (FileNotFoundException ex) {
            
            ex.printStackTrace();
            
        } catch (IOException ex) {
            
            ex.printStackTrace();
            
        } // if
        
        ///// saving ambiguous Vectors and batch information ////
        // =================================================================
        
        ////////////////////////////////////////////////////////////////////////
        ////////////////////////// saving lists ////////////////////////////////
        
        // list names for Constants.ORGANIZATION
        String list0 = ",minWordLen=5,maxWordLen=20,numPrevWords=7.txt";
        String list1 = ",minWordLen=5,maxWordLen=20,numPrevWords=1.txt";
        String list2 = ",minWordLen=3,maxWordLen=4,numPrevWords=7.txt";
        String list3 = ",minWordLen=3,maxWordLen=4,numPrevWords=1.txt";
        String list4 = ",minWordLen=1,maxWordLen=2,numPrevWords=7.txt";
        String list5 = ",minWordLen=1,maxWordLen=2,numPrevWords=1.txt";
        
        // list names for Constants.ORGANIZATION
        String list6  = ",minWordLen=5,maxWordLen=20,numPrevWords=7.txt";
        String list7  = ",minWordLen=5,maxWordLen=20,numPrevWords=1.txt";
        String list8  = ",minWordLen=3,maxWordLen=4,numPrevWords=7.txt";
        String list9  = ",minWordLen=3,maxWordLen=4,numPrevWords=1.txt";
        String list10 = ",minWordLen=1,maxWordLen=2,numPrevWords=7.txt";
        String list11 = ",minWordLen=1,maxWordLen=2,numPrevWords=1.txt";
        
        String personFullListPrefix, personListPrefix,organizationFullListPrefix, organizationListPrefix;
        if (currPass.equals(Constants.FIRST_PASS)) {
            
            personFullListPrefix =
                    Constants.FULL_LISTS_DIRECTORY_PATH + "/" +
                    Constants.FIRST_PASS + "/" +
                    Constants.PERSON + "/lessOcc=0";
            personListPrefix =
                    LISTS_DIRECTORY_LOADED_PATH + "/"
                    + Constants.FIRST_PASS + "/" +
                    Constants.PERSON + "/lessOcc=";
            
            organizationFullListPrefix =
                    Constants.FULL_LISTS_DIRECTORY_PATH +
                    Constants.FIRST_PASS + "/" +
                    Constants.ORGANIZATION + "/lessOcc=0";
            organizationListPrefix =
                    LISTS_DIRECTORY_LOADED_PATH + "/" +
                    Constants.FIRST_PASS + "/" +
                    Constants.ORGANIZATION + "/lessOcc=";
            
        } // if
        else {
            
            personFullListPrefix =
                    Constants.FULL_LISTS_DIRECTORY_PATH + "/" +
                    Constants.SECOND_PASS + "/" +
                    Constants.PERSON + "/lessOcc=0";
            personListPrefix =
                    LISTS_DIRECTORY_LOADED_PATH + "/" +
                    Constants.SECOND_PASS + "/" +
                    Constants.PERSON + "/lessOcc=";
            
            organizationFullListPrefix =
                    Constants.FULL_LISTS_DIRECTORY_PATH + "/" +
                    Constants.SECOND_PASS + "/" +
                    Constants.ORGANIZATION + "/lessOcc=0";
            organizationListPrefix =
                    LISTS_DIRECTORY_LOADED_PATH + "/" +
                    Constants.SECOND_PASS + "/" +
                    Constants.ORGANIZATION + "/lessOcc=";
            
        } // else
        
        if(saveListsCh.isSelected()){
            
            // full lists
            saveList(lists[0], 0 , personFullListPrefix + list0);
            saveList(lists[1], 0 , personFullListPrefix + list1);
            saveList(lists[2], 0 , personFullListPrefix + list2);
            saveList(lists[3], 0 , personFullListPrefix + list3);
            saveList(lists[4], 0 , personFullListPrefix + list4);
            saveList(lists[5], 0 , personFullListPrefix + list5);
            saveList(lists[6], 0 , organizationFullListPrefix + list6);
            saveList(lists[7], 0 , organizationFullListPrefix + list7);
            saveList(lists[8], 0 , organizationFullListPrefix + list8);
            saveList(lists[9], 0 , organizationFullListPrefix + list9);
            saveList(lists[10],0 , organizationFullListPrefix + list10);
            saveList(lists[11],0 , organizationFullListPrefix + list11);
            
            // lists
            saveList(lists[0], 10, personListPrefix + 10 + list0);
            saveList(lists[1], 3 , personListPrefix + 3 + list1);
            saveList(lists[2], 10, personListPrefix + 10 + list2);
            saveList(lists[3], 3 , personListPrefix + 3 + list3);
            saveList(lists[4], 10, personListPrefix + 10 + list4);
            saveList(lists[5], 3 , personListPrefix + 3 + list5);
            saveList(lists[6], 10, organizationListPrefix + 10 + list6);
            saveList(lists[7], 3 , organizationListPrefix + 3 + list7);
            saveList(lists[8], 10, organizationListPrefix + 10 + list8);
            saveList(lists[9], 3 , organizationListPrefix + 3 + list9);
            saveList(lists[10],10, organizationListPrefix + 10 + list10);
            saveList(lists[11],3 , organizationListPrefix + 3 + list11);
            
            System.out.println("Lists updated");
            
        }else{
            System.out.println("Lists not updated");
        }
        
        ////////////////////////// saving lists ////////////////////////////////
        //======================================================================
        
        
        output.println("Files saved, time needed " + Constants.toTimeString(start) );
        
    } // saveFiles
    
    /**
     * Saves a list
     *
     * @param lessOcc       Less number of occurances of words of this list in text
     * @param pathTofile    Path to file to save list
     * @param list          The list to be saved
     */
    private void saveList(Hashtable<String, ListNode> list, int lessOcc, String pathTofile) {
        
        Enumeration<ListNode> enumeration = list.elements();
        try {
            
            FileOutputStream fos = new FileOutputStream(pathTofile);
            while (enumeration.hasMoreElements()) {
                
                ListNode node = enumeration.nextElement();
                if (node.getCount() >= lessOcc) {
                    
                    fos.write((node.getKey() + " " + node.getCount() +
                            "\n").getBytes());
                    
                } // if
            } // while
            fos.close();
            
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        
        enumeration = null;
        list = null;
        
    } // saveList
    
    /**
     * Saves statistics about current batch in a file. Statistics saved are
     *
     *      - batch length
     *      - How many vectors included in batch are not to be used for training SVM_person because they violate "person safe rules"
     *      - How many vectors included in batch are not to be used for training SVM_organization because they violate "organization safe rules"
     *      - Informativeness Summuary of all vectors included in batch
     *      - How many vectors are chosen according to their "distance" from SVM_Person (
     *        The others included batch are chosen according to SVM_Organization distance)
     *
     * @param parent
     *      Parent directory of the statistics file
     * @param fileName
     *      Name of the file to save statistics.
     */
    private void saveStatistics(String parent, String fileName){
        
        // number of: 1.no person/organization safe rules applied,
        // 2. informativeness (1 - distance from hyperplanes) determened by
        // SVM for category person - we consider as distance the minimum
        // between distances from person and organization SVM
        double noPerSafe = 0.0, noOrgSafe = 0.0, informByPer = 0.0;
        double sumInfo = 0.0;// average informativeness
        for(int i = 0; i < batch.length; i++ ){
            
            if(!batch[i].isPersonSafeRulesApplied()){
                noPerSafe ++;
            }
            
            if(!batch[i].isOrganizationSafeRulesApplied()){
                noOrgSafe ++;
            }
            
            if(batch[i].isPerMinDistance()){
                informByPer ++;
            }
            
            sumInfo += batch[i].informativeness();
            
        }
        
        try {
            
            // creating parent directory
            (new File(parent)).mkdirs();
            
            FileOutputStream fout = new FileOutputStream(new File(parent, fileName));
            PrintStream printer = new PrintStream(fout);
            
            printer.print(batch.length + "|");
            printer.print(noPerSafe + "|");
            printer.print(noOrgSafe + "|" +
                    sumInfo + "|");
            printer.print(informByPer);
            printer.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        }
        
    }
    
    public void setVisible(boolean flag){
        
        super.setVisible(flag);
        
    }
    
    
    
    private class BatchLoaderListener implements ActionListener{
        
        public void actionPerformed(ActionEvent e) {
            
            if(Form.this.batch != null){
                
                int ans = JOptionPane.showConfirmDialog(null, "There is alreafy a batch computed. \n" +
                        "Do you wish to continue?", "Caution", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                if(ans != JOptionPane.YES_OPTION) return ;
                
            }
            
            JFileChooser jCh = new JFileChooser();
            jCh.setApproveButtonText("load");
            jCh.setCurrentDirectory(new File("../files"));
            jCh.setFileSelectionMode(jCh.FILES_ONLY);
            int ans = jCh.showDialog(Form.this, "load");
            if (ans != jCh.APPROVE_OPTION){
                return ;
            }
            
            ObjectInputStream ios;
            try {
                
                if (jCh.getSelectedFile().getName().endsWith(".obj")){
                    
                    ios = new ObjectInputStream(new FileInputStream(jCh.getSelectedFile()));
                    
                    try {
                        Form.this.batch = (AmbiguousVector[]) ios.readObject();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    } catch (ClassNotFoundException ex) {
                        ex.printStackTrace();
                    }
                    ios.close();
                    
                } else if (jCh.getSelectedFile().getName().endsWith(".txt")){
                    
                    
                    BufferedReader bRead =
                            new BufferedReader(
                            new FileReader(
                            jCh.getSelectedFile())
                            );
                    
                    ArrayList <AmbiguousVector> list = new ArrayList<AmbiguousVector>();
                    
                    String line;
                    while( ( line = bRead.readLine() ) != null){
                        
                        AmbiguousVector next = AmbiguousVector.retrieveFromToString(line);
                        list.add(next);
                        
                    }
                    
                    Form.this.batch = list.toArray(Form.this.batch);
                    Form.this.paramsForm.batchSizeTxt.setText(String.valueOf(Form.this.batch.length));
                    loadParams();
                    bRead.close();
                    
                } else{
                    
                    JOptionPane.showMessageDialog(null,"Not known file suffix", "Failed", JOptionPane.ERROR_MESSAGE);
                    return ;
                    
                }
                
                jCh = new JFileChooser();
                jCh.setCurrentDirectory(new File(Constants.RESULTS_DIRECTORY_PATH));
                jCh.setFileSelectionMode(jCh.FILES_ONLY);
                jCh.setDialogTitle("Choose file to save results");
                ans = jCh.showSaveDialog(Form.this);
                if (ans == jCh.CANCEL_OPTION){
                    return;
                }
                
                
                String fileName = (lastFile2KeepResults = jCh.getSelectedFile()).getName();
                
                try{
                    resultsOutput = new PrintStream((resultStream = new FileOutputStream(lastFile2KeepResults)));
                    fileName = fileName.substring(0, fileName.indexOf('.'));
                } catch(Exception exep){
                    
                    exep.printStackTrace();
                    
                }
                
                // saving statistics for batch
                saveStatistics(lastFile2KeepResults.getParent() + "/batch_stats", fileName + ".sta");
                
                if(batch.length == 0) {
                    
                    JOptionPane.showMessageDialog(
                            null, "Batch Empty - No more Vectors available in pool. \n " +
                            "System will exit","Pool is Empty", JOptionPane.INFORMATION_MESSAGE);
                    return;
                    
                }
                
                
                // fixing paths
                jCh = new JFileChooser();
                jCh.setCurrentDirectory(new File(ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH).getParentFile());
                jCh.setFileSelectionMode(jCh.DIRECTORIES_ONLY);
                
                jCh.setDialogTitle("Choose directory of active learnig files in use:");
                ans = jCh.showDialog(Form.this, "choose");
                if (ans != jCh.APPROVE_OPTION){
                    return;
                }
                
                File currActLeFiles = jCh.getSelectedFile();
                String actLeFilesName = currActLeFiles.getName();
                
                paramsForm.activeLearningFilesDirectoryTxt.setText(currActLeFiles.getCanonicalPath());
                
                paramsForm.FPusedVectorsFileNameTxt.setText(new String(
                        Constants.FP_USED_VECTORS_FILE_PATH.concat(actLeFilesName)));
                
                paramsForm.SPusedVectorsFileNameTxt.setText(new String(
                        Constants.SP_USED_VECTORS_FILE_PATH.concat(actLeFilesName)));
                
                loadParams();
                
                currActLeFiles = new File(ACTIVE_LEARNING_FILES_DIRECTORY_LOADED_PATH);
                
                // first pass
                if (currPass.equals(Constants.FIRST_PASS)) {
                    
                    actlearning = new ActiveLearningFP(
                            Constants.TIMEX_PATERNS_FILE_PATH,
                            SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
                            LISTS_DIRECTORY_LOADED_PATH, batchSize);
                    
                    resultsOutput.print("Batch loaded from file\n");
                    System.out.print("Batch loaded from file\n");
                    
                    // in case it was not possible to fill fully the batch
                    batchSize = batch.length;
                    
                    if(batch.length == 0) {
                        
                        JOptionPane.showMessageDialog(
                                null, "Batch Empty - No more Vectors available in pool. \n " +
                                "System will exit","Pool is Empty", JOptionPane.INFORMATION_MESSAGE);
                        return;
                        
                    }
                    
                    System.out.println("batch filled, batch size = " + batch.length);
                    wordToClassifyLab.setText("batch filled, batch size = " + batch.length);
                    
                } // if
                
                else if (currPass.equals(Constants.SECOND_PASS)){
                    
                    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                    /////////////////////////////////////////// loading from first pass ///////////////////////////////////////////////
                    
                    ////////////////////////////////////////////////////////////////
                    /////// reading lists of First pass decisions to be used ///////
                    ////////////// for computing Second Pass features //////////////
                    // currently used active learning files
                    
                    File dirOfFPlists =
                            new File(Constants.ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR ,
                            currActLeFiles.getName());
                    
                    // reading all decisions of FP classifiers for currently used subpool of active learning files
                    
                    System.out.println("Loading from " + Constants.ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR);
                    
                    File allDecisionsF = new File(dirOfFPlists, Constants.ACTIVE_LEARNING_FP_ALL_DECISION_LISTS_FILENAME);
                    FileInputStream in = new FileInputStream(allDecisionsF);
                    ObjectInputStream obIn;
                    try {
                        obIn = new ObjectInputStream(in);
                        allDec = (Hashtable<String, String>) obIn.readObject();
                        obIn.close();
                    } catch (ClassNotFoundException ex) {
                        ex.printStackTrace();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                    
                    // reading lists of sure classified tokens by FP classifiers, for currently used subpool of active learning files
                    File sureDecisionsF = new File(dirOfFPlists, Constants.ACTIVE_LEARNING_FP_SURE_DECISIONS_FILENAME);
                    in = new FileInputStream(sureDecisionsF);
                    try {
                        obIn = new ObjectInputStream(in);
                        sureFPDec = (Hashtable<String, String[]>) obIn.readObject();
                        obIn.close();
                    } catch (ClassNotFoundException ex) {
                        ex.printStackTrace();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                    
                    System.out.println(sureDecisionsF.getCanonicalPath() + " Loaded ");
                    System.out.println(allDecisionsF.getCanonicalPath() + " Loaded ");
                    ///================== FP LISTS LOADED ==================================
                    
                    /////////////////////////////////////////// loading from first pass ///////////////////////////////////////////////
                    //=================================================================================================================
                    
                    actlearning = new ActiveLearningSP(
                            allDec, sureFPDec, new File(FP_USED_VECTORS_FILE_LOADED_PATH),
                            Constants.TIMEX_PATERNS_FILE_PATH,
                            SENTENCE_SPLITTER_CLASSIFIER_FILE_LOADED_PATH,
                            LISTS_DIRECTORY_LOADED_PATH, batchSize);
                    
                    wordToClassifyLab.setText("Finding ambigious vectors...");
                    System.out.println("Finding ambigious vectors...");
                    
                    // the batch containing the ambiguous batch found by calling method
                    // "actlearning.findPerOrgAmbiguousVectors(...)"
                    
                    resultsOutput.print("Batch load from file:\n");
                    System.out.print("Batch load from file:\n");
                    
                    // in case it was not possible to fill fully the batch
                    batchSize = batch.length;
                    
                    System.out.println("batch filled, batch size = " + batch.length);
                    wordToClassifyLab.setText("batch filled, batch size = " + batch.length);
                    
                } else {
                    
                    try {
                        
                        throw new Exception("Wrong Pass, only first and second passes supported");
                        
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                    
                }
                
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            
            ////////////////////////////////////////////////////////////////////////
            ////////////////////////////opening streams/////////////////////////////
            long start = System.currentTimeMillis();
            // loading Lists
            Form.this.loadFullLists();
            System.out.println("Lists loaded. Time needed: " + Constants.toDigitalTimeString(start) );
            
            ////enabling buttons and menu items/////
            undoBut.setEnabled(true);
            
            // startBut.setEnabled(false);
            // findBatchMI.setEnabled(false);
            
            passM.setEnabled(false);
            fpasradTool.setEnabled(false);
            spasradTool.setEnabled(false);
            
            System.out.println(
                    "\nStart finished . . . ready to classify vectors in batch\n");
            
            perBut.setEnabled(true);
            orgBut.setEnabled(true);
            noneBut.setEnabled(true);
            
            perMI.setEnabled(true);
            orgMI.setEnabled(true);
            noneMI.setEnabled(true);
            
            ///////////////setting textfields for informing user////////////////////
            Form.this.showInformation(System.out, 0, false);
            Toolkit.getDefaultToolkit().beep();
            
            current_posInVectors = 0;
            
        }// actionPerfomred
        
    }
    
}//end of class ActiveLearningFrame
