/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.gui.main;

import ipl.nerc.Trainer;
import ipl.util.Constants;
import ipl.nerc.Tagger;
import ipl.util.file_manager.ArffFileManager;
import ipl.util.file_manager.FileManager;
import ipl.nerc.preprocessing.ResultFilesProcessor;
import java.applet.Applet;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 *
 * @author   
 */
public class Nerc extends javax.swing.JFrame {
    
    /**
     * Form to be shown for executing active learning experiments
     * (multiple category only)
     */
    private ipl.gui.active_learning.Form multActFrm;
    
    private ipl.gui.passive_learning.Form multpasFrm;
    
    /**
     * Used for single categories only - see action evets handling
     * for "go" buttons*/
    private Trainer trainerObj;
    
    private JOptionPane dialog;
    /** Creates new form NercApp */
    public Nerc()  {
        
        setMaximumSize(new Dimension(500, 500));
        
        dialog = new JOptionPane();
        initComponents();
        pasLeRadBut.doClick();
        
        trainerObj = new Trainer();
        
        multActFrm = new ipl.gui.active_learning.Form(this) ;
        multpasFrm = new ipl.gui.passive_learning.Form(this);
        Dimension frmDimens= (Toolkit.getDefaultToolkit()).getScreenSize();
        int x=(frmDimens.width - this.getWidth()) / 2;
        int y=(frmDimens.height - this.getHeight()) / 2;
        //setBounds(x, y, 400, 300);
        
    }
    
    
    /**
     * Checks if user really wants to terminate program
     */
    public  void doOnExiting(){
        
        ImageIcon questionMark = new ImageIcon(getClass().getResource(Constants.IMA_WARN_01));
        
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
        int ans = JOptionPane.showConfirmDialog(null,
                "Are you sure you want to exit?",
                "Exit?", JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE, questionMark);
        
        if (JOptionPane.YES_OPTION == ans){
            System.exit(0);
        }
        
        
    }
    /**
     * sets Trainer Object fror this main form
     */
    public Trainer getTrainerObj() {
        
        return trainerObj;
        
    }
    /**
     * Main method for runnig Nerc.
     *
     *
     * @param args the command line arguments(arguments are meaningless and thus
     * disregarded)
     */
    public static void main(String args[]) {
        
        
       /* try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        */
        new Nerc().setVisible(true);
        
    }
    
    /**
     * To be called when user wants to start a passive learning experiment
     */
    private void passiveLearning() {
        
        setEnabled(false);
        multpasFrm.setVisible(true);
        
    }
    
    /**
     * To be called when user wants to start an Active Learning experiment.
     */
    private void activeLearning(){
        
        setEnabled(false);
        multActFrm.setVisible(true);
        
    }
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        CategorybuttonGroup = new javax.swing.ButtonGroup();
        passActLearnGrp = new javax.swing.ButtonGroup();
        anotPassGrp = new javax.swing.ButtonGroup();
        genFinArtclesgrp = new javax.swing.ButtonGroup();
        norPnl = new javax.swing.JPanel();
        norNorPnl = new javax.swing.JPanel();
        catLab1 = new javax.swing.JLabel();
        pasLeRadBut = new javax.swing.JRadioButton();
        actLeRadBut = new javax.swing.JRadioButton();
        nextBut = new javax.swing.JButton();
        norCentPnl = new javax.swing.JPanel();
        catLab2 = new javax.swing.JLabel();
        genArtRad = new javax.swing.JRadioButton();
        finArtRad = new javax.swing.JRadioButton();
        defClaRad = new javax.swing.JRadioButton();
        annotateBut = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        inputMI = new javax.swing.JMenuItem();
        nextMI = new javax.swing.JMenuItem();
        exitMenuIt = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        toArff = new javax.swing.JMenuItem();
        annotMI = new javax.swing.JMenuItem();
        procRes = new javax.swing.JMenu();
        resultsMI = new javax.swing.JMenuItem();
        statsMI = new javax.swing.JMenuItem();
        matrixMI = new javax.swing.JMenuItem();
        svmMU = new javax.swing.JMenu();
        tuneSvmMI = new javax.swing.JMenuItem();
        c_gammaMI = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Greek Named Entity Recognition and Categorization");
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        norPnl.setLayout(new java.awt.BorderLayout());

        norNorPnl.setLayout(new java.awt.GridBagLayout());

        norNorPnl.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()));
        norNorPnl.setInheritsPopupMenu(true);
        catLab1.setFont(new java.awt.Font("Sylfaen", 1, 14));
        catLab1.setForeground(new java.awt.Color(153, 0, 0));
        catLab1.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        catLab1.setText("Machine Learning Selection");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 10, 2);
        norNorPnl.add(catLab1, gridBagConstraints);

        passActLearnGrp.add(pasLeRadBut);
        pasLeRadBut.setFont(new java.awt.Font("Sylfaen", 0, 12));
        pasLeRadBut.setMnemonic('p');
        pasLeRadBut.setSelected(true);
        pasLeRadBut.setText("Passive learning");
        pasLeRadBut.setToolTipText("select passive learning ang press \"next\" button to open passive learning form");
        pasLeRadBut.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        pasLeRadBut.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        norNorPnl.add(pasLeRadBut, gridBagConstraints);

        passActLearnGrp.add(actLeRadBut);
        actLeRadBut.setFont(new java.awt.Font("Sylfaen", 0, 12));
        actLeRadBut.setMnemonic('a');
        actLeRadBut.setText("Active learning");
        actLeRadBut.setToolTipText("select active learning ang press \"next\" button to open active learning form");
        actLeRadBut.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        actLeRadBut.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        norNorPnl.add(actLeRadBut, gridBagConstraints);

        nextBut.setFont(new java.awt.Font("Tahoma", 1, 12));
        nextBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/next_02.JPG")));
        nextBut.setMnemonic('n');
        nextBut.setText("Next");
        nextBut.setToolTipText("next");
        nextBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        nextBut.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        nextBut.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        nextBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 2.0;
        gridBagConstraints.insets = new java.awt.Insets(3, 4, 3, 4);
        norNorPnl.add(nextBut, gridBagConstraints);

        norPnl.add(norNorPnl, java.awt.BorderLayout.NORTH);

        norCentPnl.setLayout(new java.awt.GridBagLayout());

        norCentPnl.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()));
        catLab2.setFont(new java.awt.Font("Sylfaen", 1, 14));
        catLab2.setForeground(new java.awt.Color(153, 0, 0));
        catLab2.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        catLab2.setText("Annotate Files");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipady = 15;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(14, 0, 0, 0);
        norCentPnl.add(catLab2, gridBagConstraints);

        genFinArtclesgrp.add(genArtRad);
        genArtRad.setSelected(true);
        genArtRad.setText("General Articles");
        genArtRad.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        genArtRad.setMargin(new java.awt.Insets(0, 0, 0, 0));
        genArtRad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                genArtRadActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        norCentPnl.add(genArtRad, gridBagConstraints);

        genFinArtclesgrp.add(finArtRad);
        finArtRad.setText("Financial Articles");
        finArtRad.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        finArtRad.setMargin(new java.awt.Insets(0, 0, 0, 0));
        finArtRad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                finArtRadActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        norCentPnl.add(finArtRad, gridBagConstraints);

        genFinArtclesgrp.add(defClaRad);
        defClaRad.setText("Default Classifiers");
        defClaRad.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        defClaRad.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        norCentPnl.add(defClaRad, gridBagConstraints);

        annotateBut.setFont(new java.awt.Font("Tahoma", 1, 12));
        annotateBut.setMnemonic('g');
        annotateBut.setText("Tag Texts");
        annotateBut.setToolTipText("Tag Texts");
        annotateBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        annotateBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                annotateButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 2.0;
        gridBagConstraints.weighty = 0.5;
        norCentPnl.add(annotateBut, gridBagConstraints);

        norPnl.add(norCentPnl, java.awt.BorderLayout.CENTER);

        getContentPane().add(norPnl, java.awt.BorderLayout.CENTER);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        fileMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileMenuActionPerformed(evt);
            }
        });

        inputMI.setMnemonic('i');
        inputMI.setText("Input texts");
        inputMI.setToolTipText("Input text files to be annotated");
        inputMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                inputMIActionPerformed(evt);
            }
        });

        fileMenu.add(inputMI);

        nextMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        nextMI.setText("Next");
        nextMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextMIActionPerformed(evt);
            }
        });

        fileMenu.add(nextMI);

        exitMenuIt.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        exitMenuIt.setFont(new java.awt.Font("Garamond", 1, 14));
        exitMenuIt.setForeground(new java.awt.Color(255, 0, 0));
        exitMenuIt.setMnemonic('e');
        exitMenuIt.setText("Exit");
        exitMenuIt.setToolTipText("Termintes program");
        exitMenuIt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuIt);

        menuBar.add(fileMenu);

        editMenu.setMnemonic('d');
        editMenu.setText("Edit");
        editMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editMenuActionPerformed(evt);
            }
        });

        toArff.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.event.InputEvent.CTRL_MASK));
        toArff.setMnemonic('r');
        toArff.setText("Convert to .arff file");
        toArff.setToolTipText("Creates *.arrf files from files recognized by package libSVM, containing training vectors");
        toArff.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                toArffActionPerformed(evt);
            }
        });

        editMenu.add(toArff);

        annotMI.setMnemonic('a');
        annotMI.setText("Annotate");
        annotMI.setToolTipText("Manualy Annotate text files by using a convenient form");
        annotMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                annotMIActionPerformed(evt);
            }
        });

        editMenu.add(annotMI);

        procRes.setMnemonic('p');
        procRes.setText("Process result files");
        resultsMI.setMnemonic('s');
        resultsMI.setText("Results");
        resultsMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resultsMIActionPerformed(evt);
            }
        });

        procRes.add(resultsMI);

        statsMI.setMnemonic('t');
        statsMI.setText("Statistics");
        statsMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                statsMIActionPerformed(evt);
            }
        });

        procRes.add(statsMI);

        matrixMI.setMnemonic('x');
        matrixMI.setText("Matrix");
        matrixMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                matrixMIActionPerformed(evt);
            }
        });

        procRes.add(matrixMI);

        editMenu.add(procRes);

        menuBar.add(editMenu);

        svmMU.setMnemonic('v');
        svmMU.setText("SVM");
        tuneSvmMI.setMnemonic('t');
        tuneSvmMI.setText("Tuning");
        tuneSvmMI.setToolTipText("<html>Shows form for tuning svm c and gamma parameters.<br> Results saved in files as writen in command line.");
        tuneSvmMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tuneSvmMIActionPerformed(evt);
            }
        });

        svmMU.add(tuneSvmMI);

        c_gammaMI.setMnemonic('s');
        c_gammaMI.setText("Set c, gamma");
        c_gammaMI.setToolTipText("Set c and gamma parameters for SVM's under training");
        c_gammaMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                c_gammaMIActionPerformed(evt);
            }
        });

        svmMU.add(c_gammaMI);

        menuBar.add(svmMU);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-301)/2, (screenSize.height-271)/2, 301, 271);
    }// </editor-fold>//GEN-END:initComponents
    
    private void finArtRadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_finArtRadActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_finArtRadActionPerformed
    
    private void genArtRadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_genArtRadActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_genArtRadActionPerformed
    
    private void annotateButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_annotateButActionPerformed
        
        JFileChooser jf = new JFileChooser(Constants.FILES_DIR_PATH);
        jf.setMultiSelectionEnabled(true);
        jf.setApproveButtonText("Input");
        jf.setFileSelectionMode(jf.FILES_AND_DIRECTORIES);
        jf.setDialogTitle("Choose Input Text Files...");
        int ans = jf.showDialog(this, "Input");
        
        if(ans != jf.CANCEL_OPTION){
            
            JOptionPane jp = new JOptionPane();
            
            
            
            int filesToUse = Tagger.USE_DEFAULT_FILES;
            if(!defClaRad.isSelected()){
                
                filesToUse = genArtRad.isSelected()? Tagger.USE_FILES_FOR_GENERAL:
                    Tagger.USE_FILES_FOR_FINANCIAL;
                
            }
            
            ///////////////////////////////////////////////////////////////////////
            //////// loading files selected and files in selected directories//////
            File[]  tmp = jf.getSelectedFiles();
            if(tmp.length == 0) return;
            ArrayList<File> allSelectedFiles = new ArrayList<File>();
            for(File ifile: tmp){
                allSelectedFiles.add(ifile);
            }
            
            for(int i = 0; i<allSelectedFiles.size(); i++){
                
                File nextF = allSelectedFiles.get(i);
                if(nextF.isDirectory()){
                    
                    allSelectedFiles.remove(i--);
                    File[] filesListed = nextF.listFiles();
                    for(File nextF2: filesListed){
                        
                        allSelectedFiles.add(nextF2);
                        
                        
                    }//for
                    
                }//if directory
                
            }//for
            
            File []filesArr = new File[allSelectedFiles.size()];
            allSelectedFiles.toArray(filesArr);
            
            ///////////-----------------------------------------------/////////////
            //////// loading files selected and files in selected directories//////
            
            new Thread(new Tagger(Constants.SECOND_PASS, filesArr, filesToUse)).start();
            
        }
        
        
    }//GEN-LAST:event_annotateButActionPerformed
    
    private void fileMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileMenuActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_fileMenuActionPerformed
    
    private void inputMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_inputMIActionPerformed
        
        JFileChooser jf = new JFileChooser(Constants.FILES_DIR_PATH);
        jf.setMultiSelectionEnabled(true);
        jf.setApproveButtonText("input");
        jf.setFileSelectionMode(jf.FILES_AND_DIRECTORIES);
        jf.setDialogTitle("Choose input files...");
        int ans = jf.showDialog(this, "input");
        
        if(ans != jf.CANCEL_OPTION){
            
            JOptionPane jp = new JOptionPane();
            
            String[] filesToUse = {Constants.GENERAL_ARTICLES_TYPE, Constants.FINANCIAL_ARTICLES_TYPE, "Default", "Cancel"};
            int ch2 = jp.showOptionDialog(
                    this, "Select files you wish to use.", "Chose files",
                    JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE,
                    null, filesToUse, filesToUse[1]);
            
            if (ch2 == 3){
                return ;
            }
            int filesToUseFlag  = Tagger.USE_DEFAULT_FILES;
            if(!filesToUse[ch2].equalsIgnoreCase("Default")){
                filesToUseFlag =
                        filesToUse[ch2].equalsIgnoreCase(Constants.GENERAL_ARTICLES_TYPE)? Tagger.USE_FILES_FOR_GENERAL:
                            Tagger.USE_FILES_FOR_FINANCIAL;
            }
            
            
            ///////////////////////////////////////////////////////////////////////
            //////// loading files selected and files in selected directories//////
            File[]  tmp = jf.getSelectedFiles();
            if(tmp.length == 0) return;
            ArrayList<File> allSelectedFiles = new ArrayList<File>();
            for(File ifile: tmp){
                allSelectedFiles.add(ifile);
            }
            
            for(int i = 0; i<allSelectedFiles.size(); i++){
                
                File nextF = allSelectedFiles.get(i);
                if(nextF.isDirectory()){
                    
                    allSelectedFiles.remove(i--);
                    File[] filesListed = nextF.listFiles();
                    for(File nextF2: filesListed){
                        
                        allSelectedFiles.add(nextF2);
                        
                        
                    }//for
                    
                }//if directory
                
            }//for
            
            File []filesArr = new File[allSelectedFiles.size()];
            allSelectedFiles.toArray(filesArr);
            
             new Thread(new Tagger(Constants.SECOND_PASS, filesArr, filesToUseFlag)).start();
            
        }
        
    }//GEN-LAST:event_inputMIActionPerformed
    
    private void c_gammaMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_c_gammaMIActionPerformed
        SetCGammaForm setForm = new SetCGammaForm();
        setForm.setVisible(true);
    }//GEN-LAST:event_c_gammaMIActionPerformed
    
    private void nextMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextMIActionPerformed
        nextBut.doClick();
    }//GEN-LAST:event_nextMIActionPerformed
    
    private void annotMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_annotMIActionPerformed
        new AnnotationForm().setVisible(true);
    }//GEN-LAST:event_annotMIActionPerformed
    
    private void matrixMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_matrixMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH);
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of result files");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir="";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            // choose category
            String []catoptions = new String[2];
            catoptions[0] = new String(Constants.PERSON);
            catoptions[1] = new String(Constants.ORGANIZATION);
            int userChoseCat =
                    dialog.showOptionDialog(
                    null, "Choose category of results to process",
                    "Category processing selection", dialog.OK_CANCEL_OPTION ,
                    dialog.QUESTION_MESSAGE, null, catoptions, catoptions[0]);
            
            
            if (userChoseCat == dialog.CLOSED_OPTION){
                return ;
            }
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractMatrix(dir, catoptions[userChoseCat], dir/*Save*/);
            
        }catch(Exception e){
            e.printStackTrace();
        }
        
    }//GEN-LAST:event_matrixMIActionPerformed
    
    private void editMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editMenuActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_editMenuActionPerformed
    
    private void statsMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_statsMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH + "/stats");
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of stats");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir = "";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractStats(dir, dir/*Save*/);
            
        }catch(Exception e){e.printStackTrace();}
        
        
    }//GEN-LAST:event_statsMIActionPerformed
    
    private void resultsMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resultsMIActionPerformed
        
        JFileChooser ch = new JFileChooser();
        File initialDir = new File(Constants.RESULTS_DIRECTORY_PATH);
        if (!initialDir.exists()) initialDir = initialDir.getParentFile();
        ch.setCurrentDirectory(initialDir);
        
        ch.setDialogTitle("Chose directory of result files");
        ch.setApproveButtonText("Choose");
        ch.setFileSelectionMode(ch.DIRECTORIES_ONLY);
        ch.setApproveButtonMnemonic('c');
        
        int ans = ch.showOpenDialog(null);
        String dir="";
        if (ans == ch.APPROVE_OPTION){
            
            dir = ch.getSelectedFile().toString();
            
        }else return ;
        
        ////////////////////////////////////////////////////////////////////////
        /////////////////// processing each result file ////////////////////////
        
        try{
            
            // choose category
            String []catoptions = new String[2];
            catoptions[0] = new String(Constants.PERSON);
            catoptions[1] = new String(Constants.ORGANIZATION);
            int userChoseCat =
                    dialog.showOptionDialog(
                    null, "Choose category of results to process",
                    "Category processing selection", dialog.OK_CANCEL_OPTION ,
                    dialog.QUESTION_MESSAGE, null, catoptions, catoptions[0]);
            
            
            if (userChoseCat == dialog.CLOSED_OPTION){
                return ;
            }
            
            ResultFilesProcessor rPros = new ResultFilesProcessor();
            rPros.extractResults(dir, catoptions[userChoseCat], dir/*Save*/);
            
        }catch(Exception e){
            e.printStackTrace();
        }
        
    }//GEN-LAST:event_resultsMIActionPerformed
    
    private void tuneSvmMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tuneSvmMIActionPerformed
        
        (new TuningForm()).setVisible(true);
        
    }//GEN-LAST:event_tuneSvmMIActionPerformed
    
    private void toArffActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toArffActionPerformed
        
        (new File(Constants.ARRF_DIRECTORY_PATH)).mkdirs();
        
        
        
        String []categs = new String[]{Constants.PERSON, Constants.ORGANIZATION, "Cancel"};
        String []passes = new String[]{Constants.FIRST_PASS, Constants.SECOND_PASS, "Cancel"};
        
        int cat = JOptionPane.showOptionDialog(this, "Chose category:",
                "Category", JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE, null, categs, categs[0]);
        if (cat == 2) return ;//cancel
        
        int pass = JOptionPane.showOptionDialog(this, "Choose pass:",
                "Pass", JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE, null, passes, passes[0]);
        if (pass == 2) return ;//cancel
        
        
        File toArff;
        
        if(passes[pass].equals(Constants.FIRST_PASS)){
            
            if(categs[cat].equals(Constants.PERSON)){
                toArff = new File(Constants.FP_PERSON_TRAIN_VECTORS_FILE);
            }else{// organization
                toArff = new File(Constants.FP_ORGANIZATION_TRAIN_VECTORS_FILE);
            }
            
            
        } else{// second pass
            
            if(categs[cat].equals(Constants.PERSON)){
                toArff = new File(Constants.SP_PERSON_TRAIN_VECTORS_FILE);
            }else{// organization
                toArff = new File(Constants.SP_ORGANIZATION_TRAIN_VECTORS_FILE);
            }
            
            
        }
        
        String fname = toArff.getName();
        
        System.out.println("Converting training vectors from file \"" + toArff.getName() + "\"\n" +
                "to arrf file");
        System.out.println("");
        System.out.println("");
        try {
            
            ArffFileManager.convertToArff(
                    Constants.ARRF_DIRECTORY_PATH + "/" + fname + ".arff", toArff.getCanonicalPath(), passes[pass], categs[cat]);
            Applet.newAudioClip(getClass().getResource(Constants.SOUND_PROCESS_END_FILE)).play();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        
        
    }//GEN-LAST:event_toArffActionPerformed
    
    private void nextButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_NEXT_FILE)).play();
        
        
        if(actLeRadBut.isSelected()){
            activeLearning();
        } else{
            passiveLearning();
        }
        
        
        
        
    }//GEN-LAST:event_nextButActionPerformed
    
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        
        exitMenuIt.doClick();
        
    }//GEN-LAST:event_formWindowClosing
    
    private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
        // TODO add your handling code here:
    }//GEN-LAST:event_formWindowClosed
    
    private void exitMenuItActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItActionPerformed
        doOnExiting();
    }//GEN-LAST:event_exitMenuItActionPerformed
    
    /**
     * Performs the action event for button "goPasBut"
     * Simply calls startExperiments(arguments) ,where arguments a string table
     * containing information about the myCl.getCategory() of the experiment.
     */
    /**
     * Performs the action event for button "goActBut" .
     * Simply sets Visible actFrm(an ActLeForm object with it's menubar not visible)
     */
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup CategorybuttonGroup;
    private javax.swing.JRadioButton actLeRadBut;
    private javax.swing.JMenuItem annotMI;
    private javax.swing.JButton annotateBut;
    private javax.swing.ButtonGroup anotPassGrp;
    private javax.swing.JMenuItem c_gammaMI;
    private javax.swing.JLabel catLab1;
    private javax.swing.JLabel catLab2;
    private javax.swing.JRadioButton defClaRad;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenuItem exitMenuIt;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JRadioButton finArtRad;
    private javax.swing.JRadioButton genArtRad;
    private javax.swing.ButtonGroup genFinArtclesgrp;
    private javax.swing.JMenuItem inputMI;
    private javax.swing.JMenuItem matrixMI;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JButton nextBut;
    private javax.swing.JMenuItem nextMI;
    private javax.swing.JPanel norCentPnl;
    private javax.swing.JPanel norNorPnl;
    private javax.swing.JPanel norPnl;
    private javax.swing.JRadioButton pasLeRadBut;
    private javax.swing.ButtonGroup passActLearnGrp;
    private javax.swing.JMenu procRes;
    private javax.swing.JMenuItem resultsMI;
    private javax.swing.JMenuItem statsMI;
    private javax.swing.JMenu svmMU;
    private javax.swing.JMenuItem toArff;
    private javax.swing.JMenuItem tuneSvmMI;
    // End of variables declaration//GEN-END:variables
    
    
    
    
    
}
