/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.gui.main;


import ipl.util.NativeTuner;
import ipl.util.Constants;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import javax.swing.JApplet;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 *
 * @author  Xen.Vassilakos
 */
public class TuningForm extends javax.swing.JFrame implements Runnable {
    
    private double cfrom ;
    private double cto ;
    private double cstep ;
    private double gfrom ;
    private double gto ;
    private double gstep ;
    private int foldsValidation ;
    private Thread thread;
    
    /**
     * Creates new form TuningForm
     */
    public TuningForm() {
        
        double cfrom = 0.0;
        double cto = 5.0;
        double cstep = 1.0;
        double gfrom = -5.0;
        double gto = 5.0;
        double gstep = 1.0;
        int foldsValidation = 5;
        
        initComponents();
        Dimension scrD = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((int)(scrD.getWidth() - getWidth()) /2, (int)(scrD.getHeight() - getHeight())/2 , getWidth(), getHeight());
        
        cFromTxt.setText(String.valueOf(cfrom));
        cToTxt.setText(String.valueOf(cto));
        cStepTxt.setText(String.valueOf(cstep));
        
        gammaFromTxt.setText(String.valueOf(gfrom));
        gammaToTxt.setText(String.valueOf(gto));
        gammaStepTxt.setText(String.valueOf(gstep));
        foldValidationNumTxt.setText(String.valueOf(foldsValidation));
        
        thread = new Thread(this);
        
    }
    
    public void run(){
        
        NativeTuner tuner = new NativeTuner();
       
        JFileChooser jCh = new JFileChooser(Constants.SYSTEM_DIR_PATH);
        jCh.setDialogTitle("Chose training vectors file to be tuned");
        if(jCh.showDialog(null, "Choose") == jCh.APPROVE_OPTION){
            
            try {
                
                File toSave = new File("../files/tune", jCh.getSelectedFile().getName() + ".tun");
                toSave.getParentFile().mkdirs();
                System.out.println("Saving results in Directory:\n" + toSave.getParentFile().getCanonicalPath());
                PrintStream pOut = new PrintStream(new java.io.FileOutputStream(toSave));
                long start = System.currentTimeMillis();
                
                double []results =
                        tuner.tune(jCh.getSelectedFile().getCanonicalPath(),
                        Double.parseDouble(cFromTxt.getText()), Double.parseDouble(cToTxt.getText()), Double.parseDouble(cStepTxt.getText()),
                        Double.parseDouble(gammaFromTxt.getText()), Double.parseDouble(gammaToTxt.getText()), Double.parseDouble(gammaStepTxt.getText()),
                        Integer.parseInt(foldValidationNumTxt.getText()), pOut);
                
                System.out.println(jCh.getSelectedFile().getName() + " Tuned. \n" +
                        "Time needed: " + Constants.toDigitalTimeString(start) + "\n" +
                        "Results saved in file:\n" + toSave.getCanonicalPath());
                
                pOut.println("\n" + "Time needed: " + Constants.toDigitalTimeString(start) + "\n");
                
                pOut.close();
                
                JOptionPane.showMessageDialog(this,
                        "C: " + results[0] + "\n" +
                        "Gamma: " + results[1] + "\n" +
                        "Accuracy: " + results[2]);
                
                
            } catch (IOException ex) {
                ex.printStackTrace();
                System.err.println("Tuning error exiting ...");
                System.exit(-1);
            }
            
        }
        
        Toolkit.getDefaultToolkit().beep();
        System.gc();
        
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel11 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jPanel6 = new javax.swing.JPanel();
        cFromTxt = new javax.swing.JTextField();
        cToTxt = new javax.swing.JTextField();
        cStepTxt = new javax.swing.JTextField();
        jPanel7 = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        jLabel11 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jPanel9 = new javax.swing.JPanel();
        gammaFromTxt = new javax.swing.JTextField();
        gammaToTxt = new javax.swing.JTextField();
        gammaStepTxt = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        foldValidationNumTxt = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        jPanel10 = new javax.swing.JPanel();
        cancelBut = new javax.swing.JButton();
        okBut = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JSeparator();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("SVM Tuning..");
        setAlwaysOnTop(true);
        jPanel11.setLayout(new java.awt.GridBagLayout());

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 2), "Parameter C:"), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        jPanel5.setLayout(new java.awt.GridLayout(3, 1, 0, 10));

        jLabel1.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel1.setText("From power of 2 in:");
        jPanel5.add(jLabel1);

        jLabel5.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel5.setText("To power of 2 in:");
        jPanel5.add(jLabel5);

        jLabel3.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel3.setText("Step:");
        jPanel5.add(jLabel3);

        jPanel2.add(jPanel5, java.awt.BorderLayout.CENTER);

        jPanel6.setLayout(new java.awt.GridLayout(3, 0, 0, 10));

        jPanel6.setInheritsPopupMenu(true);
        cFromTxt.setColumns(4);
        cFromTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        cFromTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        cFromTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        cFromTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cFromTxtFocusLost(evt);
            }
        });

        jPanel6.add(cFromTxt);

        cToTxt.setColumns(4);
        cToTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        cToTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        cToTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        cToTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cToTxtFocusLost(evt);
            }
        });

        jPanel6.add(cToTxt);

        cStepTxt.setColumns(4);
        cStepTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        cStepTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        cStepTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        cStepTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cStepTxtFocusLost(evt);
            }
        });

        jPanel6.add(cStepTxt);

        jPanel2.add(jPanel6, java.awt.BorderLayout.EAST);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.ipadx = 63;
        gridBagConstraints.ipady = 2;
        gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
        jPanel11.add(jPanel2, gridBagConstraints);

        jPanel7.setLayout(new java.awt.BorderLayout());

        jPanel7.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 2), "Parameter \u03b3:"), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        jPanel8.setLayout(new java.awt.GridLayout(3, 1, 0, 10));

        jLabel11.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel11.setText("From power of 2 in:");
        jPanel8.add(jLabel11);

        jLabel10.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel10.setText("To power of 2 in:");
        jPanel8.add(jLabel10);

        jLabel12.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel12.setText("Step:");
        jPanel8.add(jLabel12);

        jPanel7.add(jPanel8, java.awt.BorderLayout.CENTER);

        jPanel9.setLayout(new java.awt.GridLayout(3, 0, 0, 10));

        jPanel9.setInheritsPopupMenu(true);
        gammaFromTxt.setColumns(4);
        gammaFromTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        gammaFromTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        gammaFromTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        gammaFromTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                gammaFromTxtFocusLost(evt);
            }
        });

        jPanel9.add(gammaFromTxt);

        gammaToTxt.setColumns(4);
        gammaToTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        gammaToTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        gammaToTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        gammaToTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                gammaToTxtFocusLost(evt);
            }
        });

        jPanel9.add(gammaToTxt);

        gammaStepTxt.setColumns(4);
        gammaStepTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        gammaStepTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        gammaStepTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        gammaStepTxt.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                gammaStepTxtFocusLost(evt);
            }
        });

        jPanel9.add(gammaStepTxt);

        jPanel7.add(jPanel9, java.awt.BorderLayout.EAST);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 63;
        gridBagConstraints.ipady = 2;
        gridBagConstraints.insets = new java.awt.Insets(8, 8, 8, 8);
        jPanel11.add(jPanel7, gridBagConstraints);

        jPanel4.setLayout(new java.awt.GridBagLayout());

        jPanel3.setLayout(new java.awt.GridBagLayout());

        foldValidationNumTxt.setColumns(4);
        foldValidationNumTxt.setFont(new java.awt.Font("Sylfaen", 1, 11));
        foldValidationNumTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        foldValidationNumTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 16;
        gridBagConstraints.weightx = 0.1;
        gridBagConstraints.weighty = 0.1;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 5);
        jPanel3.add(foldValidationNumTxt, gridBagConstraints);

        jLabel9.setFont(new java.awt.Font("Sylfaen", 0, 11));
        jLabel9.setText("Fold validation number:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.ipadx = 20;
        gridBagConstraints.weightx = 0.1;
        gridBagConstraints.weighty = 0.1;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 0);
        jPanel3.add(jLabel9, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel4.add(jPanel3, gridBagConstraints);

        jPanel10.setLayout(new java.awt.GridBagLayout());

        cancelBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/wrong.gif")));
        cancelBut.setToolTipText("Cancel");
        cancelBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.weightx = 0.8;
        gridBagConstraints.weighty = 0.8;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel10.add(cancelBut, gridBagConstraints);

        okBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/ok_02.gif")));
        okBut.setToolTipText("OK");
        okBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHEAST;
        gridBagConstraints.weightx = 0.8;
        gridBagConstraints.weighty = 0.8;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel10.add(okBut, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel4.add(jPanel10, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jPanel4.add(jSeparator2, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel11.add(jPanel4, gridBagConstraints);

        getContentPane().add(jPanel11, java.awt.BorderLayout.CENTER);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-417)/2, (screenSize.height-264)/2, 417, 264);
    }// </editor-fold>//GEN-END:initComponents
    
    private void cancelButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_DISCONNECT_FILE)).play();
        
        
        dispose();
        
    }//GEN-LAST:event_cancelButActionPerformed
    
    private void okButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_CLICK_01_FILE)).play();
        thread.start();
        
        dispose();
        
    }//GEN-LAST:event_okButActionPerformed
    
    private void gammaStepTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_gammaStepTxtFocusLost
        
        try {
            
            Double.parseDouble(gammaStepTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter gamma_step", "Searching step for gamma is wrong.", JOptionPane.WARNING_MESSAGE);
            gammaStepTxt.setText(String.valueOf(gstep));
            
        }
        
    }//GEN-LAST:event_gammaStepTxtFocusLost
    
    private void cStepTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cStepTxtFocusLost
        
        try {
            
            Double.parseDouble(cStepTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter c_step", "Searching step for c is wrong.", JOptionPane.WARNING_MESSAGE);
            cStepTxt.setText(String.valueOf(cstep));
            
        }
        
    }//GEN-LAST:event_cStepTxtFocusLost
    
    private void gammaFromTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_gammaFromTxtFocusLost
        
        try {
            
            Double.parseDouble(gammaFromTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter gamma_from", "Exponent of starting gamma parameter typed wrong.", JOptionPane.WARNING_MESSAGE);
            gammaFromTxt.setText(String.valueOf(gfrom));
            
        }
        
    }//GEN-LAST:event_gammaFromTxtFocusLost
    
    private void gammaToTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_gammaToTxtFocusLost
        
        try {
            
            Double.parseDouble(gammaToTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter gamma_to", "Exponent of ending gamma parameter typed wrong.", JOptionPane.WARNING_MESSAGE);
            gammaToTxt.setText(String.valueOf(gto));
            
        }
        
    }//GEN-LAST:event_gammaToTxtFocusLost
    
    private void cToTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cToTxtFocusLost
        
        try {
            
            Double.parseDouble(cToTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter c_to", "Exponent of ending c parameter typed wrong.", JOptionPane.WARNING_MESSAGE);
            cToTxt.setText(String.valueOf(cto));
            
        }
        
    }//GEN-LAST:event_cToTxtFocusLost
    
    private void cFromTxtFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cFromTxtFocusLost
        
        try {
            
            Double.parseDouble(cFromTxt.getText());
            
        } catch (NumberFormatException ex) {
            
            JOptionPane.showMessageDialog(this, "Wrong parameter c_from", "Exponent for starting c parameter typed wrong.", JOptionPane.WARNING_MESSAGE);
            cFromTxt.setText(String.valueOf(cfrom));
            
        }
        
    }//GEN-LAST:event_cFromTxtFocusLost
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new TuningForm().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField cFromTxt;
    private javax.swing.JTextField cStepTxt;
    private javax.swing.JTextField cToTxt;
    private javax.swing.JButton cancelBut;
    private javax.swing.JTextField foldValidationNumTxt;
    private javax.swing.JTextField gammaFromTxt;
    private javax.swing.JTextField gammaStepTxt;
    private javax.swing.JTextField gammaToTxt;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JButton okBut;
    // End of variables declaration//GEN-END:variables
    
}
