/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.gui.passive_learning;

import ipl.nerc.Trainer;
import ipl.util.Constants;
import ipl.gui.main.Nerc;
import ipl.nerc.Tester;
import ipl.splitter.SentenceSplitter;
import ipl.nerc.lists.ListManager;
import ipl.nerc.lists.ListNode;
import ipl.nerc.regex.timex.TimexMatcher;
import java.awt.Toolkit;
import java.io.*;
import java.net.MalformedURLException;
import java.util.*;
import javax.swing.*;

/**
 *
 * @author  Owner
 */
public class Form extends JFrame    {
    
    private long end;
    
    private long start;
    
    private int perTrainVecs, orgTrainVecs;
    
    /**JOption dialog box for messages*/
    private static JOptionPane dialog;
    
    private Trainer trainer;
    
    /** Manages lists for person first pass*/
    private ListManager perListManagerFP;
    
    /** Manages lists for org first pass*/
    private ListManager orgListManagerFP;
    
    /** Manages lists for person second pass*/
    private ListManager perListManagerSP;
    
    /** Manages lists for org second pass*/
    private ListManager orgListManagerSP;
    
    /** Redirected output steam for System.out*/
    private PrintStream sysDefOut;
    
    /** The last Stream where experimental results where saved*/
    private FileOutputStream resultStream ;
    private PrintStream prOutStr;
    
    /** The last File where results where saved*/
    private File lastFile2KeepResults;
    
    private String currentPass ;
    
    private String undoClearTxt = "";
    // paths loaded from parameters form
    private String
            
            /**lists directory path loaded from parameters form*/
            LISTS_DIRECTORY_LOADED_PATH,
            
            /**FP PERSON classifier model path loaded from parameters form*/
            FP_PER_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**FP ORGANIZATION classifier model path loaded from parameters form*/
            FP_ORG_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**SP ORGANIZATION classifier model path loaded from parameters form*/
            SP_PER_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**SP ORGANIZATION  classifier model path loaded from parameters form*/
            SP_ORG_CLASSIFIER_FILE_LOADED_PATH ,
            
            /**Period separator classifier model path loaded from parameters form*/
            PERIOD_SEPERATOR_CLASSIFIER_FILE_LOADED_PATH;
    
    /**First pass Training files added from pool*/
    private ArrayList<String> fp_trainFilesAdded;
    
    /**Second pass Training files added from pool*/
    private ArrayList<String> sp_trainFilesAdded;
    
    /**Frame for parameters*/
    private ParamsForm paramsForm;
    
    /**Array of Hash tables for words in lists.
     * Initialized by loadPersonLists(),loadOrganizationLists()*/
    private Hashtable<String, ListNode> lists[];
    
    /**The main gui form of NERC*/
    private Nerc mainFrm;
    
    /**Creates new form PassiveLearningForm */
    public Form(Nerc mainFrmIn) {
        
        initComponents();
        
       currentPass = Constants.FIRST_PASS;
        
        mainFrm = mainFrmIn;
        sysDefOut=System.out;
        fp_trainFilesAdded = new ArrayList<String>();
        sp_trainFilesAdded = new ArrayList<String>();
        
        trainer = new Trainer();
        
        loadParamPaths(paramsForm = new ParamsForm(this));
        perTrainVecs = orgTrainVecs = 0;
        
        
    }
    
    /**Loads paths form parameters form*/
    void loadParamPaths(ParamsForm parametersForm){
        
        //lists
        LISTS_DIRECTORY_LOADED_PATH =
                parametersForm.listsDirectoryTxt.getText();
        
        // separator
        PERIOD_SEPERATOR_CLASSIFIER_FILE_LOADED_PATH =
                parametersForm.periodSeparatorModelFileNameTxt.getText();
        
        // classifiers
        FP_ORG_CLASSIFIER_FILE_LOADED_PATH =
                parametersForm.FPOrgClassifierFileNameTxt.getText();
        
        SP_ORG_CLASSIFIER_FILE_LOADED_PATH =
                parametersForm.SPOrgClassifierFileNameTxt.getText();
        
        FP_PER_CLASSIFIER_FILE_LOADED_PATH =
                parametersForm.FPPerClassifierFileNameTxt.getText();
        
        SP_PER_CLASSIFIER_FILE_LOADED_PATH =
                parametersForm.SPPerClassifierFileNameTxt.getText();
        
    }
    
    /**
     * @return  The file that was moved from currently chosen pass pool to training set, for performing
     *          experiment.
     */
    private String getLastTrainingFileAdded(){
        
        if (currentPass.equalsIgnoreCase(Constants.FIRST_PASS))
            return getLastFPTrainingFileAdded();
        else //if (currentPass.equalsIgnoreCase(Constants.SECOND_PASS))
            return getLastSPTrainingFileAdded();
        
    }
    
    /**
     * @return  The file that was moved from first pass pool to training set, for performing
     *          experiment.
     */
    private String getLastFPTrainingFileAdded(){
        int offset = fp_trainFilesAdded.size() - 1;
        return fp_trainFilesAdded.get(offset) ;
    }
    
    /**
     * @return  The file that was moved from second pass pool to training set, for performing
     *          experiment.
     */
    private String getLastSPTrainingFileAdded(){
        int offset = sp_trainFilesAdded.size() - 1;
        return sp_trainFilesAdded.get(offset) ;
    }
    
    /**
     * Sets the last file to be moved from first pass pool to first pass training set,
     * for performing experiment - Useful for iterative experiments .
     *
     * @param trainingFile
     *      The file to be moved from pool to training set
     *
     */
    protected void setLastFp_trFileAdded(String trainingFile){
        
        if(fp_trainFilesAdded.contains(trainingFile))
            dialog.showMessageDialog(
                    null,
                    "There was an effort to add a training file in first pass training set\n" +
                    "that allready exists in training set. There are \n" +
                    "probably dublicant training files in\n" +
                    "pool and training set.","Warning!",dialog.WARNING_MESSAGE);
        
        else fp_trainFilesAdded.add(trainingFile);
        
    }
    
    /**
     * Sets the last file to be moved from first pass pool to first pass training set,
     * for performing experiment - Useful for iterative experiments .
     *
     * @param trainingFile
     *      The file to be moved from pool to training set
     *
     */
    protected void setLastSp_trFileAdded(String trainingFile){
        
        if(sp_trainFilesAdded.contains(trainingFile))
            dialog.showMessageDialog(
                    null,
                    "There was an effort to add a training file in second pass training set\n" +
                    "that allready exists in training set. There are \n" +
                    "probably dublicant training files in\n" +
                    "pool and training set.","Warning!", dialog.WARNING_MESSAGE);
        
        else sp_trainFilesAdded.add(trainingFile);
        
    }
    
    /**
     * Sets the last file to be moved from first pass pool to currently chosen pass training set,
     * for performing experiment - Useful for iterative experiments .
     *
     * @param trainingFile
     *      The file to be moved from pool to training set
     *
     */
    public void setLastTrainFileAdded(String trainingFile){
        
        if (currentPass.equalsIgnoreCase(Constants.FIRST_PASS)){
            setLastFp_trFileAdded(trainingFile);
        } else //if (currentPass.equalsIgnoreCase(Constants.SECOND_PASS)){
            setLastSp_trFileAdded(trainingFile);
        
        
        
    }
    
    /**
     * Asks user to specify  <B>files of training set</B>.
     * @return A File array containing the chosen training set (set of text files). Else, return null.
     */
    private File[] choseFilesSet(String message, String butLabel){
        
        String[]ch = new String[3];
        JFileChooser jch = new JFileChooser(message);
        jch.setApproveButtonText(butLabel);
        jch.setDialogTitle(message);
        jch.setMultiSelectionEnabled(true);
        jch.setFileSelectionMode(jch.FILES_ONLY);
        jch.setCurrentDirectory(new File(Constants.FILES_DIR_PATH));
        
        
        if(jch.showOpenDialog(this) != jch.APPROVE_OPTION) return null;
        
        File[] filesSel = jch.getSelectedFiles();
        
        return filesSel;
        
    }
    
      
    /**
     * Creates and returns paths for the output files - The files of the experiments results
     *
     * @returns The path to the output-result files
     */
    private File createResultFiles(){
        
        String flag = "per_org", category = Constants.PER_ORG;
        
        // date for naming directory of result files
        GregorianCalendar calendar = new GregorianCalendar();
        
        int month_int =
                1 + calendar.get(calendar.MONTH),
                day_int = calendar.get(calendar.DAY_OF_MONTH);
        
        String
                year = String.valueOf(calendar.get(calendar.YEAR)),
                month, day;
        
        if(month_int < 10)
            month = "0".concat(String.valueOf(month_int));
        else month = String.valueOf(month_int);
        
        if(day_int < 10) day = "0".concat(String.valueOf(day_int));
        else day = String.valueOf(day_int);
        
        String resultsDir = new String(Constants.RESULTS_DIRECTORY_PATH);
        File file2KeepResults = new File(resultsDir);
        String passFlag = currentPass;
        file2KeepResults = new File(
                file2KeepResults,
                "PL" + passFlag);
        
        file2KeepResults = new File(
                file2KeepResults,
                year + "_" + month+ "_" + day);
        
        file2KeepResults.mkdirs();
        
        String parentFile = "";
        // num of experiment files performed today (according to date-see above)
        String num = "01";
        
        try{
            
            parentFile = file2KeepResults.getParentFile().getCanonicalPath();
            //log for todays result files
            File log = new File(parentFile, "experiments.log");
            
            if (log.exists()){
                
                /////////////////////////reading from log///////////////////////
                FileReader logdat = new FileReader(log);
                // 
                char []cBuf = new char[(int)log.length()];
                logdat.read(cBuf);
                num=new String(cBuf);
                logdat.close();
                //====================reading from log==========================
                
                int numb = Integer.parseInt(num)+1;
                num = ( numb < 10 ) ? "0" + num.valueOf(numb) : num.valueOf(numb);
                
                FileWriter logWr=new FileWriter(log);
                logWr.write(num);
                logWr.flush();
                logWr.close();
                
            }
            
            else{// log does not exist
                
                FileWriter logWr=new FileWriter(log);
                logWr.write("01");
                logWr.flush();
                logWr.close();
                
            }
            
            
        } catch(IOException ex){
            
            ex.printStackTrace();
            
        }
        // FULL PATH TO FILE TO KEEP RESULTS
        file2KeepResults =
                new File(file2KeepResults, "expmnt_" + num + "." + flag);
        
        return file2KeepResults;
        
    }
    
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroup1 = new javax.swing.ButtonGroup();
        buttonGroup2 = new javax.swing.ButtonGroup();
        financialGeneralArticlesButGrp = new javax.swing.ButtonGroup();
        centerPnl = new javax.swing.JPanel();
        bottomPnl = new javax.swing.JPanel();
        backBut = new javax.swing.JButton();
        buttonsPnl = new javax.swing.JPanel();
        trainBut = new javax.swing.JButton();
        testBut = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        resultsTxt = new javax.swing.JTextPane();
        northPnl = new javax.swing.JPanel();
        northWestPnl = new javax.swing.JPanel();
        fpasradTool = new javax.swing.JRadioButton();
        spasradTool = new javax.swing.JRadioButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        backMI = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        optMenu = new javax.swing.JMenu();
        pasLeMenu = new javax.swing.JMenu();
        trainMenuItem = new javax.swing.JMenuItem();
        testMenuItem = new javax.swing.JMenuItem();
        paramMenu = new javax.swing.JMenu();
        passMenu = new javax.swing.JMenu();
        fpasradMenu = new javax.swing.JRadioButtonMenuItem();
        spasradMenu = new javax.swing.JRadioButtonMenuItem();
        changeParamMItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Passive Learning ");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
            public void windowDeactivated(java.awt.event.WindowEvent evt) {
                formWindowDeactivated(evt);
            }
        });

        centerPnl.setLayout(new java.awt.BorderLayout());

        centerPnl.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED), javax.swing.BorderFactory.createEtchedBorder()));
        centerPnl.setToolTipText("Back to initial NERC form");
        bottomPnl.setLayout(new java.awt.BorderLayout(0, 20));

        backBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/back_05.jpg")));
        backBut.setMnemonic('b');
        backBut.setText("Back");
        backBut.setToolTipText("main form");
        backBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0))), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        backBut.setMinimumSize(new java.awt.Dimension(10, 10));
        backBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backButActionPerformed(evt);
            }
        });

        bottomPnl.add(backBut, java.awt.BorderLayout.WEST);

        centerPnl.add(bottomPnl, java.awt.BorderLayout.SOUTH);

        buttonsPnl.setLayout(new java.awt.GridBagLayout());

        trainBut.setFont(new java.awt.Font("Sylfaen", 1, 13));
        trainBut.setForeground(new java.awt.Color(153, 0, 0));
        trainBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/train_icon.JPG")));
        trainBut.setMnemonic('r');
        trainBut.setText("Retrain");
        trainBut.setToolTipText("Select training files and Retrain SVM classifiers");
        trainBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(102, 0, 0), 2), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        trainBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        trainBut.setMinimumSize(new java.awt.Dimension(10, 10));
        trainBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        trainBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                trainButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(7, 6, 7, 7);
        buttonsPnl.add(trainBut, gridBagConstraints);

        testBut.setFont(new java.awt.Font("Sylfaen", 1, 13));
        testBut.setForeground(new java.awt.Color(0, 0, 102));
        testBut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/test_01.GIF")));
        testBut.setMnemonic('t');
        testBut.setText("Test");
        testBut.setToolTipText("Select Testing files and perform testing");
        testBut.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 102), 2), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        testBut.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        testBut.setMinimumSize(new java.awt.Dimension(10, 10));
        testBut.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        testBut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testButActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(7, 6, 7, 7);
        buttonsPnl.add(testBut, gridBagConstraints);

        centerPnl.add(buttonsPnl, java.awt.BorderLayout.WEST);

        resultsTxt.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createMatteBorder(2, 1, 2, 1, new java.awt.Color(0, 0, 0)), "Results"), javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED))));
        resultsTxt.setEditable(false);
        resultsTxt.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                resultsTxtMouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                resultsTxtMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                resultsTxtMouseReleased(evt);
            }
        });

        jScrollPane1.setViewportView(resultsTxt);

        centerPnl.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        getContentPane().add(centerPnl, java.awt.BorderLayout.CENTER);

        northPnl.setLayout(new java.awt.BorderLayout());

        northPnl.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED)));
        buttonGroup1.add(fpasradTool);
        fpasradTool.setSelected(true);
        fpasradTool.setText("First Pass");
        fpasradTool.setToolTipText("First Pass");
        fpasradTool.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fpasradToolActionPerformed(evt);
            }
        });

        northWestPnl.add(fpasradTool);

        buttonGroup1.add(spasradTool);
        spasradTool.setText("Second Pass");
        spasradTool.setToolTipText("Second Pass");
        spasradTool.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spasradToolActionPerformed(evt);
            }
        });

        northWestPnl.add(spasradTool);

        northPnl.add(northWestPnl, java.awt.BorderLayout.WEST);

        getContentPane().add(northPnl, java.awt.BorderLayout.NORTH);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        fileMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileMenuActionPerformed(evt);
            }
        });

        backMI.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, java.awt.event.InputEvent.CTRL_MASK));
        backMI.setMnemonic('b');
        backMI.setText("Back");
        backMI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backMIActionPerformed(evt);
            }
        });

        fileMenu.add(backMI);

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        exitMenuItem.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        exitMenuItem.setForeground(new java.awt.Color(204, 51, 0));
        exitMenuItem.setMnemonic('e');
        exitMenuItem.setText("Exit");
        exitMenuItem.setToolTipText("terminates Application");
        exitMenuItem.setBorderPainted(true);
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        optMenu.setMnemonic('p');
        optMenu.setText("Options");
        pasLeMenu.setText("Passive Learning");
        trainMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.CTRL_MASK));
        trainMenuItem.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        trainMenuItem.setText("Train");
        trainMenuItem.setToolTipText("Click to retrain classifier for first or second  pass (see Parameters|Choose pass)");
        trainMenuItem.setBorderPainted(true);
        trainMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                trainMenuItemActionPerformed(evt);
            }
        });

        pasLeMenu.add(trainMenuItem);

        testMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
        testMenuItem.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 11));
        testMenuItem.setText("Test Classifier");
        testMenuItem.setToolTipText("Click to retrain classifier for first or second  pass (see Parameters|Choose pass)");
        testMenuItem.setBorderPainted(true);
        testMenuItem.setEnabled(false);
        testMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testMenuItemActionPerformed(evt);
            }
        });

        pasLeMenu.add(testMenuItem);

        optMenu.add(pasLeMenu);

        paramMenu.setText("Parameters");
        passMenu.setText("Choose Pass");
        buttonGroup2.add(fpasradMenu);
        fpasradMenu.setSelected(true);
        fpasradMenu.setText("First pass");
        fpasradMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fpasradMenuActionPerformed(evt);
            }
        });

        passMenu.add(fpasradMenu);

        buttonGroup2.add(spasradMenu);
        spasradMenu.setText("Second Pass");
        spasradMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spasradMenuActionPerformed(evt);
            }
        });

        passMenu.add(spasradMenu);

        paramMenu.add(passMenu);

        changeParamMItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        changeParamMItem.setText("Change parameters");
        changeParamMItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeParamMItemActionPerformed(evt);
            }
        });

        paramMenu.add(changeParamMItem);

        optMenu.add(paramMenu);

        menuBar.add(optMenu);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-491)/2, (screenSize.height-353)/2, 491, 353);
    }// </editor-fold>//GEN-END:initComponents
    
    private void resultsTxtMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_resultsTxtMouseReleased
        
        if (evt.isPopupTrigger()){
            
            javax.swing.JPopupMenu popmen = new javax.swing.JPopupMenu("clear results");
            JMenuItem clearMI = new JMenuItem("clear text");
            JMenuItem undoClearMI = new JMenuItem("undo clear");
            clearMI.addActionListener(new java.awt.event.ActionListener() {
                
                public void actionPerformed(java.awt.event.ActionEvent acvt){
                    
                    undoClearTxt = resultsTxt.getText();
                    resultsTxt.setText("");
                    
                }
                
            });
            
            undoClearMI.addActionListener(new java.awt.event.ActionListener() {
                
                public void actionPerformed(java.awt.event.ActionEvent acvt){
                    
                    resultsTxt.setText(undoClearTxt);
                    
                }
                
            });
            
            popmen.add(clearMI);
            popmen.add(undoClearMI);
            popmen.show(resultsTxt, evt.getX(), evt.getY());
            
        }
        
    }//GEN-LAST:event_resultsTxtMouseReleased
    
    private void resultsTxtMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_resultsTxtMousePressed
        
        if (evt.isPopupTrigger()){
            
            javax.swing.JPopupMenu popmen = new javax.swing.JPopupMenu("clear results");
            JMenuItem clearMI = new JMenuItem("clear");
            clearMI.addActionListener(new java.awt.event.ActionListener() {
                
                public void actionPerformed(java.awt.event.ActionEvent acvt){
                    
                    resultsTxt.setText("");
                    
                }
                
            });
            
            popmen.add(clearMI);
            popmen.setVisible(true);
            
        }
        
    }//GEN-LAST:event_resultsTxtMousePressed
    
    private void resultsTxtMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_resultsTxtMouseClicked
        
    }//GEN-LAST:event_resultsTxtMouseClicked
    
    private void backMIActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backMIActionPerformed
        backBut.doClick();
    }//GEN-LAST:event_backMIActionPerformed
    
    private void spasradMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spasradMenuActionPerformed
        
        spasradTool.doClick();
        
    }//GEN-LAST:event_spasradMenuActionPerformed
    
    private void fileMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileMenuActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_fileMenuActionPerformed
    
    private void spasradToolActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spasradToolActionPerformed
        
        currentPass = Constants.SECOND_PASS;
        spasradMenu.setSelected(true);
        
    }//GEN-LAST:event_spasradToolActionPerformed
    
    private void fpasradToolActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpasradToolActionPerformed
        
        currentPass = Constants.FIRST_PASS;
        fpasradMenu.setSelected(true);
        
    }//GEN-LAST:event_fpasradToolActionPerformed
    
    private void fpasradMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fpasradMenuActionPerformed
        fpasradTool.doClick();
    }//GEN-LAST:event_fpasradMenuActionPerformed
    
    private void testMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_testMenuItemActionPerformed
        testBut.doClick();
    }//GEN-LAST:event_testMenuItemActionPerformed
    
    private void trainMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_trainMenuItemActionPerformed
        trainBut.doClick();
    }//GEN-LAST:event_trainMenuItemActionPerformed
    
    private void formWindowDeactivated(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowDeactivated
        
    }//GEN-LAST:event_formWindowDeactivated
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
            
            ImageIcon questionMark;
            questionMark = new ImageIcon(getClass().getResource(Constants.IMA_WARN_01));
            
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_ALERT_01_FILE)).play();
            int ans = JOptionPane.showConfirmDialog(null,
                    "Are you sure you want to exit?",
                    "Exit?", JOptionPane.YES_NO_OPTION,
                    JOptionPane.WARNING_MESSAGE, questionMark);
            if (JOptionPane.YES_OPTION == ans){
                System.exit(0);
            }
            
      
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    private void changeParamMItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeParamMItemActionPerformed
        
        paramsForm.setVisible(true);
        
    }//GEN-LAST:event_changeParamMItemActionPerformed
    
    private void backButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backButActionPerformed
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_BACK_FILE)).play();
        mainFrm.setVisible(true);
        mainFrm.setEnabled(true);
        sysDefOut = System.out;
        dispose();//killing this window
        
    }//GEN-LAST:event_backButActionPerformed
    
    private void testButActionPerformed(java.awt.event.ActionEvent evt) {
        
        trainBut.setEnabled(false);
        trainMenuItem.setEnabled(false);
        
        // asks for test Files
        File[] testSet;
        if((testSet = choseFilesSet("Chose set of text files to be used for testing..", "ok")) == null){
            return;
        }
        
        if(prOutStr == null){
            
            
            JFileChooser jCh = new JFileChooser();
            jCh.setCurrentDirectory(new File(Constants.RESULTS_DIRECTORY_PATH));
            jCh.setFileSelectionMode(jCh.FILES_ONLY);
            jCh.setDialogTitle("Choose file to save results");
            int ans = jCh.showSaveDialog(Form.this);
            if (ans == jCh.CANCEL_OPTION){
                return;
            }
            
            String fileName = (lastFile2KeepResults = jCh.getSelectedFile()).getName();
            try {
                resultStream = new FileOutputStream(lastFile2KeepResults);
                prOutStr = new PrintStream(resultStream);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            perTrainVecs = orgTrainVecs = -1;//unknown
            
            
        }
        
        try{
            
            TimexMatcher DRMatcher = new TimexMatcher(Constants.TIMEX_PATERNS_FILE_PATH);
            SentenceSplitter PSep = new SentenceSplitter(PERIOD_SEPERATOR_CLASSIFIER_FILE_LOADED_PATH);
            
            if(currentPass.equalsIgnoreCase(Constants.FIRST_PASS)){
                
                if(perListManagerFP == null){
                    System.out.println("Creating First Pass lists for category PERSON...");
                    start = System.currentTimeMillis();
                    perListManagerFP = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("Creating First Pass lists for category ORGANIZATION...");
                    orgListManagerFP = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("FIRST PASS LISTS DONE, " + Constants.toTimeString(start));
                    
                    start = System.currentTimeMillis();
                }
                
                Tester.testFP(DRMatcher, PSep,
                        testSet,
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        perListManagerFP, Constants.PERSON_ENTITIES_FILE_PATH,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        orgListManagerFP, Constants.ORGANIZATION_ENTITIES_FILE_PATH,
                        perTrainVecs, orgTrainVecs, prOutStr);
                
            } else{ //if(currentPass.equalsIgnoreCase(Constants.SECOND_PASS)){
                
                if(perListManagerSP == null){
                    
                    
                    System.out.println("Creating First Pass lists for category PERSON...");
                    start = System.currentTimeMillis();
                    perListManagerFP = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("Creating First Pass lists for category ORGANIZATION...");
                    orgListManagerFP = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("FIRST PASS LISTS DONE, " + Constants.toTimeString(start));
                    
                    
                    System.out.println("Creating Second Pass lists for category PERSON...");
                    start = System.currentTimeMillis();
                    perListManagerSP = ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("Creating Second Pass lists for category ORGANIZATION...");
                    orgListManagerSP = ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, LISTS_DIRECTORY_LOADED_PATH);
                    
                    System.out.println("SECOND PASS LISTS DONE, " + Constants.toTimeString(start));
                    
                    start = System.currentTimeMillis();
                    
                }
                
                Tester.testSP( DRMatcher, PSep,
                        testSet,
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        perListManagerFP,
                        SP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        perListManagerSP, Constants.PERSON_ENTITIES_FILE_PATH,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        orgListManagerFP,
                        SP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        orgListManagerSP,Constants.ORGANIZATION_ENTITIES_FILE_PATH,
                        perTrainVecs, orgTrainVecs, prOutStr);
                
            }
            
            System.setOut(sysDefOut);
            
            
            // closing The last Stream where results where saved
            prOutStr.close();
            
        }catch(Exception ex){ex.printStackTrace();}
        
        System.out.println("Testing finished");
        Toolkit.getDefaultToolkit().beep();
        // resultStream.close();
        prOutStr.close();
        prOutStr = null;
        
        trainBut.setEnabled(true);
        trainMenuItem.setEnabled(true);
        try {
            
            /////// typing results at results text /////////
            FileReader readresults = new FileReader(lastFile2KeepResults);
            char[] cbuf = new char[(int)lastFile2KeepResults.length()];
            readresults.read(cbuf);
            readresults.close();
            
            resultsTxt.setText(String.valueOf(cbuf));
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        ///////////////////////////////////////////
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_PROCESS_END_FILE)).play();
        
    }
    
    private void performTrain(PrintStream prOut, File[] trainSet){
        
        try{
            
            
            long start;
            
            // paths to files to save entities found
            String perOutFileName = Constants.PERSON_ENTITIES_FILE_PATH;
            // paths to files to save entities found
            String orgOutFileName = Constants.ORGANIZATION_ENTITIES_FILE_PATH;
            
            // setting matcher and separator because they may differ from the default
            // ones as created internally by Trainer
            trainer.setMatcher(Constants.TIMEX_PATERNS_FILE_PATH);
            trainer.setSeparator(PERIOD_SEPERATOR_CLASSIFIER_FILE_LOADED_PATH);
            
            System.out.println("Creating First Pass lists for category PERSON...");
            start = System.currentTimeMillis();
            perListManagerFP = ListManager.initListManager(
                    Constants.PERSON, trainSet,
                    Constants.FIRST_PASS, trainer.getSeparator(), LISTS_DIRECTORY_LOADED_PATH);
            
            System.out.println("Creating First Pass lists for category ORGANIZATION...");
            orgListManagerFP = ListManager.initListManager(
                    Constants.ORGANIZATION, trainSet,
                    Constants.FIRST_PASS, trainer.getSeparator(), LISTS_DIRECTORY_LOADED_PATH);
            System.out.println("FIRST PASS LISTS DONE, " + Constants.toTimeString(start));
            
            start = System.currentTimeMillis();
            
            if (currentPass.equalsIgnoreCase(Constants.FIRST_PASS)){
                
                ////////////////////////////////////////////////////////////////////
                ///////////////////////////// PERSON ///////////////////////////////
                ////////////////////////////////////////////////////////////////////
                
                perTrainVecs = trainer.trainPLFP(
                        Constants.PERSON,
                        trainSet,
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH,
                        perListManagerFP, prOut);
                System.out.println("Training for person is done");
                
                ////////////////////////////////////////////////////////////////////
                /////////////////////////ORGANIZATION///////////////////////////////
                ////////////////////////////////////////////////////////////////////
                
                orgTrainVecs = trainer.trainPLFP(
                        Constants.ORGANIZATION,
                        trainSet,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH,
                        orgListManagerFP, prOut);
                System.out.println("Training for organization is done");
                
            }
            
            ///////////////////////// SECOND PASS //////////////////////////////
            else {// currentPass.equalsIgnoreCase(Constants.SECOND_PASS)
                
                ////////////////////////////////////////////////////////////////////
                ///////////////////////////// PERSON ///////////////////////////////
                ////////////////////////////////////////////////////////////////////
                
                /////////////////////////////////// lists ////////////////////////////////////////////////
                System.out.println("Creating Second Pass lists for categories PERSON and ORGANIZATION...");
                
                perListManagerSP = ListManager.initListManager(
                        Constants.PERSON, trainSet,
                        Constants.SECOND_PASS, trainer.getSeparator(), LISTS_DIRECTORY_LOADED_PATH);
                
                orgListManagerSP = ListManager.initListManager(
                        Constants.ORGANIZATION, trainSet,
                        Constants.SECOND_PASS, trainer.getSeparator(), LISTS_DIRECTORY_LOADED_PATH);
                
                System.out.println("SECOND PASS LISTS DONE, " + Constants.toTimeString(start));
                //===================================== lists ==============================================
                
                
                //////////////////////////// second pass training for person ///////////////////////////////
                
                ////////////////////////////////////////////////////////////////////
                /////////////////////////ORGANIZATION///////////////////////////////
                ////////////////////////////////////////////////////////////////////
                
                System.out.println("Second pass training for category " + Constants.PER_ORG);
                
                int []vecsNum;
                vecsNum = trainer.trainPLSP(
                        
                        Constants.PER_ORG, trainSet,
                        trainer.getMatcher(), trainer.getSeparator(),
                        
                        FP_PER_CLASSIFIER_FILE_LOADED_PATH, perListManagerFP,
                        FP_ORG_CLASSIFIER_FILE_LOADED_PATH, orgListManagerFP,
                        
                        SP_PER_CLASSIFIER_FILE_LOADED_PATH, perListManagerSP,
                        SP_ORG_CLASSIFIER_FILE_LOADED_PATH, orgListManagerSP,
                        
                        prOut);
                
                perTrainVecs = vecsNum[0];
                orgTrainVecs = vecsNum[1];
                
                System.out.println("Training for person and Organization done");
                
            }// else- second pass
            
        }catch(Exception e){
            e.printStackTrace();
            System.exit(-1);
        }
        
    }
    
    
    
    private void trainButActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_trainButActionPerformed
        
        // asks for Training Files
        File[] trainSet;
        if((trainSet = choseFilesSet("Chose text files for training..", "ok")) == null){
            return;
        }
        
        
        long startTrain = System.currentTimeMillis();
        
        lastFile2KeepResults = createResultFiles();
        try {
            resultStream = new FileOutputStream(lastFile2KeepResults);
            prOutStr = new PrintStream(resultStream);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        performTrain(prOutStr, trainSet);
        dialog.showMessageDialog(
                null, "Time needed for performing training for both categories is : \n"+
                Constants.toTimeString(startTrain), "Time", JOptionPane.QUESTION_MESSAGE,
                new ImageIcon(getClass().getResource(Constants.IMA_INFO_01_01)));
        
        
        
        
    }//GEN-LAST:event_trainButActionPerformed
    
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        
        backBut.doClick();
        
    }//GEN-LAST:event_formWindowClosing
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton backBut;
    private javax.swing.JMenuItem backMI;
    private javax.swing.JPanel bottomPnl;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.ButtonGroup buttonGroup2;
    private javax.swing.JPanel buttonsPnl;
    private javax.swing.JPanel centerPnl;
    private javax.swing.JMenuItem changeParamMItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.ButtonGroup financialGeneralArticlesButGrp;
    private javax.swing.JRadioButtonMenuItem fpasradMenu;
    private javax.swing.JRadioButton fpasradTool;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JPanel northPnl;
    private javax.swing.JPanel northWestPnl;
    private javax.swing.JMenu optMenu;
    private javax.swing.JMenu paramMenu;
    private javax.swing.JMenu pasLeMenu;
    private javax.swing.JMenu passMenu;
    private javax.swing.JTextPane resultsTxt;
    private javax.swing.JRadioButtonMenuItem spasradMenu;
    private javax.swing.JRadioButton spasradTool;
    private javax.swing.JButton testBut;
    private javax.swing.JMenuItem testMenuItem;
    private javax.swing.JButton trainBut;
    private javax.swing.JMenuItem trainMenuItem;
    // End of variables declaration//GEN-END:variables
    
    
}
