/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package ipl.nerc;

import ipl.core.TokenNode;
import ipl.core.Tokenizer;
import ipl.nerc.lists.ListManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.splitter.SentenceSplitter;
import ipl.util.Constants;
import ipl.util.file_manager.FileManager;
import java.awt.HeadlessException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import javax.swing.JApplet;
import javax.swing.JOptionPane;

/**
 *
 * @author Xen.Vassilakos
 */

public  class Tagger implements Runnable{
    
    
    private String pass;
    private int classFilesToUse;
    private File[] filesToAnot;
    
    public static int USE_FILES_FOR_GENERAL = 0, USE_FILES_FOR_FINANCIAL = 1, USE_DEFAULT_FILES = 2;
    
    public Tagger(String pass, File[] filesToAnot, int classFilesToUse){
        
        this.pass = pass;
        this.filesToAnot = filesToAnot;
        this.classFilesToUse = classFilesToUse;
    }
    
    public void run(){
        try {
            
            
            long start = System.currentTimeMillis();
            String resultsDirPath = filesToAnot[0].getParentFile() + "/annotated_data";
            File resDir  = new File(resultsDirPath);
            resDir.mkdirs();
            File perFile = new File(resultsDirPath, "persons.txt");
            File orgFile = new File(resultsDirPath, "organizations.txt");
            perFile.getParentFile().mkdirs();
            orgFile.getParentFile().mkdirs();
            
            BufferedWriter perBuf = new BufferedWriter(new FileWriter(perFile));
            BufferedWriter orgBuf = new BufferedWriter(new FileWriter(orgFile));
            
            ////////////////////////////////////////////////////////////////////
            ////////////////First pass/////////////////////////////////////////
            if(pass.equalsIgnoreCase(Constants.FIRST_PASS)){
                
                // for every file to be anotated
                for(File nextAnotFile: this.filesToAnot){
                    
                    Tokenizer tokens = null;
                    // if user wants to use default classifiers
                    if(classFilesToUse == USE_FILES_FOR_GENERAL){
                        
                        tokens = InputTextsManager.tokenizerForFP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_GENERAL_ARTICLES_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_GENERAL_FILE_PATH),
                                Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH)
                                );
                        
                    }else if(classFilesToUse == USE_FILES_FOR_FINANCIAL){
                        
                        tokens = InputTextsManager.tokenizerForFP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_FINANCIAL_ARTICLES_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_FINANCIAL_FILE_PATH),
                                Constants.FP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH)
                                );
                        
                        
                    } else if(classFilesToUse == USE_DEFAULT_FILES){
                        
                        tokens = InputTextsManager.tokenizerForFP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH),
                                Constants.FP_PERSON_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_DIRECTORY_PATH)
                                );
                        
                        
                    } else if(classFilesToUse == -1){
                        // do nothing
                    }else{
                        System.err.println("Wrong argument for tagging.. Invalid choice for file paths");
                        System.exit(-1);
                    }//else
                    
                    
                    FileManager.write(tokens, resultsDirPath + "/" +  nextAnotFile.getName());
                    printEntity(perBuf, orgBuf, tokens, nextAnotFile.getName());
                    
                    
                }// for
                
            }// if first pass
            
            
            ////////////////////////////////////////////////////////////////////
            ////////////////second pass/////////////////////////////////////////
            if(pass.equalsIgnoreCase(Constants.SECOND_PASS)){
                
                // for every file to be anotated
                for(File nextAnotFile: this.filesToAnot){
                    
                    Tokenizer tokens = null;
                    // if user wants to use default classifiers
                    if(classFilesToUse == USE_FILES_FOR_GENERAL){
                        
                        tokens = InputTextsManager.tokenizerForSP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_GENERAL_ARTICLES_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_GENERAL_FILE_PATH),
                                Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH,
                                Constants.SP_PERSON_GENERAL_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH,
                                Constants.SP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, Constants.LISTS_GENERAL_DIRECTORY_PATH)
                                );
                        
                    }else if(classFilesToUse == USE_FILES_FOR_FINANCIAL){
                        
                        tokens = InputTextsManager.tokenizerForSP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_FINANCIAL_ARTICLES_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_FINANCIAL_FILE_PATH),
                                Constants.FP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH,
                                Constants.SP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH,
                                Constants.SP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, Constants.LISTS_FINANCIAL_DIRECTORY_PATH)
                                );
                        
                        
                    } else if(classFilesToUse == USE_DEFAULT_FILES){
                        
                        tokens = InputTextsManager.tokenizerForSP(
                                nextAnotFile.getCanonicalPath(),
                                new TimexMatcher(Constants.TIMEX_PATERNS_FILE_PATH),
                                new SentenceSplitter(Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH),
                                Constants.FP_PERSON_CLASSIFIER_FILE_PATH,
                                Constants.SP_PERSON_CLASSIFIER_FILE_PATH,
                                Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH,
                                Constants.SP_ORGANIZATION_CLASSIFIER_FILE_PATH,
                                ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, Constants.LISTS_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, Constants.LISTS_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, Constants.LISTS_DIRECTORY_PATH),
                                ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, Constants.LISTS_DIRECTORY_PATH)
                                );
                        
                        
                    } else if(classFilesToUse == -1){
                        // do nothing
                    }else{
                        System.err.println("Wrong argument for tagging.. Invalid choice for file paths");
                        System.exit(-1);
                    }//else
                    
                    
                    FileManager.write(tokens, resultsDirPath + "/" +  nextAnotFile.getName());
                    printEntity(perBuf, orgBuf, tokens, nextAnotFile.getName());
                    
                    
                }// for
                
                
            }//if second pass
            
            perBuf.close();
            orgBuf.close();
            
            PrintStream out = System.out;
            System.setOut(new PrintStream(resultsDirPath + "/time.txt"));
            System.out.println("time needed = " + Constants.toTimeString(start));
            System.out.close();
            
            JApplet.newAudioClip(getClass().getResource(Constants.SOUND_PROCESS_END_FILE)).play();
            
            System.setOut(out);
            JOptionPane.showMessageDialog(null, "See annotation results at:\n" + resDir.getCanonicalPath() );
            
            System.out.println("\n Finished");
            
        } catch (HeadlessException ex) {
            ex.printStackTrace();
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
        
    }// run()
    
    public void printEntity(BufferedWriter personBufout, BufferedWriter organizationBufout, Tokenizer tok, String filename){
        
        try {
            
            personBufout.write("\n" + filename + "\n=====================\n");
            int i = 0;
            boolean isPrevPer = false;
            for(TokenNode node = tok.get(i); i < tok.size(); node = tok.get(i++)){
                
                if(node.getTagType().equals(Constants.PERSON)){
                    if(!isPrevPer){
                        personBufout.write("\n");
                    }else{
                        personBufout.write(" ");
                    }
                    personBufout.write(node.getToken());
                    isPrevPer = true;
                    
                }else{
                    isPrevPer = false;
                }
                
            }
            personBufout.write("\n");
            
            
            organizationBufout.write("\n" + filename + "\n=====================\n");
            i = 0;
            boolean isPrevOrg = false;
            for(TokenNode node = tok.get(i); i < tok.size(); node = tok.get(i++)){
                
                if(node.getTagType().equals(Constants.ORGANIZATION)){
                    if(!isPrevOrg){
                        organizationBufout.write("\n");
                    }else{
                        organizationBufout.write(" ");
                    }
                    organizationBufout.write(node.getToken());
                    isPrevOrg = true;
                    
                }else{
                    isPrevOrg = false;
                }
                
            }
            organizationBufout.write("\n");
            
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
    
    private static void printParametersAndExit(String str) {
        
        String defaultFPPerClas=null, defaultFPOrgClas=null, defaultSPPerClas=null,
                defaultSPOrgClas=null, defaultList=null, defaultTimex=null, defaultSepar=null;
        try {
            
            defaultFPPerClas = new File(Constants.FP_PERSON_CLASSIFIER_FILE_PATH).getCanonicalPath();
            defaultFPOrgClas = new File(Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH ).getCanonicalPath();
            defaultSPPerClas = new File(Constants.SP_PERSON_CLASSIFIER_FILE_PATH ).getCanonicalPath();
            defaultSPOrgClas = new File(Constants.SP_ORGANIZATION_CLASSIFIER_FILE_PATH ).getCanonicalPath();
            defaultList = new File(Constants.LISTS_DIRECTORY_PATH).getCanonicalPath();
            defaultTimex = new File(Constants.TIMEX_PATERNS_FILE_PATH).getCanonicalPath();
            defaultSepar = new File(Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH).getCanonicalPath();
        } catch (IOException ex) {
            
            ex.printStackTrace();
            System.exit(-1);
            
        }
        
        
        
        System.err.println(str);
        System.err.println();
        System.err.println();
        System.err.println("java ipl.nerc.Tagger -d <input dir path> [options]");
        System.err.println();
        System.err.println("-d");
        System.err.println("\tThe directory containing text files that you want to process");
        System.err.println("\nOPTIONS");
        System.err.println("=======");
        System.err.println("-fin");
        System.err.println("\tTag using files specialized for financial articles");
        System.err.println("-gen");
        System.err.println("\tTag using files specialized for general newspaper articles");
        System.err.println("-cur");
        System.err.println("\tTag using files of the re-trained form of the system, specialized in a genre different than general or financial newspaper articles");
        
        System.err.println("-m");
        System.err.println("\tMode:");
        System.err.println("\t\t1. Tag using only the first pass SVM");
        System.err.println("\t\t2. Tag using both the first and second pass SVMs  (default)");
        
        System.err.println("-1p");
        System.err.println("\tThe first pass classifier filename for person\t(default: \"" + defaultFPPerClas + "\"");
        System.err.println("-1o");
        System.err.println("\tThe first pass classifier filename for organization\t(default: \"" +  defaultFPOrgClas + "\"");
        System.err.println("-2p");
        System.err.println("\tThe second pass classifier filename for person\t(default: \"" + defaultSPPerClas + "\"");
        System.err.println("-2o");
        System.err.println("\tThe second pass classifier filename for organization\t(default: \"" +  defaultSPOrgClas + "\"");
        
        System.err.println("-l");
        System.err.println("\tThe lists directory\t\t\t(default: \"" + defaultList + "\")");
        System.err.println("-t");
        System.err.println("\tThe TIMEX patterns filename\t\t(default: \"" + defaultTimex + "\"");
        System.err.println("-p");
        System.err.println("\tThe sentence splitter filename\t\t(default: \"" + defaultSepar + "\")");
        System.err.println();
        System.exit(1);
    } // printParametersAndExit
    
    public void TAG_FP(String dirPath, String perFPClass, String orgFPClass,
            String listsPath, TimexMatcher matcher, SentenceSplitter splitter) throws Exception{
        
        TAG_SP(dirPath, perFPClass, orgFPClass, null, null, listsPath,  matcher, splitter);
        
    }
    
    public void TAG_SP(String dirPath, String perFPClass, String orgFPClass, String perSPClass, String orgSPClass,
            String listsPath, TimexMatcher matcher, SentenceSplitter splitter) throws Exception {
        
        
        // finding pass, first or second, from sp classifier parameter values
        String pass = (perSPClass == null)||(orgSPClass == null) ?Constants.FIRST_PASS: Constants.SECOND_PASS;
        long start = System.currentTimeMillis();
        
        String resultsDirPath = dirPath + "/annotated_data";
        File resDir  = new File(resultsDirPath);
        resDir.mkdirs();
        
        File perFile = new File(resDir, "persons.txt");
        File orgFile = new File(resDir, "organizations.txt");
        File []filesToAnot = new File(dirPath).listFiles();
        
        /////////////////////////////////////////////////
        ////////////avoid directories////////////////////
        ////////////keep only files to annotate//////////
        ArrayList<File> tmplist = new ArrayList<File>();
        
        for(File nextFil: filesToAnot){
            
            if(nextFil.isFile())tmplist.add(nextFil);
            
        }
        filesToAnot = new File[tmplist.size()];
        tmplist.toArray(filesToAnot);
        
        /////---------------------------------------////
        
        
        BufferedWriter perBuf = new BufferedWriter(new FileWriter(perFile));
        BufferedWriter orgBuf = new BufferedWriter(new FileWriter(orgFile));
        
        ////////////////////////////////////////////////////////////////////
        ////////////////////first pass//////////////////////////////////////
        if(pass.equalsIgnoreCase(Constants.FIRST_PASS)) {
            
            // for every file to be anotated
            for(File nextAnotFile: filesToAnot){
                
                Tokenizer tokens = InputTextsManager.tokenizerForFP(
                        nextAnotFile.getCanonicalPath(),
                        matcher, splitter,
                        perFPClass,
                        orgFPClass,
                        ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, listsPath),
                        ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, listsPath)
                        );
                
                FileManager.write(tokens, resultsDirPath + "/" +  nextAnotFile.getName());
                printEntity(perBuf, orgBuf, tokens, nextAnotFile.getName());
                
            }// for
            
        }
        ////////////////////////////////////////////////////////////////////
        ////////////////second pass/////////////////////////////////////////
        else{
            // for every file to be anotated
            for(File nextAnotFile: filesToAnot){
                
                Tokenizer tokens = InputTextsManager.tokenizerForSP(
                        nextAnotFile.getCanonicalPath(),
                        matcher, splitter,
                        perFPClass, perSPClass,
                        orgFPClass, orgSPClass,
                        ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, listsPath),
                        ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, listsPath),
                        ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, listsPath),
                        ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, listsPath)
                        );
                
                FileManager.write(tokens, resultsDirPath + "/" +  nextAnotFile.getName());
                printEntity(perBuf, orgBuf, tokens, nextAnotFile.getName());
                
            }// for
            
        }//else second pass
        
        perBuf.close();
        orgBuf.close();
        
        PrintStream out = System.out;
        System.setOut(new PrintStream(resultsDirPath + "/time.txt"));
        System.out.println("time needed = " + Constants.toTimeString(start));
        System.out.close();
        
        JApplet.newAudioClip(getClass().getResource(Constants.SOUND_PROCESS_END_FILE)).play();
        
        System.setOut(out);
        File outputF = filesToAnot[0].getParentFile();
        System.out.println("See results in path:\n" + (new File(resultsDirPath).getCanonicalPath()) );
        
        System.out.println("\n Finished");
        
        
    }
    
    
    public static void main(String args[]) throws Exception {
        
        /************ testing main section
         * String s1 =
         * "-m 2 -t " + Constants.TIMEX_PATERNS_FIN_ARTICLES_FILE_PATH +
         * " -p " + Constants.SENTENCE_SPLITTER_CLASSIFIER_GENERAL_FILE_PATH +
         * " -l " + Constants.LISTS_GENERAL_DIRECTORY_PATH +
         * " -d ../files/sample_data/input_examples -m 2 " +
         * " -1P " + Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH +
         * " -2p " + Constants.SP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH;
         *
         *
         * String s2 = "-fin -d ../files/sample_data/input_examples -m 1";
         * String s3 = "-gen -d ../files/sample_data/input_examples -m 2 -1p " + Constants.FP_PERSON_CLASSIFIER_FILE_PATH;
         * String s4 = "-fin -d ../files/sample_data/input_examples -m 2 " +
         * " -1P " + Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH +
         * " -2p " + Constants.SP_PERSON_GENERAL_CLASSIFIER_FILE_PATH +
         * " -1O " + Constants.FP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH +
         * " -2o " + Constants.SP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH;
         * String s5 = "-fin -d ../files/sample_data/input_examples -m 1 " +
         * " -1P " + Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH +
         * " -2p " + Constants.SP_PERSON_GENERAL_CLASSIFIER_FILE_PATH +
         * " -1O " + Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH ;
         *
         * StringTokenizer to = new StringTokenizer(s1);
         * args = new String[to.countTokens()];
         * int ipos = 0;
         * while(to.hasMoreTokens()){
         *
         * args[ipos++] = to.nextToken();
         *
         * }
         **********************/
        
        ///////////////////////////////////////////////////////////////////////////////
        ///////////////// default values to be used if not specified //////////////////
        /////////////////////////// in command line ///////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////
        int mode = 2;
        String classFPPer = Constants.FP_PERSON_CLASSIFIER_FILE_PATH;//"CLASSIFIER_FP";
        String classFPOrg = Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH;//"CLASSIFIER_FP";
        String classSPPer = Constants.SP_PERSON_CLASSIFIER_FILE_PATH;//"CLASSIFIER_SP";
        String classSPOrg = Constants.SP_ORGANIZATION_CLASSIFIER_FILE_PATH;//"CLASSIFIER_SP";
        String testdata = "";
        String listsdir = Constants.LISTS_DIRECTORY_PATH;//"./lists";
        String patternsfn = Constants.TIMEX_PATERNS_FILE_PATH;//"./TIMEX_PATTERNS";
        String separatorfn =Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH;// "./SENTENCE_SPLITTER";
        
        if (args.length < 2) {
            printParametersAndExit("Invalid parameters number!. You must specify directory of files.");
        } // if
        
        for ( int i = 0; i < args.length; i += 2) {
            
            if (args[i].charAt(0) != '-') {
                System.err.println(args[i]);
                printParametersAndExit("Invalid parameters! You must use \"-\" before every parameter");
                
            } // if
            char param = ' ';
            
            if(i == 0){// first argument must specify -fin -gen or -cur
                // give to char virable a value according to -fin or -gen argument
                if (args[i].equalsIgnoreCase("-fin")) {
                    param = 'f';
                } // if
                else if (args[i].equalsIgnoreCase("-gen")) {
                    param = 'g';
                } // else if
                else if (args[i].equalsIgnoreCase("-cur")) {
                    param = 'u';
                } // else if
                else {
                    printParametersAndExit("First argument must be on of the following:\n" +
                            "\t-fin for using files for financial articles\n" +
                            "\t-gen for using files for general articles\n" +
                            "\t-cur\n for using files of the retrained form of the system");
                } // else
                
            } else {// if (i!=0)
                
                param = args[i].charAt(1);
                
            } // else
            
            switch (param) {
                case '1':
                    if(args[i].charAt(2) == 'p'){
                        classFPPer = args[i + 1];
                    }else if(args[i].charAt(2) == 'o'){
                        classFPOrg = args[i + 1];
                    }else if(args[i].charAt(2) == 'P'){
                        classFPPer = args[i + 1];
                    }else if(args[i].charAt(2) == 'O'){
                        classFPOrg = args[i + 1];
                    }else{
                        printParametersAndExit("Invalid parameters! Must be either \"-1P\" or \"-1O\"");
                    }
                    break;
                case '2':
                    if(args[i].charAt(2) == 'p'){
                        classSPPer = args[i + 1];
                    }else if(args[i].charAt(2) == 'o'){
                        classSPOrg = args[i + 1];
                    }else if(args[i].charAt(2) == 'P'){
                        classFPPer = args[i + 1];
                    }else if(args[i].charAt(2) == 'O'){
                        classFPOrg = args[i + 1];
                    }else{
                        printParametersAndExit("Invalid parameters! Must be either \"-1P\" or \"-1O\"");
                    }
                    break;
                case 'd':
                    testdata = args[i + 1];
                    break;
                case 'l':
                    listsdir = args[i + 1];
                    break;
                case 't':
                    patternsfn = args[i + 1];
                    break;
                case 'p':
                    separatorfn = args[i + 1];
                    break;
                case 'm':
                    mode = Integer.parseInt(args[i + 1]);
                    if ((mode != 1)&&(mode != 2)) {
                        
                        printParametersAndExit("Invalid mode!");
                        
                    }
                    break;
                case 'f'://financial
                    classFPPer = Constants.FP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH;
                    classSPPer = Constants.SP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH;
                    classFPOrg = Constants.FP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH;
                    classSPOrg = Constants.SP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH;
                    listsdir = Constants.LISTS_FINANCIAL_DIRECTORY_PATH;
                    patternsfn = Constants.TIMEX_PATERNS_FINANCIAL_ARTICLES_FILE_PATH; 
                    separatorfn = Constants.SENTENCE_SPLITTER_CLASSIFIER_FINANCIAL_FILE_PATH;
                    i--;
                    break;
                case 'g':// general
                    classFPPer = Constants.FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH;
                    classSPPer = Constants.SP_PERSON_GENERAL_CLASSIFIER_FILE_PATH;
                    classFPOrg = Constants.FP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH;
                    classSPOrg = Constants.SP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH;
                    listsdir = Constants.LISTS_GENERAL_DIRECTORY_PATH;
                    patternsfn = Constants.TIMEX_PATERNS_GENERAL_ARTICLES_FILE_PATH; 
                    separatorfn = Constants.SENTENCE_SPLITTER_CLASSIFIER_GENERAL_FILE_PATH;
                    i--;
                    break;
                case 'u':// currently trained
                    classFPPer = Constants.FP_PERSON_CLASSIFIER_FILE_PATH;
                    classFPOrg = Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH;
                    classSPPer = Constants.SP_PERSON_CLASSIFIER_FILE_PATH;
                    classSPOrg = Constants.SP_ORGANIZATION_CLASSIFIER_FILE_PATH;
                    listsdir = Constants.LISTS_DIRECTORY_PATH;
                    patternsfn = Constants.TIMEX_PATERNS_FILE_PATH;
                    separatorfn =Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH;
                    i--;
                    break;
                default:
                    printParametersAndExit("Invalid parameter " + param);
                    break;
            } // switch
        } // for
        
        classFPPer = new File(classFPPer).getCanonicalPath();
        classSPPer = new File(classSPPer).getCanonicalPath();
        classFPOrg = new File(classFPOrg).getCanonicalPath();
        classSPOrg = new File(classSPOrg).getCanonicalPath();
        listsdir = new File(listsdir).getCanonicalPath();
        patternsfn = new File(patternsfn).getCanonicalPath();
        separatorfn = new File(separatorfn).getCanonicalPath();
        testdata = new File(testdata).getCanonicalPath();
        
        TimexMatcher matcher = new TimexMatcher(patternsfn);
        SentenceSplitter splitter = new SentenceSplitter(separatorfn);
        ListManager listManagerFPPer = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, listsdir);
        ListManager listManagerFPOrg = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, listsdir);
        ListManager listManagerSPPer = ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, listsdir);
        ListManager listManagerSPOrg = ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, listsdir);
        
        if (testdata.compareTo("") == 0) {
            printParametersAndExit("You must speciafy the text files directory path!");
        } // if
        if (!(new File(testdata)).exists() || !(new File(testdata)).isDirectory()) {
            printParametersAndExit("Invalid text files directory path!");
        } // if
        if (!(new File(classFPPer)).exists()) {
            printParametersAndExit("First pass person classifier path is not valid!");
        } // if
        if(!(new File(classFPPer)).isFile()) {
            printParametersAndExit("Invalid first pass person classifier filename!");
        } // if
        
        if (!(new File(classSPPer)).exists() || !(new File(classSPPer)).isFile()) {
            printParametersAndExit("Invalid second pass person classifier filename!");
        } // if
        if (!(new File(classFPOrg)).exists() || !(new File(classFPPer)).isFile()) {
            printParametersAndExit("Invalid first pass organization classifier filename!");
        } // if
        if (!(new File(classSPOrg)).exists() || !(new File(classSPPer)).isFile()) {
            printParametersAndExit("Invalid second pass organization classifier filename!");
        } // if
        if (!(new File(listsdir)).exists() || !(new File(listsdir)).isDirectory()) {
            printParametersAndExit("Invalid lists directory!");
        } // if
        if (!(new File(patternsfn)).exists() || !(new File(patternsfn)).isFile()) {
            printParametersAndExit("Invalid TIMEX patterns filename!");
        } // if
        if (!(new File(separatorfn)).exists() || !(new File(patternsfn)).isFile()) {
            printParametersAndExit("Invalid sentence splitter filename!");
        } // if
        
        
        
        System.err.println("\n*\n*\n*Files and directories used:");
        System.err.println(classFPPer);
        System.err.println(classFPOrg);
        if(mode == 2) System.err.println(classSPPer);
        if(mode == 2) System.err.println(classSPOrg);
        System.err.println(testdata);
        System.err.println(listsdir);
        System.err.println(patternsfn);
        System.err.println(separatorfn);
        System.err.println("\n*\n*\n*");
        
        Tagger tagger = new Tagger(null, null, Tagger.USE_DEFAULT_FILES);
        
        if (mode == 1) {// first pass
            System.out.println("Annotating files...\n");
            tagger.TAG_FP(testdata, classFPPer, classFPOrg, listsdir, matcher, splitter);
        } // if
        else {// second pass (mode = 2)
            
            System.out.println("Annotating files...\n");
            tagger.TAG_SP(testdata, classFPPer, classFPOrg, classSPPer, classSPOrg, listsdir, matcher, splitter);
        } // else
        
        System.exit(0);
        
        
        
    } // end of main
    
}// Tagger  class



