/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning;

import ipl.nerc.Trainer;
import ipl.util.Constants;
import ipl.nerc.active_learning.data.InformationVector;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.features.OrganizationFeaturesManager;
import ipl.nerc.features.PersonFeaturesManager;
import ipl.util.file_manager.DirectoryManager;
import ipl.util.file_manager.LibSVMFileManager;
import ipl.splitter.SentenceSplitter;
import ipl.nerc.preprocessing.TextPreprocessor;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.util.Auxiliary;
import ipl.nerc.lists.ListManager;
import java.awt.HeadlessException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Set;
import java.util.StringTokenizer;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;

/**
 *
 * @author Xen.Vassilakos
 */
public class PrepareSP  {
    
    private ListManager perListManager, orgListManager;
    private String pass;
    
    /**
     * Creates a new instance of PrepareSP
     */
    public PrepareSP(String pass) {
        
        this.pass = pass;
        perListManager = ListManager.initListManager(Constants.PERSON, pass, Constants.LISTS_DIRECTORY_PATH);
        orgListManager = ListManager.initListManager(Constants.ORGANIZATION, pass, Constants.LISTS_DIRECTORY_PATH);
        
    }
    
    /**
     * Creates a new instance of PrepareSP
     */
    public PrepareSP(String pass, String listsDirectoryPath) {
        
        this.pass = pass;
        perListManager = ListManager.initListManager(Constants.PERSON, pass, listsDirectoryPath);
        orgListManager = ListManager.initListManager(Constants.ORGANIZATION, pass, listsDirectoryPath);
        
        
    }
    
    /**
     * Method to be used for merging sure decisions of previous pass
     * Merges Hashtable files, selected in selectedFiles ArrayList parameter.
     * The merged Hashtables are both saved in File output and returned.
     */
    public Hashtable <String, String[]> mergeHashTab1(File output, ArrayList<File> selectedFiles){
        
        ObjectInputStream in;
        Hashtable<String, String[]> toReturn = new Hashtable();
        
        try {
            
            for (File nextF: selectedFiles){
                
                System.out.println("Merging file:\n" + nextF.getName());
                in = new ObjectInputStream(new FileInputStream(nextF));
                Hashtable<String, String[]> nextH = new Hashtable<String, String[]>();
                nextH = (Hashtable<String, String[]>)in.readObject();
                in.close();
                Enumeration en = nextH.keys();
                while (en.hasMoreElements()){
                    
                    String key = (String)en.nextElement();
                    toReturn.put(key, nextH.get(key));
                    
                }// while
                
            }// for
            
            ObjectOutputStream out =
                    new ObjectOutputStream(
                    new FileOutputStream(output));
            out.writeObject(toReturn);
            out.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return toReturn;
        
    }
    
    /**
     * Method to be used for merging all decisions of previous pass.
     * Merges Hashtable files, selected in selectedFiles ArrayList parameter.
     * The merged Hashtables are both saved in File output and returned.
     */
    public Hashtable <String, String> mergeHashTab2(File output, ArrayList<File> selectedFiles){
        
        ObjectInputStream in;
        Hashtable<String, String> toReturn = new Hashtable();
        
        try {
            
            for (File nextF: selectedFiles){
                
                System.out.println("Merging file:\n" + nextF.getName());
                in = new ObjectInputStream(new FileInputStream(nextF));
                Hashtable<String, String> nextH = new Hashtable<String, String>();
                nextH = (Hashtable<String, String>)in.readObject();
                in.close();
                Enumeration en = nextH.keys();
                while (en.hasMoreElements()){
                    
                    String key = (String)en.nextElement();
                    toReturn.put(key, nextH.get(key));
                    
                }// while
                
            }// for
            
            ObjectOutputStream out =
                    new ObjectOutputStream(
                    new FileOutputStream(output));
            out.writeObject(toReturn);
            out.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return toReturn;
        
    }
    
    /**
     * Reads from an Inforamation file the desicions by previous pass svms, and constructs a
     * hashtable data structure to return.
     * The hashtable returned, uses the in each record the String concatenation of
     * the file name and the position of a token as a key, and the decision made
     * for this token as the value.
     */
    public static  Hashtable<String, String> loadVectorsToConvert(
            String inputFileName, PrintStream output) {
        
        // to remenber which vectors are read so far.
        // It maps the answer read from file for a token, to that token
        Hashtable <String, String> vecAnswer = new Hashtable<String,String>();
        // counter for tokens in text from input file
        int count = 0;
        ////////////////////////////////////////////////////////////////////////
        //////////////////////reading from input file///////////////////////////
        File file = new File(inputFileName);
        byte[] buff = new byte[(int) file.length()];
        String text = "";
        try {
            FileInputStream fis = new FileInputStream(file);
            fis.read(buff);
            fis.close();
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
        text = new String(buff);
        //////////////////////reading from input file///////////////////////////
        //======================================================================
        
        // tokenized per line text
        StringTokenizer strTok = new StringTokenizer(text, "\n");
        
        while (strTok.hasMoreTokens()) {
            
            count++;
            String vector = strTok.nextToken();
            strTok.nextToken(); // word
            strTok.nextToken(); // neibourhood
            String pos = strTok.nextToken();// position of token
            String fileName = strTok.nextToken();// file name of token
            
            // answer by previous pass svms
            String answer;
            StringTokenizer ansTok = new StringTokenizer(vector);
            answer = ansTok.nextToken();
            if(!ansTok.equals(Constants.PERSON) && !ansTok.equals(Constants.ORGANIZATION)) {
                answer = "";
                System.err.println("ActiveLearning.loadVectorsToConvert: ERROR1\n" +
                        "*\tIncorrect category read.");
                System.exit(1);
            } // else
            pos = pos.trim();
            fileName = fileName.trim();
            String key = fileName + pos;
            
            if (vecAnswer.containsKey(key)) {
                
                System.err.println("ActiveLearning.loadVectorsToConvert: ERROR2\n" +
                        "*\tDupicant records found.");
                System.exit(1);
            } // if
            else {
                // remember you have read this about this specific vector information
                vecAnswer.put(key, answer);
            } // else
            vector = null;
        } // while
        
        output.println(count);
        
        strTok = null;
        return vecAnswer;
        
    } // loadVectorsToConvert
    
    /**
     * Converts training vectors used in previous pass, in vectors to be used in next pass.
     *
     *
     * @param dir
     *      Directory with all files consisting pool of active learning vectors
     * @param timexMatcherFileName
     *      Path to Matcher for tiome expressions
     * @param periodSeparatorFileName
     *      Path to classifier for period separation
     * @param perModelFileName
     *      Path to to previous pass classifier for PERSON category
     * @param orgModelFileName
     *      Path to to previous pass classifier for ORGANIZATION separation
     * @param perListManager
     *      ListManager object for categoty PERSON
     * @param orgListManager
     *      ListManager object for categoty ORGANIZATION
     * @param inputFilePath
     *      Path to file containing information about previous pass training vectors
     *      eg  ..../FPvectorsInformations.txt
     * @param perOutFileName
     *      Path to file containing information about next pass training vectors
     *      for category PERSON eg ..../PERSON_TRAINSECOND PASS
     * @param orgOutFileName
     *      Path to file containing information about next pass training vectors
     *      for category ORGANIZATION eg ..../ORGANIZATION_TRAINSECOND PASS
     *
     */
    public static void convertFPtoSP(
            String dir, String timexMatcherFileName, String periodSeparatorFileName,
            String perModelFileName, String orgModelFileName,
            Hashtable<String, String[]> sureFPDec, Hashtable<String, String> allFPDec, String fileName,
            ListManager perListManager,
            ListManager orgListManager, String inputFilePath, String perOutputFilePath,
            String orgOutputFilePath) throws Exception {
        
        
        // current file of pool
        File curr;
        //  margin for sure decisions by FP classifier
        double margin = 0.85;
        // counters for vectors "cut" by safe rules" in SP. perYes++ when cat by safe rures
        int perYes = 0, perNo = 0;
        int orgYes = 0, orgNo = 0;
        
        // loading training vectors token answers from previous pass
        Hashtable<String, String> hashtable = loadVectorsToConvert(inputFilePath, System.out);
        
        DirectoryManager dirManager = new DirectoryManager(dir);
        LibSVMFileManager perOutputFileManager = new LibSVMFileManager(perOutputFilePath);
        LibSVMFileManager orgOutputFileManager = new LibSVMFileManager(orgOutputFilePath);
        
        TimexMatcher matcher = new TimexMatcher(timexMatcherFileName);
        SentenceSplitter separator = new SentenceSplitter(periodSeparatorFileName);
        
        ////////////////////////////////////////////////////////////////////////
        ////////////// prepering models for both categories ////////////////////
        svm_model perModel = svm.svm_load_model(perModelFileName);
        int per_nr_class = svm.svm_get_nr_class(perModel);
        int []perCategories = new int[per_nr_class];
        svm.svm_get_labels(perModel, perCategories);
        double[] per_prob_estimates = new double[per_nr_class];
        
        svm_model orgModel = svm.svm_load_model(orgModelFileName);
        int org_nr_class = svm.svm_get_nr_class(orgModel);
        int []orgCategories = new int[org_nr_class];
        svm.svm_get_labels(orgModel, orgCategories);
        double[] org_prob_estimates = new double[org_nr_class];
        ////////////// prepering models for both categories ////////////////////
        //====================================================================//
        
        // for every file in active learning pool
        while ( (curr = dirManager.getNextFile()) != null) {
            
            if (curr.isFile()) {
                
                ////////////////////////////////////////////////////////////////
                ///////////////////////// FP testing ///////////////////////////
                //// for creating sure list decision of FP model over texts ////
                ////////////////////////////////////////////////////////////////
                
                String perAttr[];
                int perDistFromStartOfEntity = -1;
                // temporary for creating list with previous pass classifier sure decisions
                String perTemp[] = new String[512];
                int perListNum = 0;
                
                String orgAttr[];
                int orgDistFromStartOfEntity = -1;
                // temporary for creating list with previous pass classifier sure decisions
                String orgTemp[] = new String[512];
                int orgListNum = 0;
                
                ////////////////////////////////////////////////////////////////
                /////////////////////// reading text ///////////////////////////
                FileInputStream fis = new FileInputStream(curr);
                byte []buff = new byte[ (int) curr.length()];
                fis.read(buff);
                fis.close();
                String text = new String(buff);
                /////////////////////// reading text ///////////////////////////
                ////========================================================////
                
                FeaturesManager perFPVectors, orgFPVectors;
                perFPVectors = new PersonFeaturesManager(curr.getName(),text, perListManager, matcher, separator);
                orgFPVectors = new OrganizationFeaturesManager(curr.getName(),text, orgListManager, matcher, separator);
                
                // for each token of current file's text
                while ( (perAttr = perFPVectors.nextFeatVec(perDistFromStartOfEntity)) != null
                        && (orgAttr = orgFPVectors.nextFeatVec(orgDistFromStartOfEntity)) != null) {
                    
                    // answers and cannonical diastances from SP models of each category
                    double perAnswer, perCanDist, orgAnswer, orgCanDist;
                    
                    //////////////////////////////////////////////////////////////
                    /////////////////////////// person ///////////////////////////
                    int perNumOfAttributes = perAttr.length - 1 ;
                    svm_node perNodes[] = new svm_node[perNumOfAttributes];
                    
                    for (int i = 0; i < perNumOfAttributes; i++) {
                        
                        perNodes[i] = new svm_node();
                        perNodes[i].index = i+1;
                        perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                        
                    } // for
                    
                    // in case it is not surelly negative for person category because
                    // of person safe rules
                    if (perNumOfAttributes > 0) {
                        
                        // PREDICT
                        perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                        if (perCategories[0] == -1) {
                            perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                        } // if
                        else {
                            perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                        } // else
                        
                    } // if
                    else{
                        
                        perAnswer = -1;
                        perCanDist = -1;
                        
                    }
                    
                    if ((perAnswer == 1) && (perDistFromStartOfEntity == -1)) {
                        
                        perDistFromStartOfEntity = 1;
                        String currTok = perFPVectors.tokenAt(perFPVectors.getOffset() - 1).getToken();
                        if ((currTok.length() >= 3) && (perCanDist > margin)) {
                            perTemp[perListNum++] = currTok.toLowerCase();
                        }
                    } // if
                    else if (perAnswer == 1) {
                        
                        perDistFromStartOfEntity++;
                        String currTok = perFPVectors.tokenAt(perFPVectors.getOffset() - 1).getToken();
                        if ((currTok.length() >= 3) && (perCanDist > margin)) {
                            perTemp[perListNum++] = currTok.toLowerCase();
                        }
                    } // else if
                    else {
                        perDistFromStartOfEntity = -1;
                    } // else if
                    
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////////////organization///////////////////////////
                    
                    int orgNumOfAttributes = orgAttr.length - 1;
                    svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                    
                    for (int i = 0; i < orgNumOfAttributes; i++) {
                        
                        orgNodes[i] = new svm_node();
                        orgNodes[i].index = i+1;
                        orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                        
                    } // for
                    // PREDICT
                    
                    // in case it is not surelly negative for organization category because
                    // of organization safe rules
                    if (orgNumOfAttributes > 0) {
                        
                        orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                        if (orgCategories[0] == -1) {
                            orgCanDist =
                                    org_prob_estimates[1] - org_prob_estimates[0];
                        } // if
                        else {
                            orgCanDist =
                                    org_prob_estimates[0] - org_prob_estimates[1];
                        } // else
                    } // if
                    else{
                        orgAnswer = -1;
                        orgCanDist = -1;
                        
                    }
                    
                    if ((orgAnswer == 1) && (orgDistFromStartOfEntity == -1)) {
                        
                        orgDistFromStartOfEntity = 1;
                        String currOrgTok = orgFPVectors.tokenAt(orgFPVectors.getOffset() - 1).getToken();
                        if ((currOrgTok.length() >= 3) && (orgCanDist > margin)) {
                            orgTemp[orgListNum++] = currOrgTok.toLowerCase();
                        }
                    } // if
                    
                    else if (orgAnswer == 1) {
                        
                        orgDistFromStartOfEntity++;
                        String currOrgTok = orgFPVectors.tokenAt(orgFPVectors.getOffset() - 1).getToken();
                        if ((currOrgTok.length() >= 3) && (orgCanDist > margin)) {
                            orgTemp[orgListNum++] = currOrgTok.toLowerCase();
                        }
                    } // else if
                    else {
                        orgDistFromStartOfEntity = -1;
                    } // else if
                    
                }// while ... all tokens of current text
                
                // Sorting FP list for current text
                String []perList = Auxiliary.getSubList(perTemp, perListNum);
                Arrays.sort(perList);
                
                // Sorting FP list for current text
                String []orgList = Auxiliary.getSubList(orgTemp, orgListNum);
                Arrays.sort(orgList);
                
                
                ///////////////////////////////////////////////////////////////
                ///////// Creating training file for second  pass ////////////
                ///////////////////////////////////////////////////////////////
                
                FeaturesManager perSPVectors, orgSPVectors;
                perSPVectors = new PersonFeaturesManager(perFPVectors.getTokens(), perListManager);
                orgSPVectors = new OrganizationFeaturesManager(orgFPVectors.getTokens(), orgListManager);
                
                perDistFromStartOfEntity = -1;
                orgDistFromStartOfEntity = -1;
                
                // for every token of current text
                while (
                        (perAttr = perSPVectors.nextFeatVec(perDistFromStartOfEntity, sureFPDec, allFPDec, curr.getName())) != null
                        &&
                        (orgAttr = orgSPVectors.nextFeatVec(orgDistFromStartOfEntity, sureFPDec, allFPDec, curr.getName())) != null) {
                    
                    String key = Integer.toString(perSPVectors.getOffset() - 1) + curr.getName();
                    if ((perAttr.length != 1) && (hashtable.containsKey(key))) {
                        
                        perAttr[0] = hashtable.get(key);
                        perOutputFileManager.setData(perAttr);
                        perNo++;
                        
                    } // if
                    
                    if ((perAttr.length == 1) && (hashtable.containsKey(key))) {
                        perYes++;
                    } // if
                    // Checking if named entity is still open (there are more tokens of this entity)
                    // It is used as a feature for the end of the named entity
                    if ((perAttr[0].compareTo(Constants.PERSON) == 0) && (perDistFromStartOfEntity == -1)) {
                        perDistFromStartOfEntity = 1;
                    } // if
                    else if (perAttr[0].compareTo(Constants.PERSON) == 0) {
                        perDistFromStartOfEntity++;
                    } // else if
                    else if ((perAttr[0].compareTo(Constants.PERSON)!= 0) ) {
                        perDistFromStartOfEntity = -1;
                    } // else if
                    
                    //////////////////////organizations/////////////////////////
                    
                    key = Integer.toString(orgSPVectors.getOffset() - 1) + curr.getName();
                    if ((orgAttr.length != 1) && (hashtable.containsKey(key))) {
                        
                        orgAttr[0] = hashtable.get(key);
                        orgOutputFileManager.setData(orgAttr);
                        orgNo++;
                        
                    } // if
                    
                    if ((orgAttr.length == 1) && (hashtable.containsKey(key))) {
                        orgYes++;
                    } // if
                    // Checking if named entity is still open (there are more tokens of this entity)
                    // It is used as a feature for the end of the named entity
                    if ((orgAttr[0].compareTo(Constants.ORGANIZATION) == 0) && (orgDistFromStartOfEntity == -1)) {
                        orgDistFromStartOfEntity = 1;
                    } // if
                    else if (orgAttr[0].compareTo(Constants.ORGANIZATION) == 0) {
                        orgDistFromStartOfEntity++;
                    } // else if
                    else if ((orgAttr[0].compareTo(Constants.ORGANIZATION)!= 0) ) {
                        orgDistFromStartOfEntity = -1;
                    } // else if
                    
                    
                } // while
                
                perFPVectors = null;
                perSPVectors = null;
                perList = null;
                perTemp = null;
                
                orgFPVectors = null;
                orgSPVectors = null;
                orgList = null;
                orgTemp = null;
                
            } // if
            
        } // while
        
        
        perOutputFileManager.closeFile();
        orgOutputFileManager.closeFile();
        
        System.out.println("Not ok by Person safe rules: " + perYes);
        System.out.println("Ok by Person safe rules: " + perNo);
        System.out.println("Not ok by Organization safe rules: " + perYes);
        System.out.println("Ok by Organization safe rules: " + perNo);
        
    } //
    
    
    /**
     * Creates and saves lists with <br>
     * (1) all decisions for all tokens, and <br>
     * (2) lists for tokens according to sure decisions by previous pass classifiers.<br><br>
     * Sure decisions are considered those canonical possibilities assigned by previous pass classifiers
     * that are equal or grater than a margin used (eg 0.75).
     * The created lists are both saved as a HashTable Object and as ordinary text files.
     *
     *
     * @param margin
     *      A margin for considering a desicion by previous classifier as sure
     * @param sourceDir
     *      Directory with all files consisting pool of active learning vectors
     * @param timexMatcherFileName
     *      Path to Matcher for tiome expressions
     * @param periodSeparatorFileName
     *      Path to classifier for period separation
     * @param perModelFileName
     *      Path to previous pass classifier for PERSON category
     * @param orgModelFileName
     *      Path to to previous pass classifier for ORGANIZATION separation
     * @param perListManager
     *      ListManager object for categoty PERSON
     * @param orgListManager
     *      ListManager object for categoty ORGANIZATION
     * @param inputFilePath
     *      Path to file containing information about previous pass training vectors
     *      eg  ..../FPvectorsInformations.txt
     * @param perOutFileName
     *      Path to file containing information about next pass training vectors
     *      for category PERSON eg ..../PERSON_TRAINSECOND PASS
     * @param orgOutFileName
     *      Path to file containing information about next pass training vectors
     *      for category ORGANIZATION eg ..../ORGANIZATION_TRAINSECOND PASS
     * @param dirToSave
     *      Directory to save files created
     * @return An array of two hashtables:<br>
     * (1)Hashtable[0]:The concatenation of the named entity category, "#" and the file name as key, and a string array containing
     * tokens (the actual list of tokens for this file and category / eg sureFPDec.put(txtFile.getName() + "_" + Constants.PERSON, personList);
     * <br>
     * (2)Hashtable[1]:The concatenation of the named entity category, "#", the file name, "#" and the position of the token in file as key,
     * and the cannonical possibillity assigned for this token and category (in String format).
     *
     * If any key of a token does not exist in a sureFPDec returned, then it is surelly "cat" by category's safe rules.
     */
    public Hashtable[] savePrevPassDecisionsLists(
            double margin, File sourceDir, String timexMatcherFileName,
            String periodSeparatorFileName, String perModelFileName,
            String orgModelFileName, File dirToSave)  {
        
        // for every file in active learning pool
        File pool[] = sourceDir.listFiles();
        
        // will be filled with sure desicions
        Hashtable<String, String[]> sureFPDec = new Hashtable<String, String[]>(2*pool.length, 0.9f);
        // will be filled with cannonical possibilities of all tokens
        Hashtable<String, String> allFPDec = new Hashtable<String, String>(2000*pool.length);
        
        try {
            
            // = new File(dirToSave, sourceDir.getName());
            dirToSave.mkdirs();
            
            // counters for vectors "cut" by safe rules" in SP. perYes++ when cat by safe rures
            int perYes = 0, perNo = 0;
            int orgYes = 0, orgNo = 0;
            
            TimexMatcher matcher = new TimexMatcher(timexMatcherFileName);
            SentenceSplitter separator = new SentenceSplitter(periodSeparatorFileName);
            
            ////////////////////////////////////////////////////////////////////////
            ////////////// prepering models for both categories ////////////////////
            svm_model perModel = svm.svm_load_model(perModelFileName);
            int per_nr_class = svm.svm_get_nr_class(perModel);
            int []perCategories = new int[per_nr_class];
            svm.svm_get_labels(perModel, perCategories);
            double[] per_prob_estimates = new double[per_nr_class];
            
            svm_model orgModel = svm.svm_load_model(orgModelFileName);
            int org_nr_class = svm.svm_get_nr_class(orgModel);
            int []orgCategories = new int[org_nr_class];
            svm.svm_get_labels(orgModel, orgCategories);
            double[] org_prob_estimates = new double[org_nr_class];
            ////////////// prepering models for both categories ////////////////////
            //====================================================================//
            
            int c_file = 0;// counter for files
            for (File curr: pool) {
                
                if (curr.isFile()) {
                    
                    c_file ++;
                    // Hashtables for filling temporarly single occurances (according to equality) of "sure" entity names
                    Hashtable<String, String> surePerList = new Hashtable<String, String>();
                    Hashtable<String, String> sureOrgList = new Hashtable<String, String>();
                    
                    // arrays with entity names found
                    String[] persons = new String[0];// just initializing
                    String[] organizations = new String[0];// just initializing
                    ////////////////////////////////////////////////////////////////
                    ///////////////////////// FP testing ///////////////////////////
                    //// for creating sure list decision of FP model over texts ////
                    ////////////////////////////////////////////////////////////////
                    
                    String perAttr[];
                    int perDistFromStartOfEntity = -1;
                    // temporary for creating list with previous pass classifier sure decisions
                    int perListNum = 0;
                    
                    String orgAttr[];
                    int orgDistFromStartOfEntity = -1;
                    // temporary for creating list with previous pass classifier sure decisions
                    int orgListNum = 0;
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////////// reading text ///////////////////////////
                    FileInputStream fis = new FileInputStream(curr);
                    byte []buff = new byte[ (int) curr.length()];
                    fis.read(buff);
                    fis.close();
                    String text = new String(buff);
                    /////////////////////// reading text ///////////////////////////
                    ////========================================================////
                    
                    FeaturesManager perFPVectors, orgFPVectors;
                    
                    
                    perFPVectors = new PersonFeaturesManager(curr.getName(), text, perListManager, matcher, separator);
                    orgFPVectors = new OrganizationFeaturesManager(curr.getName(), text, orgListManager, matcher, separator);
                    
                    
                    // for each token of current file's text
                    while ( (perAttr = perFPVectors.nextFeatVec(perDistFromStartOfEntity)) != null
                            && (orgAttr = orgFPVectors.nextFeatVec(orgDistFromStartOfEntity)) != null) {
                        
                        // answers and cannonical diastances from SP models of each category
                        double perAnswer, perCanDist, orgAnswer, orgCanDist;
                        
                        //////////////////////////////////////////////////////////////
                        /////////////////////////// person ///////////////////////////
                        int perNumOfAttributes = perAttr.length - 1 ;
                        svm_node perNodes[] = new svm_node[perNumOfAttributes];
                        
                        for (int i = 0; i < perNumOfAttributes; i++) {
                            
                            perNodes[i] = new svm_node();
                            perNodes[i].index = i+1;
                            perNodes[i].value = Auxiliary.atof(perAttr[i+1]);
                            
                        } // for
                        
                        // in case it is not surelly negative for person category because
                        // of person safe rules
                        if (perNumOfAttributes > 0) {
                            
                            // PREDICT
                            perAnswer = svm.svm_predict_probability(perModel, perNodes, per_prob_estimates);
                            if (perCategories[0] == -1) {
                                perCanDist = per_prob_estimates[1] - per_prob_estimates[0];
                            } // if
                            else {
                                perCanDist = per_prob_estimates[0] - per_prob_estimates[1];
                            } // else
                            
                        } // if
                        else{
                            
                            perAnswer = -1;
                            perCanDist = -1;
                            
                        }
                        
                        String perToken = perFPVectors.tokenAt(perFPVectors.getOffset() - 1).getToken().toLowerCase();
                        String key = Constants.PERSON + "#" + curr.getName() + "#" + (perFPVectors.getOffset() - 1);
                        allFPDec.put(key, String.valueOf(perCanDist));
                        
                        if ((perAnswer == 1) && (perDistFromStartOfEntity == -1)) {
                            
                            perDistFromStartOfEntity = 1;
                            String currTok = perFPVectors.tokenAt(perFPVectors.getOffset() - 1).getToken().toLowerCase();
                            if ((currTok.length() >= 3) && (perCanDist > margin) ) {
                                surePerList.put(currTok, currTok);
                            }
                            
                        } // if
                        else if (perAnswer == 1) {
                            
                            perDistFromStartOfEntity++;
                            String currTok = perFPVectors.tokenAt(perFPVectors.getOffset() - 1).getToken().toLowerCase();
                            if ((currTok.length() >= 3) && (perCanDist > margin) ) {
                                surePerList.put(currTok, currTok);
                            }
                            
                        } // else if
                        else {
                            perDistFromStartOfEntity = -1;
                        } // else if
                        
                        
                        ////////////////////////////////////////////////////////////////
                        /////////////////////////organization///////////////////////////
                        
                        int orgNumOfAttributes = orgAttr.length - 1;
                        svm_node orgNodes[] = new svm_node[orgNumOfAttributes];
                        
                        for (int i = 0; i < orgNumOfAttributes; i++) {
                            
                            orgNodes[i] = new svm_node();
                            orgNodes[i].index = i+1;
                            orgNodes[i].value = Auxiliary.atof(orgAttr[i+1]);
                            
                        } // for
                        // PREDICT
                        
                        // in case it is not surelly negative for organization category because
                        // of organization safe rules
                        if (orgNumOfAttributes > 0) {
                            
                            orgAnswer = svm.svm_predict_probability(orgModel, orgNodes, org_prob_estimates);
                            if (orgCategories[0] == -1) {
                                orgCanDist =
                                        org_prob_estimates[1] - org_prob_estimates[0];
                            } // if
                            else {
                                orgCanDist =
                                        org_prob_estimates[0] - org_prob_estimates[1];
                            } // else
                            
                        } // if
                        else{
                            orgAnswer = -1;
                            orgCanDist = -1;
                            
                        }
                        
                        String orgToken = orgFPVectors.tokenAt(orgFPVectors.getOffset() - 1).getToken().toLowerCase();
                        key = Constants.ORGANIZATION + "#" + curr.getName() + "#" + (orgFPVectors.getOffset() - 1);
                        allFPDec.put(key, String.valueOf(orgCanDist));
                        
                        if ((orgAnswer == 1) && (orgDistFromStartOfEntity == -1)) {
                            
                            orgDistFromStartOfEntity = 1;
                            String currTok = orgFPVectors.tokenAt(orgFPVectors.getOffset() - 1).getToken().toLowerCase();
                            if ((currTok.length() >= 3) && (orgCanDist > margin) ) {
                                sureOrgList.put(currTok, currTok);
                            }
                            
                        } // if
                        
                        else if (orgAnswer == 1) {
                            
                            orgDistFromStartOfEntity++;
                            String currTok = orgFPVectors.tokenAt(orgFPVectors.getOffset() - 1).getToken().toLowerCase();
                            if ((currTok.length() >= 3) && (orgCanDist > margin) ) {
                                sureOrgList.put(currTok, currTok);
                            }
                            
                        } // else if
                        else {
                            orgDistFromStartOfEntity = -1;
                        } // else if
                        
                    }// while ... all tokens of current text
                    
                    // Sorting FP list for current text
                    persons = (String[]) surePerList.values().toArray(persons);
                    Arrays.sort(persons);
                    
                    // Sorting FP list for current text
                    organizations = (String[]) sureOrgList.values().toArray(organizations);
                    Arrays.sort(organizations);
                    
                    sureFPDec.put(Constants.PERSON + "#" + curr.getName(), persons);
                    sureFPDec.put(Constants.ORGANIZATION + "#" + curr.getName(), organizations);
                    
                    /*
                    File perTxt = new File(dirToSave, Constants.PERSON + "#" + currFile.getName() + ".txt");
                    System.out.println("Saving lists in text file: \n" + perTxt.getCanonicalPath());
                    BufferedWriter fout = new BufferedWriter(
                            new FileWriter(perTxt));
                    for (String next : persons){
                     
                        fout.write(next + "\n");
                     
                    }
                    fout.close();
                     
                    File orgTxt = new File(dirToSave, Constants.ORGANIZATION + "#" + currFile.getName() + ".txt");
                    System.out.println("Saving lists in text file: \n" + orgTxt.getCanonicalPath());
                    fout = new BufferedWriter(
                            new FileWriter(orgTxt));
                    for (String next : organizations){
                     
                        fout.write(next + "\n");
                     
                    }
                    fout.close();
                     */
                    System.out.println(c_file + ": Finished processing file: " + curr.getName());
                    
                    // calling the garbage collector after proccessing 550 files
                    if(c_file % 550 == 0) System.gc();
                    
                }// if it's a file
                
            }// for each file of pool
            
            System.out.print("\n\n\n");
            File objectFile = new File(dirToSave, Constants.ACTIVE_LEARNING_FP_SURE_DECISIONS_FILENAME);
            System.out.println("Saving sure decision lists in hashtable object file:\n" +
                    objectFile.getCanonicalPath());
            FileOutputStream fout = new FileOutputStream(objectFile);
            ObjectOutputStream obStr = new ObjectOutputStream(fout);
            obStr.writeObject(sureFPDec);
            obStr.close();
            
            objectFile = new File(dirToSave, Constants.ACTIVE_LEARNING_FP_ALL_DECISION_LISTS_FILENAME);
            System.out.println("Saving all decision lists in hashtable object file:\n" +
                    objectFile.getCanonicalPath());
            fout = new FileOutputStream(objectFile);
            obStr = new ObjectOutputStream(fout);
            obStr.writeObject(allFPDec);
            obStr.close();
            
            File txtFile = new File(dirToSave, "All_Decisions.txt");
            System.out.println("Saving all decision lists in text file:\n" +
                    txtFile.getCanonicalPath());
            BufferedWriter bfout = new BufferedWriter(new FileWriter(txtFile));
            
            Enumeration<String> enumDec = allFPDec.keys();
            for (
                    String nextKey = enumDec.nextElement();
            enumDec.hasMoreElements();
            nextKey = enumDec.nextElement()){
                
                bfout.write(nextKey + ":" + (String)(allFPDec.get(nextKey)) + "\n");
                
            }
            bfout.close();
            
            txtFile = new File(dirToSave, "Sure_Decisions.txt");
            System.out.println("Saving sure decision lists in text file:\n" +
                    txtFile.getCanonicalPath());
            bfout = new BufferedWriter(new FileWriter(txtFile));
            
            enumDec = sureFPDec.keys();
            for (
                    String nextKey = enumDec.nextElement();
            enumDec.hasMoreElements();
            nextKey = enumDec.nextElement()){
                
                String[] names = (String[])(sureFPDec.get(nextKey));
                for (String name: names){
                    
                    bfout.write(nextKey + ":" + name + "\n");
                    
                }
            }
            
            bfout.close();
            
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        Hashtable toReturn[] = new Hashtable[2];
        toReturn[0] = sureFPDec;
        toReturn[1] = allFPDec;
        
        return toReturn;
        
    }
    
    public PrevPassDecionListsThread getPrevPassDecionListsThread(
            String timexMatcherFileNameIn,
            String periodSeparatorFileNameIn, String perModelFileNameIn,
            String orgModelFileNameIn){
        
        
        return new PrevPassDecionListsThread(
                -1,
                timexMatcherFileNameIn,
                periodSeparatorFileNameIn,
                perModelFileNameIn,
                orgModelFileNameIn);
        
    };
    
    protected Hashtable<String, ArrayList<InformationVector>> loadVectorsInfoFile(String fileName) {
        
        Hashtable<String, ArrayList<InformationVector>> usedVectors = new Hashtable<String,ArrayList<InformationVector>>();
        Set<String> keys = usedVectors.keySet();
        
        try {
            
            File f = new File(fileName);
            FileInputStream fis = new FileInputStream(f);
            byte[] buff = new byte[ (int) f.length()];
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text =
                    TextPreprocessor.removeNoisySpaces(
                    text);
            StringTokenizer strTok = new StringTokenizer(text, "\n");
            int c = 0;
            int line = 0;
            
            while (strTok.hasMoreTokens()) {
                
                String categ, feats, token, phrase, posit, fname;
                
                String catFeats = strTok.nextToken();
                categ = catFeats.substring(0, catFeats.indexOf("/t"));
                feats = catFeats.substring(catFeats.indexOf("/t") + 1);
                token = strTok.nextToken();
                phrase = strTok.nextToken();
                posit = strTok.nextToken();
                int phrpos = Integer.parseInt(strTok.nextToken());
                fname = strTok.nextToken(); // System.out.print((++c) + ": Used Vector filename: " + fname + ", @");
                int pos = Auxiliary.atoi(posit);// System.out.println("pos: " + pos );
                InformationVector temp = new InformationVector(fname, pos, phrpos, categ, feats, token);
                String key = fname;
                ArrayList<InformationVector> tempL;
                if(keys.contains(key)){
                    
                    tempL = usedVectors.get(key);
                    
                }else {
                    
                    tempL = new ArrayList<InformationVector>();
                    usedVectors.put(key, tempL);
                    
                }
                tempL.add(temp);
                
                
            } // while
            
            strTok = null;
        }catch (Exception e) {
            e.printStackTrace();
        } // catch
        System.out.println("Number of used vectors loaded: " + usedVectors.size());
        
        return usedVectors;
        
    } // loadUsedVectors
    
    
    
    
    public void featuresRecomputeSP(
            String infoVecsPath, File textsDirectory,
            String personModelFileNameFP, String listsPath,
            String personModelFileNameSP,
            String orgModelFileNameFP, String orgModelFileNameSP,
            TimexMatcher matcher,
            SentenceSplitter separator, PrintStream output){
        
        
        try {
            
            ListManager personListManagerFP = ListManager.initListManager(Constants.PERSON, Constants.FIRST_PASS, listsPath);
            
            ListManager personListManagerSP = ListManager.initListManager(Constants.PERSON, Constants.SECOND_PASS, listsPath);
            
            ListManager orgListManagerFP = ListManager.initListManager(Constants.ORGANIZATION, Constants.FIRST_PASS, listsPath);
            
            ListManager orgListManagerSP = ListManager.initListManager(Constants.ORGANIZATION, Constants.SECOND_PASS, listsPath);
            
            StringBuffer perResult = new StringBuffer();
            StringBuffer orgResult = new StringBuffer();
            // contains used vectors info . uses filename as key
            Hashtable<String, ArrayList<InformationVector>> htab = loadVectorsInfoFile(infoVecsPath);
            Set<String> keys = htab.keySet();
            
            long start = System.currentTimeMillis();
            output.println("Vectors Info file loaded. Recomputing features . . . ");
            
            
            File curr; //the current file under testing (-->see while loop)
            DirectoryManager txtDirMan = new DirectoryManager(textsDirectory.getCanonicalPath());
            
            
            ////////////////////////////////////////////////////////////////////////
            /////////////////////loading classifiers (models)///////////////////////
            svm_model perModelFP = svm.svm_load_model(personModelFileNameFP);
            svm_model orgModelFP = svm.svm_load_model(orgModelFileNameFP);
            //====================loading classifiers (models)======================
            //======================================================================
            
            // loading number of classies (in our case it is two : "1" and "-1")
            
            // for every file of test set
            while ( (curr = txtDirMan.getNextFile()) != null) {
                
                if (curr.isFile()&& keys.contains(curr.getName())) {
                    
                    // arrays where values for features are kept
                    // in string representation
                    String SPPerAttr[];
                    String SPOrgAttr[];
                    // distancies from start of entities found. If -1 then it is the first
                    // (could be the only one) token of a sequence of tokens belonging to
                    // the same entity such as "A. Papanikolas"
                    int distanceFromStartOfPer = -1;
                    int distanceFromStartOfOrg = -1;
                    
                    
                    ////////////////////////////////////////////////////////////////
                    /////////////////text retrieval from this file//////////////////
                    FileInputStream fis = new FileInputStream(curr);
                    byte []buff = new byte[(int)curr.length()];
                    fis.read(buff);
                    fis.close();
                    //context of retrieved text
                    String text = new String(buff);
                    //===============text retrieval from this file==================
                    //==============================================================
                    
                    
                    
                    // Second and First Pass feature vector managers for each category
                    PersonFeaturesManager perVecManagerSP, perVecManagerFP;
                    OrganizationFeaturesManager orgVecManagerSP, orgVecManagerFP;
                    
                    // PERSON
                    perVecManagerSP =
                            new PersonFeaturesManager(curr.getName(),
                            text, personListManagerSP, matcher, separator);
                    perVecManagerFP =
                            new PersonFeaturesManager(curr.getName(),
                            text, personListManagerFP, matcher, separator);
                    
                    // ORGANIZATION
                    orgVecManagerSP =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, orgListManagerSP, matcher, separator);
                    orgVecManagerFP =
                            new OrganizationFeaturesManager(curr.getName(),
                            text, orgListManagerFP, matcher, separator);
                    
                    // getting sure decisions list and all decisions cannonical posibilities from fp classifiers
                    Hashtable []FPDecisions =
                            Trainer.findFPDecisionsForSP(perVecManagerFP, perModelFP, orgVecManagerFP, orgModelFP, 0.85, curr.getName());
                    
                    Hashtable <String, String[]> sureDec = (Hashtable <String, String[]> ) FPDecisions[0];
                    Hashtable <String, String> allDec = (Hashtable <String, String> ) FPDecisions[1];
                    
                    
                    // for each token in text...
                    while (
                            ((SPPerAttr = perVecManagerSP.nextFeatVec(
                            distanceFromStartOfPer, sureDec, allDec, curr.getName()))!= null) &&
                            
                            ((SPOrgAttr = orgVecManagerSP.nextFeatVec(
                            distanceFromStartOfOrg, sureDec, allDec, curr.getName())) != null)
                            ) {
                        
                        ArrayList<InformationVector> infoVcs = htab.get(curr.getName()) ;
                        
                        for(InformationVector next: infoVcs){
                            
                            if (next.getPosition() == perVecManagerSP.getOffset()){
                                
                                String ans ;
                                //////////////////// person /////////////////////////////
                                if(next.getCategory().equalsIgnoreCase(Constants.PERSON)){
                                    ans = "1";
                                }else {
                                    ans = "-1";
                                }
                                perResult.append(ans);
                                perResult.append("\t");
                                for(String feat: SPPerAttr){
                                    
                                    perResult.append(feat);
                                    perResult.append("\t");
                                    
                                }
                                perResult.append("\n");
                                
                                
                                ////////////////// organization ////////////////////////
                                if(next.getCategory().equalsIgnoreCase(Constants.ORGANIZATION)){
                                    ans = "1";
                                }else {
                                    ans = "-1";
                                }
                                orgResult.append(ans);
                                orgResult.append("\t");
                                for(String feat: SPOrgAttr){
                                    
                                    orgResult.append(feat);
                                    orgResult.append("\t");
                                    
                                }
                                orgResult.append("\n");
                                
                                
                            }
                            
                        }
                        
                    } // if
                    
                }
                
            } // while--> for all files in testing set
            
            FileOutputStream out = new FileOutputStream("./trainRecompPer.txt");
            out.write(perResult.toString().getBytes());
            out.close();
            
            out = new FileOutputStream("./trainRecompOrg.txt");
            out.write(orgResult.toString().getBytes());
            out.close();
            
            output.println("DONE, TIME NEEDED: " + Constants.toTimeString(start));
            output.println("See results in directory: \n" +
                    "" + (new File("")).getCanonicalPath());
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
    
    public SPFeaturesRecomputorThread getFeaturesRecomputedThread(
            String infoVecsPath, File textsDirectory, String listspath,
            String personModelFileNameFP,
            String personModelFileNameSP,
            String orgModelFileNameFP, String orgModelFileNameSP,
            TimexMatcher matcher,
            SentenceSplitter separator, PrintStream output){
        
        return new SPFeaturesRecomputorThread(
                infoVecsPath, textsDirectory, listspath,
                personModelFileNameFP, personModelFileNameSP,
                orgModelFileNameFP, orgModelFileNameSP,
                matcher,  separator, output);
        
    }
    
    
    /************************* DEBUGING ONLY ******************************************
     * public static void main(String []args){
     *
     *
     * PrepareSP prepSP = new PrepareSP(Constants.FIRST_PASS, Constants.LISTS_DIRECTORY_PATH);
     * Runnable run = prepSP.getPrevPassDecionListsThread(
     * Constants.TIMEX_PATERNS_FILE_PATH, Constants.SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH,
     * Constants.FP_PERSON_CLASSIFIER_FILE_PATH, Constants.FP_ORGANIZATION_CLASSIFIER_FILE_PATH);
     *
     * Thread thr = new Thread(run);
     *
     * thr.start();
     *
     * }
     *
     **********************************************************************************/
    
    
    private class PrevPassDecionListsThread implements Runnable{
        
        private String timexMatcherFileName;
        private String periodSeparatorFileName;
        private String perModelFileName;
        private String orgModelFileName;
        private ListManager perListManager;
        private ListManager orgListManager;
        private double threshold;
        
        public PrevPassDecionListsThread(
                double theshold,
                String timexMatcherFileNameIn,
                String periodSeparatorFileNameIn, String perModelFileNameIn,
                String orgModelFileNameIn){
            
            this.threshold = theshold;
            timexMatcherFileName = timexMatcherFileNameIn;
            periodSeparatorFileName = periodSeparatorFileNameIn;
            perModelFileName = perModelFileNameIn;
            orgModelFileName = orgModelFileNameIn;
            perListManager = PrepareSP.this.perListManager;
            orgListManager = PrepareSP.this.orgListManager;
            
        }
        // does not ask user for threshold
        public PrevPassDecionListsThread(
                String timexMatcherFileNameIn,
                String periodSeparatorFileNameIn, String perModelFileNameIn,
                String orgModelFileNameIn, ListManager perListManagerIn,
                ListManager orgListManagerIn){
            
            this(
                    -1,
                    timexMatcherFileNameIn,
                    periodSeparatorFileNameIn,
                    perModelFileNameIn,
                    orgModelFileNameIn);
            
        }
        
        public void run(){
            
            File actLeFiles = new File(Constants.ACTIVE_LEARNING_POOL_OF_FILES_DIR_PATH);
            JFileChooser jCh = new JFileChooser(actLeFiles.getParentFile());
            jCh.setDialogTitle("Directoty of active learning training files");
            jCh.setFileSelectionMode(jCh.DIRECTORIES_ONLY);
            File dirToSave = null;
            if(jCh.showDialog(null, "Choose Directory") == jCh.APPROVE_OPTION){
                
                String thrld;
                if (threshold == -1){
                    
                    thrld = JOptionPane.showInputDialog(null, "Please enter threshold for sure decisions: ", 0.8);
                    if (thrld == null) return ;
                    
                }else thrld = String.valueOf(threshold);
                
                dirToSave = new File(Constants.ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR);
                
                savePrevPassDecisionsLists(
                        Double.parseDouble(thrld), jCh.getSelectedFile(),
                        timexMatcherFileName, periodSeparatorFileName,
                        perModelFileName, orgModelFileName, dirToSave);
                try {
                    
                    JOptionPane.showMessageDialog(null, "Decision lists ready. See results in path:\n" +
                            dirToSave.getCanonicalPath());
                } catch (HeadlessException ex) {
                    ex.printStackTrace();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
                
            }
            
            System.gc();
            
        }
        
    }
    
    
    public class SPFeaturesRecomputorThread implements Runnable{
        
        private String infoVecsPath;
        private File textsDirectory;
        private String pass;
        private String personModelFileNameFP;
        private ListManager personListManagerFP;
        private String personModelFileNameSP;
        private String listsPath;
        private String orgModelFileNameFP;
        private String orgModelFileNameSP;
        
        private TimexMatcher matcher;
        private SentenceSplitter separator;
        private PrintStream output;
        
        public SPFeaturesRecomputorThread(
                String infoVecsPath, File textsDirectory,
                String personModelFileNameFP, String listspath,
                String personModelFileNameSP,
                String orgModelFileNameFP, String orgModelFileNameSP,
                TimexMatcher matcher,
                SentenceSplitter separator, PrintStream output) {
            
            this.infoVecsPath = infoVecsPath;
            this.textsDirectory = textsDirectory;
            this.pass = pass;
            this.personModelFileNameFP = personModelFileNameFP;
            this.personModelFileNameSP = personModelFileNameSP;
            this.listsPath = listspath;
            this.orgModelFileNameFP = orgModelFileNameFP;
            this.orgModelFileNameSP = orgModelFileNameSP;
            this.matcher = matcher;
            this.separator = separator;
            this.output = output;
            
            
        }
        
        public void run(){
            
            PrepareSP.this.featuresRecomputeSP(
                    infoVecsPath, textsDirectory, listsPath,
                    personModelFileNameFP,
                    personModelFileNameSP,
                    orgModelFileNameFP, orgModelFileNameSP,
                    matcher,
                    separator, output);
            
        }}
    
}