/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import java.util.ArrayList;


public class EntropyNode {
    
    protected int total;
    protected int entity;
    protected int nonentity;
    protected double entropy;
    protected ArrayList<FeatureVector> featureVectors;
    
    public EntropyNode() {
        this.total = 0;
        this.entity = 0;
        this.nonentity = 0;
        this.featureVectors = new ArrayList<FeatureVector>();
    } // constructor: EntropyNode
    
    protected void add(FeatureVector vector, boolean is_entity) {
        total++;
        if (is_entity) {
            entity++;
            featureVectors.add(featureVectors.size(), vector);
        } // if
        else {
            nonentity++;
            featureVectors.add(0, vector);
        } // else
    } // add
    
    protected void computeEntropy() {
        double p1 = (double) entity / total;
        double p2 = (double) nonentity / total;
        if ((p1 != 0) && (p2 != 0)) {
            entropy = - (p1 * Math.log(p1) + p2 * Math.log(p2));
        } // if
        else {
            entropy = 0;
        } // else
    } // computeEntropy
    
} // end of class EntropyNode
