/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 *
 * @author Xen.Vassilakos
 */
public class InformationVector extends UsedVector {
  
    private String category;
    private ArrayList <String> features;
    private String token;
    private String pharse;
    private int posInPhrase;
    
    /**
     * Creates a new instance of InformationVector
     */
    public InformationVector(String fileName, int position, int posPhrs, String categ, String feats, String tok) {
       
        super(fileName, position);
        
        
        category = categ;
        token = tok;
        posInPhrase = posPhrs;
        ////////////////////////////////////////////////////////////////////////
        ////////////////// converting to separated features ////////////////////
        features = new ArrayList<String>();
        StringTokenizer tokenizer = new StringTokenizer(feats);
        
        while(tokenizer.hasMoreTokens()){
            
            String val = tokenizer.nextToken();
            val = val.substring(val.indexOf(':'));
            
            getFeatures().add(val);
        
        }
        
    }

    public String getPharse() {
        return pharse;
    }

    public String getToken() {
        return token;
    }

    public String getCategory() {
        return category;
    }

    public ArrayList<String> getFeatures() {
        return features;
    }
    
    public boolean checkToken(String tok, int pos){
    
        return this.getToken().equals(tok) && this.getPosition() == pos;
    
    }

    public int getPosInPhrase() {
        return posInPhrase;
    }
    
}
