/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import ipl.util.Constants;
import ipl.nerc.active_learning.data.AmbiguousVector;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 *
 * @author Owner
 */
public class PoolHandler {
    
    /** Not allowed to instanciate BatchHandler */
    private  PoolHandler() {
    }
    
    public final static String num_amb_vec_tag = "#number of  ambiguous vectors: " ;
    public final static String num_of_batch_tag = "#number of batch: ";
    public final static String created_on_tag = "#created on: ";
    /*public final static String file_tag = "@file = ";
    public final static String position_tag = "@position = ";
    public final static String canonical_distance_tag = "@canonical_distance = ";
     */
    public static final String batch_delimeter = "______________________________________________";
    
    /**
     *
     * @param batch
     *      The batch of AmbiguousVector objects to be saved .
     */
    public static void saveBatch(AmbiguousVector[] batch){
        
        saveBatch(batch, System.out);
        
    }
    
    /**
     *
     * @param batch
     *      The batch of AmbiguousVector objects to be saved .
     */
    public static void saveBatch(AmbiguousVector[] batch, PrintStream messagesOutput){
        
        long start = System.currentTimeMillis();
        
        // for reading former batches
        FileReader in;
        BufferedReader bin;
        
        // for writing batch nodes in text file
        FileWriter out;
        BufferedWriter bout;
        
        // for keeping date of saving batch
        java.util.Calendar calendar=java.util.Calendar.getInstance();
        Date date = calendar.getTime();
        
        try{
            
            // file where we keep number of the last saved batch
            File batchlog =
                    new File(Constants.ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH,"batches.log");
            // number of batch to be saved
            String number;
            if (!batchlog.exists())
                number="0";
            else{
                
                in = new FileReader(batchlog);
                bin = new BufferedReader(in);
                number= bin.readLine();
                bin.close();
                
            }
            
            
            File batchTxtFile =
                    new File(Constants.ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH,
                    number + "_" +
                    calendar.get(calendar.YEAR ) + "-"
                    + (calendar.get(calendar.MONTH)+1) + "-"
                    + (calendar.get(calendar.DAY_OF_MONTH)+1) +
                    ".bchtxt");
            
            // in case it is the first time we save a batch
            batchTxtFile.getParentFile().mkdirs();
            
            messagesOutput.println("Updating file: "+ batchlog);
            out = new FileWriter(batchlog);
            bout = new BufferedWriter(out);
            
            // fixing number string to be saved
            int num = Integer.parseInt(number);
            num++;
            
            number = ( (num<10) ?"0":"" ) + String.valueOf(num);
            // writing number of last saved batch in log file
            bout.write(number);
            bout.close();
            
            
            ////////////////////////////////////////////////////////////////////
            ////////////////////////saving to text file/////////////////////////
            
            // saving to text file
            messagesOutput.println("Updating file: "+ batchTxtFile);
            
            PrintStream appendOut = new PrintStream(new FileOutputStream(batchTxtFile));
            appendOut.append(batch_delimeter + "\n");
            appendOut.append(num_of_batch_tag + number + "\n");
            appendOut.append(num_amb_vec_tag + batch.length + "\n");
            
            int size = batch.length;
            for (int i=0;i<size;i++){
                
                
                StringBuffer features = new StringBuffer();
                String []feats = batch[i].getFeatureVector();
                for (int feat_i = 0; feat_i < feats.length; feat_i++){
                    
                    features.append(feats[feat_i]+",");
                    
                }
                appendOut.append(
                        batch[i].getFileName() + "|"
                        + batch[i].getPosition() + "|"
                        + batch[i].getMinDistanceHyperplane() + "|"
                        + feats.length + "|"
                        + features.toString() + "\n");
                
            }
            
            ////////////////////////////////////////////////////////////////////
            ////////////////////////////////////////////////////////////////////
            ////////////////////////////////////////////////////////////////////
            ///////////////// saving object of last batch //////////////////////
            messagesOutput.println("Updating file: "+ batchTxtFile);
            
            File batchObj =
                    new File(Constants.ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH,
                    number + "_" +
                    calendar.get(calendar.YEAR ) + "-"
                    + (calendar.get(calendar.MONTH)+1) + "-"
                    + (calendar.get(calendar.DAY_OF_MONTH)+1) +
                    ".bchObj");
            
            FileOutputStream batchObjFileStr = new FileOutputStream(batchObj);
            ObjectOutputStream objOutStr = new ObjectOutputStream(batchObjFileStr);
            
            objOutStr.writeObject(batch);
            
            messagesOutput.println(
                    "batch No" + number + " saved.\n" +
                    "Time needed: " + Constants.toTimeString(start, System.currentTimeMillis()));
            
            appendOut.close();
            
            
        } catch(FileNotFoundException ex){
            
            ex.printStackTrace();
            System.exit(-1);
            messagesOutput.println("batch not saved: ");
            
        } catch(IOException ioex){
            
            ioex.printStackTrace();
            System.exit(-1);
            messagesOutput.println("batch not saved: ");
        }
        
        messagesOutput.println("batch saved: ");
        
        
    }// save batch
    
    
    
    
    public static ArrayList<AmbiguousVector[]> loadFormerObjectBatches(){
        
        ArrayList < AmbiguousVector[] > toReturn = new ArrayList < AmbiguousVector[] > ();
        
        // directory of objects with suffix ".bchObj"
        File directory = new File(Constants.ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH);
        File[] files = directory.listFiles(new java.io.FileFilter() {
            public boolean accept(File pathname) {
                
                return pathname.getName().endsWith(".bchObj");
                
            }
        }); 
        
        if(files == null){
            
            System.out.println("No former batches exist");
            return null;
        }
        
        try {
            
            ObjectInputStream objIn ;
            
            int numOfBatchesLeft = files.length -1;
            
            while(numOfBatchesLeft > -1){
                
                objIn = new ObjectInputStream(new FileInputStream(files[numOfBatchesLeft--] ));
                try {
                    toReturn.add((AmbiguousVector[])objIn.readObject());
                } catch (IOException ex) {
                    ex.printStackTrace();
                } catch (ClassNotFoundException ex) {
                    ex.printStackTrace();
                }
                
                objIn.close();
                
            }
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return toReturn;
        
    }// loadFormerObjectBatches();
    
    /**
     * Reads all former batches from file:
     * <p>
     * Constants.ACTIVE_LEARNING_FORMER_BATCHES_FILE_PATH+"\batches.txt" .
     *
     *
     * @return An array list containing batches (ArrayList<AmbiguousVector[]>)
     *      Caution! each ambiguous vector object contain information only
     *      about position , informativeness and file of each token
     */
    public static ArrayList<AmbiguousVector[]> loadFormerBatches(){
        
        return loadFormerBatches(System.out);
        
    }
    
    /**
     * Reads all former batches from file:
     * <p>
     * Constants.ACTIVE_LEARNING_FORMER_BATCHES_FILE_PATH+"\batches.txt" .
     *
     *
     * @return An array list containing batches (ArrayList<AmbiguousVector[]>)
     *      Caution! each ambiguous vector object contain information only
     *      about position , informativeness and file of each token
     */
    public static ArrayList<AmbiguousVector[]> loadFormerBatches(PrintStream outputStream){
        
        long start = System.currentTimeMillis();
        
        System.out.println("\nLoading Former batches . . . \n");
        if (outputStream != System.out) outputStream.println("Loading Former batches");
        
        ArrayList < AmbiguousVector[] >  batches =
                new ArrayList < AmbiguousVector[] >();
        
        FileReader in;
        BufferedReader  input;
        
        
        // directory of objects with suffix ".bchtxt"
        File directory = new File(Constants.ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH);
        File[] files = directory.listFiles(new java.io.FileFilter() {
            public boolean accept(File pathname) {
                
                return pathname.getName().endsWith(".bchtxt");
                
            }
        });
        
        if(files.length == 0){
            
            System.out.println("No former batches exist");
            return null;
        }
        
        try{
            int bchCount = 0;
            // for each text file of a batch
            for (int file_i = 0; file_i < files.length; file_i++){
                
                File nextBatchFile = files[file_i];
                
                // reading . . .
                outputStream.println("Reading from file: "+ nextBatchFile.getName());
                in = new FileReader(nextBatchFile);
                input = new BufferedReader(in);
                StringBuffer content = new StringBuffer();
                String line;
                StringTokenizer tok;
                //==============================================================
                
                
                // separating batches and putting them in arraylist batches
                line = input.readLine();
                while(line!=null && !line.equals(batch_delimeter)){
                    
                    input.readLine();
                    
                    // HOW MANY VECTORS IN BATCH
                    int batchLength;
                    int i_tmp=0;
                    
                    tok = new StringTokenizer(input.readLine());
                    String batchLengthNum = tok.nextToken(num_amb_vec_tag);
                    
                    batchLength = Integer.parseInt(batchLengthNum);
                    bchCount++;
                    outputStream.println(bchCount+": One more former batch read. size = " + batchLength);
                    
                    AmbiguousVector[] tmp = new AmbiguousVector[100];//batchLength];
                    line = input.readLine();
                    if(line!=null) do{
                        
                        tok = new StringTokenizer(line,"|");
                        
                        // read file of next vector
                        String file = tok.nextToken();
                        System.out.println("file " + file+"&");
                        
                        // read position in file of next vector
                        String position = tok.nextToken();
                        System.out.println("position " + position+"&");
                        int pos = Integer.parseInt(position);
                        
                        // read position in file of next vector
                        String canDist = tok.nextToken();
                        // informativeness (distance from svm's hyperplane) of ambiguous vector's token in file
                        double informativeness =
                                Double.parseDouble(canDist);
                        System.out.println("canDist " + canDist +"&");
                        
                        // read number of features
                        String feats = tok.nextToken();
                        int featsNum = Integer.parseInt(feats);
                        System.out.println("number of features " + featsNum+"&");
                        
                        String []features = new String[featsNum];
                        String featStr = tok.nextToken();
                        tok = new StringTokenizer(featStr,",");
                        for(int feat_i= 0; feat_i < featsNum; feat_i++){
                            
                            features[feat_i] = tok.nextToken();
                            
                        }
                        
                        // create ambiguous vector node
                        AmbiguousVector ambVec =
                                new AmbiguousVector(file,pos,informativeness,features);
                        
                        tmp[i_tmp++] = ambVec;
                        
                    }// do
                    while((line = input.readLine())!=null);
                    
                    // add temporary batch to list to be returned
                    batches.add(tmp);
                    
                }// while
                
                input.close();
                
            }// for file_i
            
        }catch(FileNotFoundException ex){
            
            ex.printStackTrace();
            System.exit(-1);
            
        } catch(IOException ioex){
            
            ioex.printStackTrace();
            System.exit(-1);
            
        }
        
        System.out.println("batches loaded. Time needed: "+ Constants.toTimeString(start,System.currentTimeMillis()));
        
        return batches;
        
    }// loading former batches
    
    /**
     * Sortes ambiguous vectors of pool according to informativeness measure.
     * It performs ascending sorting according to the distance of each ambiguous
     * vector from the svm hyperplane. The absolute value of the cannonical
     * possibility assigned by svm, is used to represent distance from hyperplane.
     * <p>
     * This method also readjusts batchsize in case pool is smaller than batchSize
     * <p>
     * A convenient method  to be used for active learning experiments
     * that use informativeness measure as a step in computing final batch .
     *
     *
     * @param pool
     *      Pool of ambiguous vectors
     * @param batchSize
     *      A raper object of an integer indicating batchSize,
     *      in order to be readjusted
     * @return An array containing sorted by diastance AmbiguousVector objects
     */
    static public  AmbiguousVector[] ascDistSortedArray(ArrayList<AmbiguousVector> pool){
        
        AmbiguousVector tmp[] = new AmbiguousVector[pool.size()] ;
        tmp = pool.toArray(tmp);
        
        // performs ascending sorting.
        Arrays.sort(tmp , AmbiguousVector.distComparator());// comparator for ambiguous nodes
        
        // counter for vectors computed by not appling safe rules for one of two categories
        int countNoSafe = 0;
        for (int i=0; i<tmp.length;i++) {
            
            if(!tmp[i].isPersonSafeRulesApplied()||!tmp[i].isOrganizationSafeRulesApplied()){
        
                countNoSafe++;
                /*System.out.print("i = " + i);
                System.out.println(
                        ":" + tmp[i].getDistanceHyperplane() + "\n safeRules Applied: a)Person: "
                        + tmp[i].isPersonSafeRulesApplied()
                        + "b)Organization: "+ tmp[i].isOrganizationSafeRulesApplied());
              */  
            }
            
                
            
        }
        
        System.out.println("Number of vectors without \"Safe rules\": " + countNoSafe + "\n");
        
        return tmp;
        
    }
    
    
    
    
    public static ArrayList<AmbiguousVector> loadSamplePool(
            int howMany, ArrayList<AmbiguousVector> pool, PrintStream output)
            throws IOException, FileNotFoundException{
        
        // a clone to use
        ArrayList<AmbiguousVector> poolClone = (ArrayList<AmbiguousVector>) pool.clone();
        // readjusting howmany (if neccesary)
        howMany = howMany < poolClone.size()? howMany: (int)(poolClone.size() * 0.1);
        
        // sample pool to be returned
        ArrayList<AmbiguousVector> toReturn = new ArrayList<AmbiguousVector>();
        
        // directory to save sample sub pool
        File dir = new File(Constants.ACTIVE_LEARNING_SAMPLE_POOL_DIR_PATH);
        dir.mkdirs();
        // file to read (and save if not exists yet) sample pool
        File objFile = new File(dir,"samplePool.obj");
        ObjectInputStream objStr ;
        
        // if exists then read it and return it
        if(objFile.exists()){
            
            FileInputStream toRead = new FileInputStream(objFile);
            objStr = new ObjectInputStream(toRead);
            
            output.println("Prexisting sample pool loaded");
            output.println("File loaded:\n" + objFile.getCanonicalPath());
            
            try {
                
                toReturn = (ArrayList<AmbiguousVector>)objStr.readObject();
                objStr.close();
                
            } catch (ClassNotFoundException ex) {
                
                ex.getCause();
                ex.printStackTrace();
                
            } catch (IOException ex) {
                
                ex.getCause();
                ex.printStackTrace();
            }
            
            return toReturn;
            
        }// if
        // else create it and return it
        else{
            
            for(int i=0; i < howMany; i++){
                
                int randIndex = (int)(Math.random() * poolClone.size());
                toReturn.add(poolClone.remove(randIndex));
                
                
            }
            
            ObjectOutputStream objout =
                    new ObjectOutputStream(
                    new FileOutputStream(objFile)
                    );
            
            objout.writeObject(toReturn);
            
            output.println("New Sample pool created and saved");
            output.println("File saved:\n" + objFile.getCanonicalPath());
            
            objout.close();
            
            return toReturn;
            
        }// else
        
        
        
        
    }
    
    /**
     * Sortes ascendingly all ambiguous vectors in pool according to:
     * <p>*informativeness+(1-)*representativeness value .
     *
     * A convenient method  to be used for active learning experiments
     * that use informativeness measure as a step in computing final batch .
     *
     * @return An array containing sorted (as described above) AmbiguousVector nodes
     *
     */
    public static AmbiguousVector.InfoRepresentScoreNode[] informRepresSortedArray(
            int perFeat, int orgFeat, ArrayList<AmbiguousVector> pool,
            int batchSize) {
        
        // array of nodes containing informativeness, representativeness
        // and a ballanced score of the above two measures, for each AmbiguousVector
        AmbiguousVector.InfoRepresentScoreNode infRep[] =
                new AmbiguousVector.InfoRepresentScoreNode[pool.size()];
        
        int poolSize = pool.size();
        // position in array to be returned
        int pos =0;
        // filling array with nodes that keep the informativeness and
        // representativeness values for each ambiguous vector in the pool
        
        // a sample subpool for computing representativeness
        ArrayList<AmbiguousVector> samplePool = new ArrayList<AmbiguousVector>();
        try {
            
            samplePool = loadSamplePool(1000, pool, System.out);
            
        } catch (IOException ex) {
            
            ex.printStackTrace();
            
        }
        
        for (int poolPos = 0; poolPos < poolSize; poolPos++){
            
            AmbiguousVector nextAmbVec = pool.get(poolPos);
            
            ////////////////////////////////////////////////////////////////////
            //////// filling sub pool with 1000 vectors nearest to//////////////
            //////// vector @ poolPos, in order to compute representativeness///
            //////// in a faster way than using the whole pool//////////////////
            
            infRep[pos++] =
                    nextAmbVec.infoReprNode(
                    nextAmbVec.representativeness(samplePool, perFeat, orgFeat ) );
            
            
        }
        
        // ascending sorting - the best (informativeness-representativeness) scores
        // are at the bottom of the array
        Arrays.sort(infRep);
        batchSize = Math.min(infRep.length,batchSize);
        return infRep;
        
    }
    
    public static int[] getFeatureNums(String passFlag, String category){
        
        int perFeatNum = 0, orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{// second pass
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        int[] ret = new int[2];
        ret[0] = perFeatNum;
        ret[1] = orgFeatNum;
        // return new int[]{perFeatNum,orgFeatNum};
        
        return ret;
        
    }
    
    
    /**
     * Calculates the average cosine similarity between vectors in pool.
     * (uses System.out (default output) as an output for messages)
     *
     * @param pool
     *          The pool containing SimilarityVector Objects
     * @param FileToSave
     *          A File object implementing the file where results will be saved
     * @param Category
     *          One of PERSON, ORGANIZATION, PERSON_ORGANIZATION
     *          (@see Constants for more)
     * @param passFlag
     *          A flag indicating which pass is it (FIRST or SECOND
     *          @see Constants for more)
     *
     * @return
     *          An Array of double values: pos0:sum, pos1:size, pos2:avg
     */
    static public  double[] avgSimilarity(
            ArrayList<SimilarityVector> pool,File FileToSave,
            String category, String passFlag){
        
        return avgSimilarity(pool, FileToSave,category,passFlag,System.out);
        
        
    }
    
    /**
     * Calculates the average cosine similarity between vectors in pool.
     *
     *
     * @param pool
     *          The pool containing SimilarityVector Objects
     * @param FileToSave
     *          A File object implementing the file where results will be saved
     * @param Category
     *          One of PERSON, ORGANIZATION, PERSON_ORGANIZATION
     *          (@see Constants for more)
     * @param passFlag
     *          A flag indicating which pass is it (FIRST or SECOND
     *          @see Constants for more)
     * @param output
     *          A PrintStream object to use as an output
     *
     * @return
     *          An Array of double values: pos0:sum, pos1:size, pos2:avg
     */
    static public  double[] avgSimilarity(
            ArrayList<SimilarityVector> pool,File FileToSave,
            String category, String passFlag, PrintStream output){
        
        output.println("size of pool -> " + pool.size());
        
        FileToSave = new File(FileToSave + ".avgSimilarity");
        FileToSave.getParentFile().mkdirs();
        /*
        int[] featNums = AmbiguousVector.getFeatureNums(category, passFlag);
        int perFeatNum = featNums[0] , orgFeatNum = featNums[1] ;
         */
        int perFeatNum = 0,orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{// second pass
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        Iterator<SimilarityVector> iter =  pool.iterator();
        
        double sum = 0.0;
        int j = 1;// indexes all vectors after current (nextAmb) in pool
        int size = pool.size();
        
        // for every SimilarityVector in pool
        while(iter.hasNext()){
            
            SimilarityVector nextAmb = iter.next();
            // add to sum nextAmb's similarity to the other SimilarityVectors in pool
            // (without adding its similarity that is allready computed and added -> these
            // SimilarityVecors are the ones before nextAmb in the pool)
            
            for(int i = j; i < size; i++){
                sum += nextAmb.cosineSimillarityTo(pool.get(i), perFeatNum, orgFeatNum);
            }
            j++;
            
        }
        // suposing pool.size() = n, we have added to sum, a number of cosine
        // similarities equale to:
        // (n-1)+(n-2)+...+2+1 = [(n-1)/2]*[(n-1) + 1] = [(n-1)*n]/2 = n^2/2 - n/2
        double avg = 2*sum / (Math.pow(size,2) - size);
        
        try {
            
            BufferedWriter fout =
                    new BufferedWriter(
                    new FileWriter(FileToSave));
            fout.write("summuary of similarities = " + sum+ "\n");
            fout.write("size = " + size + "\n");
            fout.write("avg = " + String.valueOf(avg));
            fout.close();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        double[] toReturn = {sum, size, avg};
        return toReturn;
        
    }
    
    public static void main(String args[])throws IOException{
        
        // loadFormerBatches(System.out);
        
        
    }
    
}
