/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.active_learning.data;

import ipl.util.Constants;
import ipl.util.Auxiliary;
import java.io.Serializable;
import java.util.*;

/**
 * A feature vector which is ambiguous because the svm classifier is less sure
 * about classifing it. Objects of this class are used for filling batches in
 * active learning experiments for improoving svm classifier model.
 */
public class SimilarityVector implements Serializable  {
    
    /**
     * Values of features of this ambiguous feature vector.
     * Althought the type of those values is double precision, the values
     * are stored in an array in String format.
     */
    protected String []featureVector;
   
    
   
    /**
     * Constructor:
     * 
     * @param featureVector 
     *      An array of feature values in String format. 
     *
     * 
     */
    public SimilarityVector(
    String []featureVector) {
        
        this.featureVector = featureVector;
        
    } // constructor: AmbiguousVector
    
   
    /**
     * @return         
     *    
     */
    public String[] getFeatureVector() {
        
        return featureVector;
        
    } // getFeatureVector

    /**
     * The representativeness of this ambiguousVectoreNode is defind to its density.
     * This method computes and returns the density of this AmbiguousVetorNode,
     * given a pool of ambiguousVectorNodes. Its density is equal to the
     * average simillarity of this and all the other AmbiguousVector Objaects
     * in pool.
     */
    public double representativeness(
            ArrayList<SimilarityVector> pool, String category,String passFlag)
            throws Exception{
                     
        double sum = 0.0;
        
                int perFeatNum = 0, orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{// second pass
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        for (int ipos=0;ipos<pool.size();ipos++){
            SimilarityVector next = pool.get(ipos);
            
            if(next != this)
                sum = sum + cosineSimillarityTo(next,perFeatNum, orgFeatNum); 
            
        }
        
        return sum/(pool.size()-1);
        
        
        
    }
    
     /**
     *
     *
     * @return The cosine similarity of this Ambiguous Vector Node X to an other
     * Ambiguous Vector Node  Y .
     *
     *          Cos(x,y) = <x,y>/||x||*||y||
     *
     *          where x = (x1,,xn), y=(y1,,yn)
     *          inner product of x and y: <x,y> = x1*y1++xn*yn
     *          ||x|| = sqrt(<x,x>)
     *
     */
    public double cosineSimillarityTo(SimilarityVector y,String category, String passFlag){
        
        String []featureVector_X = this.featureVector;
        String []featureVector_Y = y.featureVector;
        
        int perFeatNum = 0, orgFeatNum = 0;
        if (passFlag.equalsIgnoreCase(Constants.FIRST_PASS)){
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_FP;
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_FP;
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_FP;
            
            
        } else{
            
            if (category.equalsIgnoreCase(Constants.PER_ORG)){
                perFeatNum = Constants.PER_FEAT_NUM_SP;
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            }
            
            else if (category.equalsIgnoreCase(Constants.PERSON))
                perFeatNum = Constants.PER_FEAT_NUM_SP;
            
            
            
            else if (category.equalsIgnoreCase(Constants.ORGANIZATION))
                orgFeatNum = Constants.ORG_FEAT_NUM_SP;
            
        }
        
        return cosineSimillarityTo(y,perFeatNum,orgFeatNum);
        
    }
    
    
     
    
    
    /**
     *
     *
     * @return The cosine similarity of this Ambiguous Vector Node X to an other
     * Ambiguous Vector Node  Y .
     *
     *          Cos(x,y) = <x,y>/||x||*||y||
     *
     *          where x = (x1,,xn), y=(y1,,yn)
     *          inner product of x and y: <x,y> = x1*y1++xn*yn
     *          ||x|| = sqrt(<x,x>)
     *
     */
    public double cosineSimillarityTo(
            SimilarityVector y, int perFeatNum, int orgFeatNum){
        
        String []featureVector_X = this.featureVector;
        String []featureVector_Y = y.featureVector;
        
        /////////////////////////////////////////////////////////////////////////
        ///////////////////////<x,y>=x1*y1++xn*yn//////////////////////////////
        // <x,y> =
        double innerProd =
                innerProduct(featureVector_X,featureVector_Y,perFeatNum,orgFeatNum);
        // <x,y>/||x||        where ||x|| = sqrt(<x,x>
        innerProd /=
                Math.sqrt(
                innerProduct(featureVector_X,featureVector_X,
                perFeatNum,orgFeatNum));
        // <x,y>/(||x||*||y||)        where ||y|| = sqrt(<y,y>
        innerProd /=
                Math.sqrt(
                innerProduct(featureVector_Y,featureVector_Y,
                perFeatNum,orgFeatNum));
        
        
        return innerProd;
        
        
    }
    
    
    /**
     * computes and returns the inner product of feature vectors X and Y
     * <x,y> =x1*y1++xn*yn.
     * Features are converted to double values from String representation
     */
    private double innerProduct(
            String []featureVector_X ,String []featureVector_Y,
            int perFeatNum,int orgFeatNum ){
        
        double innerProd = 0.0;
        for (int i = 1;i < perFeatNum + 1; i++){
            // skiping "PERSON" at featureVector[0]
            innerProd += Auxiliary.atof(featureVector_X[i])*Auxiliary.atof(featureVector_Y[i]);
        }
        
        for (int i = perFeatNum + 2; i < orgFeatNum + perFeatNum + 2; i++){
            // skiping "ORGANIZATION" at featureVector[perFeatNum + 1]
            innerProd += Auxiliary.atof(featureVector_X[i])*Auxiliary.atof(featureVector_Y[i]);
        }
        
        return innerProd;
        
    }
    
    
    
  
    /**
     * @return The String representation of this object
     */
    public String toString(){
        
        //features: from array to string
        StringBuffer features=new StringBuffer("");
        for(int ipos=1;ipos<featureVector.length;ipos++)
            features.append(" " + ipos + ":" + featureVector[ipos]);
        
        String representation=
                "AmbiguousVector=" +
                "{" +
                " (feature vector="+features.toString()+"), "+
              
                "}";
        
        return representation;
        
    }
   
    
} // end of AmbiguousVector

