/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.lists;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Enumeration;

import ipl.nerc.preprocessing.TextPreprocessor;
import ipl.splitter.SentenceSplitter;
import ipl.util.Constants;

public class ListManager {
    
   /*
    * stop words
    */
    private Hashtable<String, String> safeList;
    // 6 lists (hashtable arrays) for person repeated words
    // string= lexi , komvos (key xana lexi)
    private Hashtable<String, ListNode>[] listsOfPerRepWords;
    private Hashtable<String, ListNode>[] listsOfOrgRepWords;
    // kathe thesi se kathe lista , avg emfanisewn lexewn ana list
    private int[] avgOfPerRepWordsLists;
    private int[] avgOfOrgRepWordsLists;
    // lista me baftistika onomata proswpwn
    private String[] listOfPersonNames;
    
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
    
    public ListManager(String safeListFileName, String[] listsOfPerFreqTokens,
            String[] listsOfOrgFreqTokens, String personNamesFN) {
        if (safeListFileName != null) {
            safeList = loadSafeList(new File(safeListFileName));
        } // if
        
        if (listsOfPerFreqTokens != null) {
            
            for(String listPath : listsOfPerFreqTokens){
                
                File list = new File(listPath);
                if(!list.exists()){
                    
                    list.getParentFile().mkdirs();
                    try {
                        list.createNewFile();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                    
                }// if
                
            }// for
            
            int num = listsOfPerFreqTokens.length;
            listsOfPerRepWords = new <String, HashNode>Hashtable[num];
            for (int i = 0; i < num; i++) {
                listsOfPerRepWords[i] = loadFrequencyList(new File(listsOfPerFreqTokens[i]));
            } // for
            
            avgOfPerRepWordsLists = new int[num];
            for (int i = 0; i < num; i++) {
                avgOfPerRepWordsLists[i] = getAvg(listsOfPerRepWords[i]);
            } // for
        } // if
        
        if (listsOfOrgFreqTokens != null) {
            
            for(String listPath : listsOfOrgFreqTokens){
                
                File list = new File(listPath);
                if(!list.exists()){
                    
                    list.getParentFile().mkdirs();
                    try {
                        list.createNewFile();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                    
                }// if
                
            }// for
            
            int num = listsOfOrgFreqTokens.length;
            listsOfOrgRepWords = new <String, HashNode>Hashtable[num];
            for (int i = 0; i < num; i++) {
                listsOfOrgRepWords[i] = loadFrequencyList(new File(listsOfOrgFreqTokens[i]));
            } // for
            
            avgOfOrgRepWordsLists = new int[num];
            for (int i = 0; i < num; i++) {
                avgOfOrgRepWordsLists[i] = getAvg(listsOfOrgRepWords[i]);
            } // for
            
        } // if
        
        if (personNamesFN != null) {
            listOfPersonNames = loadListOfPerNames(new File(personNamesFN));
        } // if
        
    } // constructor: ListManager
    
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
    
    private Hashtable<String, String> loadSafeList(File file) {
        
        Hashtable<String, String> tmp = new Hashtable<String, String>();
        
        try {
            FileInputStream fis = new FileInputStream(file.getAbsolutePath());
            byte[] buff = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text = TextPreprocessor.removeNoisySpaces(text);
            StringTokenizer strTok = new StringTokenizer(text, "\n");
            while (strTok.hasMoreTokens()) {
                String tok = strTok.nextToken();
                tmp.put(tok, tok);
            } // while
            strTok = null;
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        return tmp;
    } // loadSafeList
    
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
    
    private Hashtable<String, ListNode> loadFrequencyList(File file) {
        Hashtable<String, ListNode> tmp = new Hashtable<String, ListNode>();
        
        try {
            
            FileInputStream fis = new FileInputStream(file.getAbsolutePath());
            byte[] buff = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text = TextPreprocessor.removeNoisySpaces(text);
            StringTokenizer strTok = new StringTokenizer(text, " \n");
            while (strTok.hasMoreTokens()) {
                String tok = strTok.nextToken();
                tmp.put(tok, new ListNode(tok, Integer.parseInt(strTok.nextToken())));
            } // while
            strTok = null;
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        return tmp;
    } // loadFrequencyList
    
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
    
    private int getAvg(Hashtable<String, ListNode> ht) {
        if (ht.size() == 0) {
            return 0;
        } // if
        Enumeration<ListNode> enumeration = ht.elements();
        int count = 0;
        while (enumeration.hasMoreElements()) {
            count += enumeration.nextElement().getCount();
        } // while
        
        return ((int) count / ht.size());
    } // getAvg
    
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
    
    private String[] loadListOfPerNames(File file) {
        String []tmp;
        String text = "";
        try {
            FileInputStream fis = new FileInputStream(file.getAbsolutePath());
            byte[] buff = new byte[(int) file.length()];
            fis.read(buff);
            fis.close();
            text = new String(buff);
            text = TextPreprocessor.removeNoisySpaces(text);
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        StringTokenizer strTok = new StringTokenizer(text, "\n");
        int listLength = strTok.countTokens();
        tmp = new String[listLength];
        for (int i = 0; i < listLength; i++) {
            tmp[i] = strTok.nextToken();
        } // for
        strTok = null;
        return tmp;
    } // loadListOfPerNames
    
    //////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////
    
    public Hashtable<String, String> getSafeList() {
        return safeList;
    } // getSafeList
    
    public Hashtable<String, ListNode> getRepeatedWordsList(int offset, String category) {
        if (category.compareTo(Constants.PERSON) == 0) {
            return listsOfPerRepWords[offset];
        } else {
            return listsOfOrgRepWords[offset];
        }
    } // getRepeatedWordsList
    
    public int getAvgOfRepeatedWordsList(int offset, String category) {
        if (category.compareTo(Constants.PERSON) == 0) {
            return avgOfPerRepWordsLists[offset];
        } else {
            return avgOfOrgRepWordsLists[offset];
        }
    } // getAvgOfRepeatedWordsList
    
    public String[] getListOfPersonNames() {
        return listOfPersonNames;
    } // getListOfPersonNames
    
    //////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////
    
    public static ListManager initListManager(String category, File[] trainFiles, String passFlag,
            SentenceSplitter period_separator,
            String outputDirName) {
        String tagType;
        ListManager listManager;
        
        // PERSON
        String perlists[] = new String[6];
        tagType = Constants.PERSON;
        perlists[0] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 5, 20, 7, period_separator, outputDirName);
        perlists[1] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 5, 20, 1, period_separator, outputDirName);
        perlists[2] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 3, 4, 7, period_separator, outputDirName);
        perlists[3] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 3, 4, 1, period_separator, outputDirName);
        perlists[4] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 1, 2, 7, period_separator, outputDirName);
        perlists[5] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 1, 2, 1, period_separator, outputDirName);
        
        // ORGANIZATION
        String orglists[] = new String[6];
        tagType = Constants.ORGANIZATION;
        orglists[0] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 5, 20, 7, period_separator, outputDirName);
        orglists[1] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 5, 20, 1, period_separator, outputDirName);
        orglists[2] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 3, 4, 7, period_separator, outputDirName);
        orglists[3] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 3, 4, 1, period_separator, outputDirName);
        orglists[4] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 10, 1, 2, 7, period_separator, outputDirName);
        orglists[5] = ListCreator.getPreviousTokens(trainFiles, tagType, passFlag, 3, 1, 2, 1, period_separator, outputDirName);
        
        if (category.compareTo(Constants.PERSON) == 0) {
            listManager = new ListManager(outputDirName + "/PersonSafeList.txt",
                    perlists, orglists,
                    outputDirName + "/greekNames.txt");
        } else {
            listManager = new ListManager(outputDirName + "/OrganizationSafeList.txt",
                    perlists, orglists,
                    outputDirName + "/greekNames.txt");
        }
        
        return listManager;
    } // initListManager
    
    public static ListManager initListManager(String category, String passFlag, String path) {
        ListManager listManager;
        
        // PERSON
        String perlists[] = new String[6];
        perlists[0] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=10,minWordLen=5,maxWordLen=20,numPrevWords=7.txt";
        perlists[1] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=3,minWordLen=5,maxWordLen=20,numPrevWords=1.txt";
        perlists[2] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=10,minWordLen=3,maxWordLen=4,numPrevWords=7.txt";
        perlists[3] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=3,minWordLen=3,maxWordLen=4,numPrevWords=1.txt";
        perlists[4] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=10,minWordLen=1,maxWordLen=2,numPrevWords=7.txt";
        perlists[5] = path + "/" + passFlag + "/" + Constants.PERSON + "/lessOcc=3,minWordLen=1,maxWordLen=2,numPrevWords=1.txt";
        
        // ORGANIZATION
        String orglists[] = new String[6];
        orglists[0] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=10,minWordLen=5,maxWordLen=20,numPrevWords=7.txt";
        orglists[1] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=3,minWordLen=5,maxWordLen=20,numPrevWords=1.txt";
        orglists[2] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=10,minWordLen=3,maxWordLen=4,numPrevWords=7.txt";
        orglists[3] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=3,minWordLen=3,maxWordLen=4,numPrevWords=1.txt";
        orglists[4] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=10,minWordLen=1,maxWordLen=2,numPrevWords=7.txt";
        orglists[5] = path + "/" + passFlag + "/" + Constants.ORGANIZATION + "/lessOcc=3,minWordLen=1,maxWordLen=2,numPrevWords=1.txt";
        
        if (category.compareTo(Constants.PERSON) == 0) {
            listManager = new ListManager(Constants.LISTS_DIRECTORY_PATH + "/PersonSafeList.txt",
                    perlists, orglists, Constants.LISTS_DIRECTORY_PATH + "/greekNames.txt");
        } else {
            listManager = new ListManager(Constants.LISTS_DIRECTORY_PATH + "/OrganizationSafeList.txt",
                    perlists, orglists, Constants.LISTS_DIRECTORY_PATH + "/greekNames.txt");
            
        }
        
        return listManager;
        
    } // initListManager
    
} // end of class ListManager
