/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.preprocessing;

import ipl.util.Constants;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;

/**
 *
 * @author p3020021
 */
public class ResultFilesProcessor {
    
    /** Creates a new instance of ResultsProcessor */
    public ResultFilesProcessor() {
    }
    
    
    /**
     * This method processes the result files of a category ,which are saved in "path".
     * By result file, we mean each file saved after executing an experiment,
     * containing the confusion matrix(es) printed after finishing the experiment.
     * Supposinng the category chosen is PERSON,wWhen processing ends, resuts are
     * gathered in two files:
     *      [1].txt has the F-measure, the precision and the recall results for
     *          tokens that the svm categorised as PERSON
     *      [-1].txt has the F-measure, the precision and the recall results for
     *          tokens that the svm did not categorise as PERSON
     * In case a result file is not formed as expected, it is just ignored and the
     * user is informed for the number of not processed files. The context of the
     * created files can be copied and pasted to a spreadsheet such as an EXCEL
     * spreadsheet.
     *
     * @param cateName
     *          The name of the category's results to be processed.
     * @param categNum
     *          Witch one of the categories tested to process .(starts from 1 not 0)
     *          This number refers to the series of results printed in result files.
     *          For example in multiple categories per_org, we print firstly the results for
     *          PERSON. So by setting this parameter to 1, you get results for PERSON.
     *          Setting it to 2, processes results for organizations.
     * @param separLine
     *          A String indicating the starting substring of the line that is
     *          printed before each experiment's confusion matrix(es)
     *          This line (or lines) also separates the (possibly more
     *          than just one in case of parallel categories testing)
     *          confusion matrix(es) printed in each file.
     * @param saveTo
     *          path to the directory to save processed results
     */
    public  void extractResults(String path, String category, String saveTo)
    throws IOException{
        
        // counter for not processed files
        int counter = 0;
        
        // DIRECTORY CONTAINING RESULT FILES
        File dir = new File(path);
        
        // WRITE THE RESULTS IN THE SAME DIRECTORY AS INDICATED BY PARAMETER "PATH"
        // NEGATIVE ERSULTS
        File storeFile = new File(saveTo, category+"[-1].txt");
        storeFile.getParentFile().mkdirs();
        BufferedWriter bufWrter =
                new BufferedWriter(
                new FileWriter(
                storeFile));
        
        // POSITIVE RESULTS
        storeFile = new File(saveTo, category+"[1].txt");
        storeFile.getParentFile().mkdirs();
        
        BufferedWriter bufWrter2 =
                new BufferedWriter(
                new FileWriter(
                storeFile));
        
        // PRINTTING TABLE TITILES FOR EACH COLUMN
        bufWrter.write(
                "ITERATION\tCATEGORY\tVECTORS\t" +
                "PRECISION\tRECALL\tF-MEASURE\n");
        
        bufWrter2.write(
                "ITERATION\tCATEGORY\tVECTORS\t" +
                "PRECISION\tRECALL\tF-MEASURE\n");
        
        
        // THE RESULT FILES TO BE READ
        File []files = new File(path).listFiles();
        
        BufferedReader reader ;
        
        // FOR EACH RESULTS-FILE IN DIRECTORY "PATH"
        label: for (int i = 0; i<files.length;i++){
            
            File result = files[i];
            // ignore files in following cases
            if(result.isDirectory()||
                    !result.getName().startsWith("expmnt_") ||
                    result.getName().endsWith(".sta") // it is a statstics file for batch in active learning
                    )continue label;
            
            // file to read...
            reader = new BufferedReader(new FileReader(result));
            
            ////////////////////////////////////////////////////////////////////
            /////////////finding vectors for gateg//////////////////////////////
            // HOW MANY VECTORS FOUND IN TRAINING PROCESS FOR categ
            String num = new String();
            
            // SEARCH UNTILL FINDING NUMBER OF VECTORS FOR CATEGORY
            String searchLine ;
            while((searchLine = reader.readLine())!=null){
                
                if (searchLine.startsWith(category+" training vectors = ")){// eg :"PERSON training vectors = 7455"
                    
                    StringTokenizer tok = new StringTokenizer(searchLine, category+" training vectors = ");
                    // tok.nextToken();
                    num = tok.nextToken();
                    // System.out.println("num of vecs read = " + num);
                    break;
                    
                }// if
                
            }// while
            
            if(searchLine==null){
                
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "Either category given to resultsProcessing method is incorrect, " +
                        "or file :\n"+result.getName()+"\nis incorrect");
                
                Toolkit.getDefaultToolkit().beep();
                
                counter++;
                continue label; // PROCESS NEXT FILE
                
            }// if
            
            
            //===============finding vectors for gateg==========================
            //==================================================================
            
            
            
            
            if ( !readUntillSeparator(reader, "===")){
                
                counter++;
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "1st separator line before confusion matrix not found in file:/n/t/t"
                        +result.getName());
                
                continue ;
                
            }
            if ( !readUntillSeparator(reader, "===")){
                counter++;
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "2nd separator line before confusion matrix results not found in file:/n/t/t"
                        +result.getName());
                
                continue ;
                
            }
            
            
            // first results line from confusion matrix
            String line = reader.readLine();
            
            if(line==null){
                
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "No line to read after finding 2nd separator of confusion matrix in file:/n/t/t"
                        +result.getName());
                counter++;
                
                continue label;
                
            }
            // READING CONFUSION MATRIX
            StringTokenizer stok = new StringTokenizer(line);
            // TOKENS THAT WE WILL READ FROM THE LINE
            // precision - recall - f-measure - svmNum:negative("-1") or positive("1")
            String pres, rec, fMeas, svmNum;
            
            ////////////////////////////////////////////////////////////////////
            ////////////////read for categNum = |-1/////////////////////////////
            ////////////////////////////////////////////////////////////////////
            if (stok.hasMoreTokens())
                pres = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                rec = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                fMeas = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                svmNum=stok.nextToken();// just to avoid "|"
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                svmNum = svmNum.concat(stok.nextToken());
            else {counter++;continue label;}
            
            // pres = pres.replace('.', ',');
            // rec = rec.replace('.', ',');
            // fMeas = fMeas.replace('.', ',');
            
            // System.out.println("Line read after separetor :\n" + line);
            // System.out.println("tokenized to:"+pres+"\t"+rec+"\t"+fMeas);
            
            // writing iteration number
            String iteration = result.getName().substring(7);
            iteration = iteration.substring(0, iteration.indexOf('.'));
            bufWrter.write(iteration + "\t");
            
            bufWrter.write(category+"\t");
            bufWrter.write(num+"\t");
            bufWrter.write(pres+"\t");
            bufWrter.write(rec+"\t");
            bufWrter.write(fMeas+"\t");
            bufWrter.write(svmNum+"\n");
            
            stok = new StringTokenizer(line = reader.readLine());
            // closing reader, no need to read more from file
            reader.close();
           
            ////////////////////////////////////////////////////////////////////
            ////////////////read for categNum = |1/////////////////////////////
            ////////////////////////////////////////////////////////////////////
            if (stok.hasMoreTokens())
                pres = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                rec = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                fMeas = stok.nextToken();
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                svmNum = stok.nextToken();// just to avoid "|"
            else {counter++;continue label;}
            if (stok.hasMoreTokens())
                svmNum = svmNum.concat(stok.nextToken());
            else {counter++;continue label;}
            
            
            // pres=pres.replace('.', ',');
            // rec=rec.replace('.', ',');
            // fMeas=fMeas.replace('.', ',');
            
            // System.out.println("2nd line read after separetor :\n" + line);
            // System.out.println("tokenized to:"+pres+"\t"+rec+"\t"+fMeas);
            
            // writing iteration number
            iteration = result.getName().substring(7);
            iteration = iteration.substring(0, iteration.indexOf('.'));
            bufWrter2.write(iteration + "\t");
            
            bufWrter2.write(category + "\t");
            bufWrter2.write(num + "\t");
            bufWrter2.write(pres + "\t");
            bufWrter2.write(rec + "\t");
            bufWrter2.write(fMeas + "\t");
            bufWrter2.write(svmNum + "\n");
            
            
        }// for each file
        bufWrter.close();
        bufWrter2.close();
        
        JOptionPane.showMessageDialog(
                null,
                counter > 0 ? ("Failure in processing " + counter + " result files\n" +
                "Please try manually"):
                    "No failure in processing results files.", "Review of processing",
                JOptionPane.INFORMATION_MESSAGE);
        
    }
    
    
    
    public  void extractMatrix(String path, String category, String saveTo) throws IOException{
        
        // counter for not processed files
        int counter = 0;
        
        // DIRECTORY CONTAINING RESULT FILES
        File dir = new File(path);
        
        // WRITE THE RESULTS IN THE SAME DIRECTORY AS INDICATED BY PARAMETER "PATH"
        // NEGATIVE ERSULTS
        File  storeFile = new File(saveTo, "matrix resulrs(" + category+").txt");
        storeFile.getParentFile().mkdirs();
        BufferedWriter bufWrter =
                new BufferedWriter(
                new FileWriter(
                storeFile));
        
        
        
        // PRINTTING TABLE TITILES FOR EACH COLUMN
        bufWrter.write(
                "ITERATION" +
                "\tALL VECTORS USED" +
                "\tOF WHICH BEING " + category +
                "\tCORRECTLY CLASSIFIED AS " + category +
                "\tINCORRECTLY CLASSIFIED AS " + category +
                "\tSUM OF CLASSIFIED AS " + category +
                "\tTRAINING VECTORS FOR " + category + "\n");
        
        // THE RESULT FILES TO BE READ
        File []files = new File(path).listFiles();
        
        BufferedReader reader ;
        
        // FOR EACH RESULTS-FILE IN DIRECTORY "PATH"
        label: for (int i = 0; i<files.length;i++){
            
            File result = files[i];
            
            // ignore files in following cases
            if(result.isDirectory()||
                    !result.getName().startsWith("expmnt_") ||
                    result.getName().endsWith(".sta") // it is a statstics file for batch in active learning
                    )continue label;
            
            // file to read...
            reader = new BufferedReader(new FileReader(result));
            
            ////////////////////////////////////////////////////////////////////
            /////////////finding vectors for gateg//////////////////////////////
            // HOW MANY VECTORS FOUND IN TRAINING PROCESS FOR categ
            String num = new String();
            
            // SEARCH UNTILL FINDING NUMBER OF VECTORS FOR CATEGORY
            String searchLine ;
            while((searchLine = reader.readLine()) != null){
                
                if (searchLine.startsWith(category + " training vectors =")){ // eg :"PERSON training vectors = 7455"
                    
                    StringTokenizer tok = new StringTokenizer(searchLine, category + " training vectors =");
                    // reading training vectors
                    num = tok.nextToken();
                    // System.out.println("num of vecs read = " + num);
                    break;
                    
                }// if
                
            }// while
            
            if(searchLine == null){
                
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "Either category given to resultsProcessing method is incorrect, " +
                        "or file :\n"+result.getName()+"\nis incorrect");
                
                Toolkit.getDefaultToolkit().beep();
                
                counter++;
                continue label; // PROCESS NEXT FILE
                
            }// if
            
            
            //===============finding vectors for gateg==========================
            //==================================================================
            
            
            
            
            if ( !readUntillSeparator(reader, "===")){
                
                counter++;
                System.err.println(
                        
                        "There was a problem with processing results.\n" +
                        "1st separator line before confusion matrix not found in file:/n/t/t"
                        +result.getName());
                
                continue ;
                
            }
            
            
            // first matrix line from confusion matrix
            String line = reader.readLine();
            
            if(line == null){
                
                System.err.println(
                        
                        "There was a problem with processing matrix results.\n" +
                        "No line to read after finding 2nd separator of confusion matrix in file:/n/t/t"
                        +result.getName());
                
                counter++;
                
                continue label;
                
            }
            // READING CONFUSION MATRIX
            StringTokenizer stok = new StringTokenizer(line);
            
            //System.out.println("file" + result.getName()); 
            //System.out.println("line " + line); 
            
            // TOKENS THAT WE WILL READ FROM THE MATRIX LINES
            String
                    correctly_neg = null, // number of those correctly classified as negative vectors for the category
                    incorrectly_neg = null, // number of those incorrectly classified as negative vectors for the category
                    incorrectly_pos = null, // number of those incorrectly classified as positive vectors for the category
                    correctly_pos = null; // number of those correctly classified as positive vectors for the category
            
           
            
            ////////////////////////////////////////////////////////////////////
            ////////////////read for categNum = |-1/////////////////////////////
            ////////////////////////////////////////////////////////////////////
            if (stok.hasMoreTokens())
                correctly_neg = stok.nextToken();
            else {
                counter++;
                continue label;
            }
            
            if (stok.hasMoreTokens())
                incorrectly_pos = stok.nextToken();
            
            else {
                counter++;
                continue label;
            }
            
            
            
            // reading next line of matrix
            line = reader.readLine();
            stok = new StringTokenizer(line);
           
            //System.out.println("line " + line); 
            
            // closing reader, no need to read more from file
            reader.close();
            
            if (stok.hasMoreTokens())
                incorrectly_neg = stok.nextToken();
            
            else {
                counter++;
                continue label;
            }
            
            if (stok.hasMoreTokens())
                correctly_pos = stok.nextToken(); 
            
            else {
                counter++;
                continue label;
            }
            
            //System.out.println("correctly_neg" + correctly_neg); 
           // System.out.println("incorrectly_neg" + incorrectly_neg); 
           // System.out.println("incorrectly_pos" + incorrectly_pos); 
            //System.out.println("correctly_pos" + correctly_pos); 
          
            
            // writing iteration number
            String iteration = result.getName().substring(7);
            iteration = iteration.substring(0, iteration.indexOf('.'));
            
            int
                    allUsed = 0, // all vectors, even those cut by safe rules
                    beingReallyPos = 0, // vectors in test set really bellonging to category
                    correctCla = 0, //  vectors correctly classified as positive or negative
                    incorrectCla = 0, //  vectors incorrectly classified as positive
                    sumOfClaAsPos = 0, //  vectors correctly and incorrectly classified as positive
                    cutBySafeRules = 0;
            
            allUsed = Integer.parseInt(correctly_neg) + Integer.parseInt(incorrectly_neg) + Integer.parseInt(correctly_pos) + Integer.parseInt(incorrectly_pos);
            beingReallyPos = Integer.parseInt(incorrectly_neg) + Integer.parseInt( correctly_pos);
            correctCla = Integer.parseInt(correctly_pos);
            incorrectCla = Integer.parseInt(incorrectly_pos);
            
            /*
             ALL_VECTORS_USED, // all vectors, even those cut by safe rules
                    OF_WHICH_BEING, // vectors in test set really bellonging to category
                    CORRECTLY_CLASSIFIED_AS, //  vectors correctly classified as positive
                    INCORRECTLY_CLASSIFIED_AS, //  vectors incorrectly classified as positive
                    SUM_OF_CLASSIFIED_AS;// //  vectors correctly and incorrectly classified as positive
            */
            
            bufWrter.write(iteration + "\t");
            
            bufWrter.write(allUsed + "\t"); // all vectors used (including those cut by safe rules)
            bufWrter.write(beingReallyPos + "\t");// realy positive in test set
            bufWrter.write(correctly_pos + "\t");// correctly classified as positive
            bufWrter.write(incorrectly_pos + "\t");// incorrectly >> >>
            bufWrter.write((correctCla + incorrectCla) + "\t"); // all vectors calassified as positive
            bufWrter.write(num+ "\n");// number of training vectors
            
            // pres=pres.replace('.', ',');
            // rec=rec.replace('.', ',');
            // fMeas=fMeas.replace('.', ',');
            
            // System.out.println("2nd line read after separetor :\n" + line);
            // System.out.println("tokenized to:"+pres+"\t"+rec+"\t"+fMeas);
            
            // writing iteration number
            iteration = result.getName().substring(7);
            iteration = iteration.substring(0, iteration.indexOf('.'));
           
            
        }// for each file
        bufWrter.close();
        
        JOptionPane.showMessageDialog(
                null,
                counter > 0 ? ("Failure in processing " + counter + " result files\n" +
                "Please try manually"):
                    "No failure in processing results files.", "Review of processing",
                JOptionPane.INFORMATION_MESSAGE);
        
    }
    
    
    public void extractStats(String path, String saveTo)
    throws IOException{
        
        // counter for not processed files with errors
        int counter = 0;
        
        // DIRECTORY CONTAINING RESULT FILES
        File dir = new File(path);
        
        // STATISTICAL RESULTS
        File storeFile = new File(saveTo, "stats.txt");
        storeFile.getParentFile().mkdirs();
        
        // stream for output gathered statistics
        BufferedWriter bufWrter =
                new BufferedWriter(
                new FileWriter(
                storeFile));
        
        
        // PRINTTING COLUMN TITILES FOR EACH COLUMN
        bufWrter.write(
                "No\t" +
                "BATCH\t" +
                "PER_SAFE_RULES\t" +
                "ORG_SAFE_RULES\t" +
                "PER_TRAIN\t" +
                "ORG_TRAIN\t" +
                "SEL_ORG\t" +
                "SEL_PER\t" +
                "AVG_INFORM\n");
        
        String numOfExpernt,
                batchSize, // how many vectors in batch
                according2PerScore, // how many selected according to distance by person svm hyperplane
                SELECT_BY_ORG_perc, // how many selected according to distance by organization svm hyperplane
                SELECT_BY_PER_perc, // the same for person
                perSafeRules, // number of person training vectors - all in batch minus the ones surelly negative by safe rules
                orgSafeRules, // number of organization training vectors - all vectors in batch minus the ones surelly negative by safe rules
                score;// average informativeness
        
        // THE RESULT FILES TO BE READ
        File []files = new File(path).listFiles();
        
        BufferedReader reader ;
        
        // FOR EACH STATS-FILE IN DIRECTORY "PATH"
        label: for (int i = 0; i < files.length; i++){
            
            File statFile = files[i];
            // ignore files in following cases
            if(statFile.isDirectory()||
                    !statFile.getName().startsWith("expmnt_") ||
                    !statFile.getName().endsWith(".sta") // it is a statstics file for batch in active learning
                    )continue label;
            
            // file to read...
            reader = new BufferedReader(new FileReader(statFile));
            numOfExpernt =
                    statFile.getName().substring(
                    statFile.getName().indexOf("_") + 1,
                    statFile.getName().indexOf(".sta")
                    );
            System.out.println("* " + numOfExpernt  + ":\tProcessing file: " + statFile.getName());
            
            String searchLine ;
            if((searchLine = reader.readLine())!=null){
                
                StringTokenizer tok = new StringTokenizer(searchLine, "|");
                
                // reading each token
                batchSize = tok.nextToken();
                perSafeRules = tok.nextToken();
                orgSafeRules = tok.nextToken();
                score = tok.nextToken();
                according2PerScore = tok.nextToken();
                
                // computing double values
                int batchSizeNum = Integer.parseInt(batchSize);
                int perTrainVecs = (int) (batchSizeNum - Double.parseDouble(perSafeRules));
                int orgTrainVecs = (int) (batchSizeNum - Double.parseDouble(orgSafeRules));
                double avgScore = Double.parseDouble(score) / batchSizeNum;
                double selByPer = 100 * Double.parseDouble(according2PerScore) / batchSizeNum; // how many selected by person SVM (percentage)
                double selByOrg = 100 - selByPer; // how many selected by organization SVM (percentage)
                
                
                // converting to String again
                batchSize = String.valueOf(batchSizeNum);
                perSafeRules = String.valueOf(perTrainVecs);
                orgSafeRules = String.valueOf(orgTrainVecs);
                score = String.valueOf(avgScore);
                according2PerScore = String.valueOf(selByPer) ;
                according2PerScore= according2PerScore.substring(0, according2PerScore.indexOf(".") + 2 ) ;
                
                SELECT_BY_ORG_perc = String.valueOf(selByOrg);
                SELECT_BY_ORG_perc = SELECT_BY_ORG_perc.substring(0, SELECT_BY_ORG_perc.indexOf(".") + 2 ) ;
                
                SELECT_BY_PER_perc = String.valueOf(selByPer);
                SELECT_BY_PER_perc = SELECT_BY_PER_perc.substring(0, SELECT_BY_PER_perc.indexOf(".") + 2 ) ;
                
                String what2type =
                        numOfExpernt + "\t" +
                        batchSize + "\t" +
                        according2PerScore + "\t" +
                        SELECT_BY_ORG_perc + "\t" +
                        perSafeRules + "\t" +
                        orgSafeRules + "\t" +
                        SELECT_BY_ORG_perc + "\t" +
                        SELECT_BY_PER_perc + "\t" +
                        score + "\n";
                
                
                bufWrter.write(what2type);//.replace('.',','));
                
            }// if
            
            else {
                counter++;
            }
            
            
        }
        
        bufWrter.close();
        
        JOptionPane.showMessageDialog(
                null,
                counter > 0 ? ("Failure in processing " + counter + " result files\n" +
                "Please try manually"):
                    "No failure in processing stats.", "Review of processing",
                JOptionPane.INFORMATION_MESSAGE);
        
        
    }
    
    /**
     * Auxiliary method for resultsProcessing method above.
     *
     * @return  false if no line separ found by reading lines from reader.
     *          Otherwise it returns true.
     */
    private  boolean readUntillSeparator(BufferedReader reader,String separ)
    throws IOException{
        
        String nextLine;
        do{
            
            nextLine = reader.readLine();
            
        }
        while(nextLine!=null && !nextLine.startsWith(separ));
        
        if (nextLine==null)
            return false;
        else
            return true;
        
        
    }
    
    
    
}
