/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.preprocessing;

import ipl.util.Constants;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;

public class TextPreprocessor {
    
    
    public static String removeNoisySpaces(String text) {
        
        String tmp;
        int start, end;
        
        for (int i = 0; i < text.length(); i++) {
            
            if ( (Character.isWhitespace(text.charAt(i))) && (text.charAt(i) != ' ') &&
                    (text.charAt(i) != '\n')) {
                
                if (i == 0) {
                    
                    tmp = text.substring(1, text.length());
                    
                } // if
                else {
                    
                    start = i;
                    end = start + 1;
                    tmp = text.substring(0, start);
                    tmp = tmp.concat(text.substring(end, text.length()));
                    
                } // else
                
                text = tmp;
                
            } // if
            
        } // for
        
        return text;
        
    } // removeNoisySpaces
    
    protected static String removeHTMLtags(String text) {
        int start, end;
        String tmp;
        
        //        (body).
        if (text.toLowerCase().indexOf("<body") != -1) {
            text = text.substring(text.toLowerCase().indexOf("<body"));
        } // if
        else {
            return "";
        } // else
        
        //   SCRIPT
        while ((start = text.toLowerCase().indexOf("<script")) != -1) {
            end = text.toLowerCase().indexOf("</script>") + "</script>".length();
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end, text.length()));
        } // while
        
        //    (tags)     
        //  <  > .
        start = 0;
        while ((start = text.indexOf("<", start)) != -1) {
            if (text.toLowerCase().startsWith("<p>", start) ||
                    (text.toLowerCase().startsWith("<h", start) && !text.toLowerCase().startsWith("<hr>", start)) ||
                    text.toLowerCase().startsWith("</h", start)) {
                start++;
            } // if
            else {
                end = text.indexOf(">", start);
                tmp = text.substring(0, start);
                text = tmp.concat(text.substring(end + 1));
            } // else
        } // while
        
        //    &nbsp;  &nbsp
        while (text.indexOf("&nbsp;") != -1) {
            start = text.indexOf("&nbsp;");
            end = start + "&nbsp;".length();
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end, text.length()));
        } // while
        while (text.indexOf("&nbsp") != -1) {
            start = text.indexOf("&nbsp");
            end = start + "&nbsp".length();
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end, text.length()));
        } // while
        
        //    
        text = removeNoisySpaces(text);
        
        //       
        while (text.indexOf("\n ") != -1) {
            start = text.indexOf("\n ") + 1;
            end = start + 1;
            tmp = text.substring(0, start);
            tmp = tmp.concat(text.substring(end, text.length()));
            text = tmp;
        } // while
        
        //       
        while (text.indexOf(" \n") != -1) {
            start = text.indexOf(" \n");
            end = start + 1;
            tmp = text.substring(0, start);
            tmp = tmp.concat(text.substring(end, text.length()));
            text = tmp;
        } // while
        
        //    
        while (text.indexOf("\n\n") != -1) {
            start = text.indexOf("\n\n") + 1;
            end = start + 1;
            tmp = text.substring(0, start);
            tmp = tmp.concat(text.substring(end, text.length()));
            text = tmp;
        } // while
        
        text = text.replaceAll("<p>", " " + Constants.PARAGRAPH_TAG);
        text = text.replaceAll("<P>", " " + Constants.PARAGRAPH_TAG);
        
        return text;
    } // removeHTMLtags
    
    protected static String removeTags(String text) {
        int start, end;
        String tmp;
        
        //    (tags)     
        //  <  > .
        while (text.indexOf("<") != -1) {
            start = text.indexOf("<");
            end = text.indexOf(">");
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end + 1, text.length()));
        } // while
        
        //    &nbsp;  &nbsp
        while (text.indexOf("&nbsp;") != -1) {
            start = text.indexOf("&nbsp;");
            end = start + "&nbsp;".length();
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end, text.length()));
        } // while
        while (text.indexOf("&nbsp") != -1) {
            start = text.indexOf("&nbsp");
            end = start + "&nbsp".length();
            tmp = text.substring(0, start);
            text = tmp.concat(text.substring(end, text.length()));
        } // while
        
        //    
        text = removeNoisySpaces(text);
        
        return text;
    } // removeTags
    
    protected static String tagSubtitles(String text) {
        int start = 0, end;
        while ((start = text.toLowerCase().indexOf("<h", start)) != -1) {
            end = text.indexOf(">", start);
            String tmp = text.substring(0, start);
            text = tmp + Constants.SUBTITLE_START_TAG + text.substring(end + 1);
        } // while
        start = 0;
        while ((start = text.toLowerCase().indexOf("</h", start)) != -1) {
            end = text.indexOf(">", start);
            String tmp = text.substring(0, start);
            text = tmp + Constants.SUBTITLE_END_TAG + text.substring(end + 1);
        } // while
        return text;
    } // tagSubtitles
    
    protected static String setArticleTag(String text) {
        return ("<ARTICLE>\n\n" + text + "</ARTICLE>");
    } // setArticleTag
    
    protected static String setBodyTag(String text) {
        return ("<BODY>\n" + text + "\n</BODY>\n\n");
    } // setBodyTag
    
    
    
    
} // end of class TextPreprocessor
