/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import java.io.*;
import java.util.StringTokenizer;
//import weka.classifiers.*;
//import weka.classifiers.trees.J48;

import ipl.util.file_manager.ArffFileManager;

public class AttributeEvaluator {
    
    String [][] attrVectors;
    String [][] attributes;
    final int numOfClassValues;
    final int numOfAttributes;
    final int numOfInstances;
    
    public AttributeEvaluator(File arffFile) {
        String text = "";
        
        try {
            FileInputStream fis = new FileInputStream(arffFile);
            byte []buff = new byte[(int) arffFile.length()];
            fis.read(buff);
            fis.close();
            text = new String(buff);
            text = ipl.nerc.preprocessing.TextPreprocessor.removeNoisySpaces(text);
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        
        int index = text.indexOf("@data");
        numOfAttributes = loadAttrValues(text.substring(0, index));
        numOfInstances = loadAttrVectors(text.substring(index));
        numOfClassValues = initNumOfClassValues();
    } // constructor: AttributeEvaluator
    
    private int loadAttrValues(String header) {
        //   @relation tag
        header = header.substring(header.indexOf("\n") + 1);
        
        //     
        StringTokenizer lineTok = new StringTokenizer(header, "\n");
        int numAttr = lineTok.countTokens();
        attributes = new String[numAttr][2];
        
        //         
        String line;
        for (int i = 0; i < numAttr; i++) {
            line = lineTok.nextToken();
            StringTokenizer tok = new StringTokenizer(line, " ");
            tok.nextToken();
            attributes[i][0] = tok.nextToken();
            attributes[i][1] = tok.nextToken();
        } // for
        
        return numAttr;
    } // loadAttrValues
    
    private int loadAttrVectors(String body) {
        //   @data tag
        body = body.substring(body.indexOf("\n") + 1);
        
        //      (instances)
        StringTokenizer lineTok = new StringTokenizer(body, "\n");
        int numInstances = lineTok.countTokens();
        attrVectors = new String[numInstances][numOfAttributes];
        
        //     
        String line;
        for (int i = 0; i < numInstances; i++) {
            line = lineTok.nextToken();
            StringTokenizer attr = new StringTokenizer(line, ",");
            for (int j = 0; j < numOfAttributes; j++) {
                attrVectors[i][j] = attr.nextToken();
            } // for
        } // for
        return numInstances;
    } // loadAttrVectors
    
    private int initNumOfClassValues() {
        StringTokenizer strTok = new StringTokenizer(attributes[attributes.length - 1][1], ",");
        return strTok.countTokens();
    } // getNumOfClassValues
    
    public double[][] evaluate(int attr) {
        double precision_recall[][] = new double[numOfClassValues][2];
        String trainFileName = "train" + attr + ".arff";
        ArffFileManager arffManager = new ArffFileManager(trainFileName);
        
        String header = "@relation nerc" + attr + "\n";
        header += "@attribute " + attributes[attr][0] + " " + attributes[attr][1] + "\n";
        header += "@attribute " + attributes[numOfAttributes - 1][0] + " " + attributes[numOfAttributes - 1][1] + "\n";
        header += "@data";
        arffManager.setHeader(header);
        ///////////////////////////////////////////////////////////
        for (int i = 0; i < numOfInstances; i++) {
            //      arffManager.setData("\n" + attrVectors[i][attr] + "," + attrVectors[i][numOfAttributes - 1]);
        } // for
        ///////////////////////////////////////////////////////////
        String parameters[] = new String[2];
        parameters[0] = "-t";
        parameters[1] = trainFileName;
        try {
            //System.out.println(Evaluation.evaluateModel(new J48(), parameters));
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
        return precision_recall;
    } // evaluate
    
    public static void main(String args[]) throws Exception {
        if (args.length!=1){
        
            System.err.println("You must give the path to the arrf file.\n" +
                    "You must give only one argument to Attribute Evaluator class\n" +
                    "e.g. java - AttributeEvaluator c:\\my_directory_path\\attributes.arff");
            System.exit(-1);
            
        }
        System.out.print("Loading arff file...");
        AttributeEvaluator evaluator = new AttributeEvaluator(new File(args[0]));
        System.err.println(" DONE");
        for (int i = 0; i < evaluator.numOfAttributes - 1; i++) {
            System.out.print("Evaluating attribute " + i + " of " + evaluator.numOfAttributes + " ...");
            evaluator.evaluate(i);
            System.out.println(" DONE");
        }
    } // end of main
    
} // end of class AttributeEvaluator
