/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import java.io.*;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.Arrays;

import ipl.util.file_manager.DirectoryManager;

public class Auxiliary {
    
    public static String[] getSubList(String[] temp, int end) {
        String list[] = new String[end];
        for (int i = 0; i < end; i++) {
            list[i] = temp[i];
        } // for
        return list;
    } // getSubList
    
    public static double atof(String s) {
        return Double.valueOf(s).doubleValue();
    } // atof
    
    public static int atoi(String s) {
        return Integer.parseInt(s);
    } // atoi
    
    public static void initConfusionMatrix(int [][] confusionMatrix) {
        int size = confusionMatrix.length;
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                confusionMatrix[i][j] = 0;
            } // for
        } // for
    } // initConfusionMatrix
    
    public static void updateConfusionMatrix(int [][] confusionMatrix, double rightAnswer, double answer, PrintStream output) {
        if (rightAnswer == -1) {
            if (answer == -1) {
                confusionMatrix[0][0]++;
            } //  if
            else if (answer == 1) {
                confusionMatrix[0][1]++;
            } // else if
            else {
                output.println("updateConfusionMatrix: outOR1");
                System.exit(1);
            } // else
        } // if
        else if (rightAnswer == 1) {
            if (answer == -1) {
                confusionMatrix[1][0]++;
            } //  if
            else if (answer == 1) {
                confusionMatrix[1][1]++;
            } // else if
            else {
                output.println("updateConfusionMatrix: outOR2");
                System.exit(1);
            } // else
        } // else if
        else {
            output.println("updateConfusionMatrix: outOR3");
            System.exit(1);
        } // else
    } // updateConfusionMatrix
    
    public static double round(double f, int dec) {
        String strDouble = Double.toString(f);
        if (strDouble.length() > (dec + 2)) {
            strDouble = strDouble.substring(0, strDouble.indexOf(".") + dec + 1);
        } // if
        return Double.parseDouble(strDouble);
    } // round
    
    public static double[][] computePrecisionRecall(int [][] confusionMatrix) {
        
        int size = confusionMatrix.length;
        
        // how many lines X 3 (in other words an array 2X3). In this array, both precision, recall and f-measure will be stored after being computed
        double [][] prec_rec = new double[size][3];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < 3; j++) {
                prec_rec[i][j] = 0;
            } // for
        } // for
        
        
        /*
         *For all cells in the array, sumurize all vectors in each cell of the confusion matrix.
         *After summurizing:
         *[0,0] = those that are negative in test set
         *[0,1] = those classified as negative by svm
         *[1,0] = those that are positive in test set
         *[1,1] = those classified as positive by svm
         */
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                
                prec_rec[i][0] += confusionMatrix[i][j];
                prec_rec[j][1] += confusionMatrix[i][j];
                
            } // for
        } // for
        
        // after next "for", in first line there will be computed for negative tokens recall, precision, f-measure
        // in second line there will be computed for positive tokens recall, precision, f-measure
        for (int i = 0; i < size; i++) {
            prec_rec[i][0] = ((double) confusionMatrix[i][i] / prec_rec[i][0]);
            prec_rec[i][1] = ((double) confusionMatrix[i][i] / prec_rec[i][1]);
            prec_rec[i][2] = ((2 * prec_rec[i][0] * prec_rec[i][1]) / (prec_rec[i][0] + prec_rec[i][1]));
        } // for
        
        return prec_rec;
    } // computePrecisionRecall
    
    public static void printPrecisionRecall(double [][] prec_rec, int size, PrintStream output) {
        
        output.println("Precision\tRecall\tF-measure");
        output.println("==========================================");
        
        for (int i = 0; i < size; i++) {
            
            output.print(round(prec_rec[i][1], 4) + "\t");
            output.print(round(prec_rec[i][0], 4) + "\t");
            output.print(round(prec_rec[i][2], 4) + "\t");
            
            if (i == 0) {
                output.println("| -1");
            } // if
            else {
                output.println("|  1");
                
            } // else
            
        } // for
        
    } // printPrecisionRecall
    
    public static void printConfusionMatrix(int [][] confusionMatrix, PrintStream output) {
        
        output.println("= Confusion matrix =");
        output.println("-1\t1\t");
        output.println("====================");
        
        for (int i = 0; i < 2; i++) {
            
            for (int j = 0; j < 2; j++) {
                output.print(confusionMatrix[i][j] + "\t");
            } // for
            
            if (i == 0) {
                output.println("| -1");
            } // if
            
            else {
                output.println("|  1");
            } // else
            
        } // for
        
    } // printConfusionMatrix
    
    public static void createTrainsAndTestDirs(String dirName) {
        
        File dir = new File(dirName);
        
        if (!dir.isDirectory()) {
            
            System.err.println(dir.getName() + " is not a directory.");
            return;
            
        } // if
        
        DirectoryManager dirm = new DirectoryManager(dir.getAbsolutePath());
        File train1Dir = new File(dir.getParent() + File.separator + "train1");
        train1Dir.mkdir();
        File train2Dir = new File(dir.getParent() + File.separator + "train2");
        train2Dir.mkdir();
        File testDir = new File(dir.getParent() + File.separator + "test");
        testDir.mkdir();
        
        FileInputStream fis;
        FileOutputStream fos;
        File curr;
        byte []buff;
        int length;
        int less_than_six_perc = 90;
        int less_than_twelve_perc = 90;
        int more_than_twelve_perc = 90;
        int less_than_six_perc_train = 50;
        int less_than_twelve_perc_train = 50;
        int more_than_twelve_perc_train = 50;
        
        Random r = new Random(System.currentTimeMillis());
        
        while ((curr = dirm.getNextFile()) != null) {
            try {
                length = (int) curr.length();
                buff = new byte[length];
                fis = new FileInputStream(curr);
                fis.read(buff);
                fis.close();
                
                if (length < 6144) {
                    // train
                    if (Math.abs(r.nextInt() % 100) < less_than_six_perc) {
                        less_than_six_perc -= 1;
                        // train1
                        if (Math.abs(r.nextInt() % 100) < less_than_six_perc_train) {
                            less_than_six_perc_train -= 20;
                            fos = new FileOutputStream(train1Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // if
                        // train2
                        else {
                            less_than_six_perc_train += 20;
                            fos = new FileOutputStream(train2Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // else
                    } // if
                    // test
                    else {
                        less_than_six_perc += 9;
                        fos = new FileOutputStream(testDir.getAbsolutePath() + File.separator + curr.getName());
                    } // else
                } // if
                else if (length < 12288) {
                    // train
                    if (Math.abs(r.nextInt() % 100) < less_than_twelve_perc) {
                        less_than_twelve_perc -= 1;
                        // train1
                        if (Math.abs(r.nextInt() % 100) < less_than_twelve_perc_train) {
                            less_than_twelve_perc_train -= 20;
                            fos = new FileOutputStream(train1Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // if
                        // train2
                        else {
                            less_than_twelve_perc_train += 20;
                            fos = new FileOutputStream(train2Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // else
                    } // if
                    // test
                    else {
                        less_than_twelve_perc += 9;
                        fos = new FileOutputStream(testDir.getAbsolutePath() + File.separator + curr.getName());
                    } // else
                } // else if
                else {
                    // train
                    if (Math.abs(r.nextInt() % 100) < more_than_twelve_perc) {
                        more_than_twelve_perc -= 1;
                        // train1
                        if (Math.abs(r.nextInt() % 100) < more_than_twelve_perc_train) {
                            more_than_twelve_perc_train -= 20;
                            fos = new FileOutputStream(train1Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // if
                        // train2
                        else {
                            more_than_twelve_perc_train += 20;
                            fos = new FileOutputStream(train2Dir.getAbsolutePath() + File.separator + curr.getName());
                        } // else
                    } // if
                    // test
                    else {
                        more_than_twelve_perc += 9;
                        fos = new FileOutputStream(testDir.getAbsolutePath() + File.separator + curr.getName());
                    } // else
                } // else
                
                fos.write(buff);
                fos.close();
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
        } // while
    } // createTrainAndTestDir
    
    public static void createSetsForCrossValidation(String dirName, int folds) {
        
        File dir = new File(dirName);
        
        if (!dir.isDirectory()) {
            System.err.println(dir.getName() + " is not a directory.");
            return;
        } // if
        
        File[] dirs = new File[folds];
        
        int[] currLength = new int[folds];
        
        for (int i = 0; i < folds; i++) {
            dirs[i] = new File(dir.getAbsolutePath() + File.separator + i);
            dirs[i].mkdir();
            currLength[i] = 0;
        } // for
        
        DirectoryManager dirm = new DirectoryManager(dir.getAbsolutePath());
        
        FileInputStream fis;
        FileOutputStream fos;
        File curr;
        byte []buff;
        int length, totalLength = 0;
        Random r = new Random(System.currentTimeMillis());
        
        while ((curr = dirm.getNextFile()) != null) {
            if (curr.isFile()) {
                totalLength += (int) curr.length();
            } // if
        } // while
        totalLength = ((totalLength / folds) + 3072);
        
        dirm.reset();
        while ((curr = dirm.getNextFile()) != null) {
            try {
                if (curr.isFile()) {
                    length = (int) curr.length();
                    buff = new byte[length];
                    fis = new FileInputStream(curr);
                    fis.read(buff);
                    fis.close();
                    
                    boolean end = false;
                    while (!end) {
                        int pos = Math.abs(r.nextInt() % folds);
                        if (currLength[pos] + length <= totalLength) {
                            end = true;
                            currLength[pos] += length;
                            fos = new FileOutputStream(dirs[pos].getAbsolutePath() + File.separator + curr.getName());
                            fos.write(buff);
                            fos.close();
                        } // if
                    } // while
                } // if
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch
        } // while
    } //createSetsForCrossValidation
    
    public static String removeNoisySpaces(String text) {
        String tmp;
        int start, end;
        
        for (int i = 0; i < text.length(); i++) {
            if ( (Character.isWhitespace(text.charAt(i))) && (text.charAt(i) != ' ') &&
                    (text.charAt(i) != '\n')) {
                if (i == 0) {
                    tmp = text.substring(1, text.length());
                } // if
                else {
                    start = i;
                    end = start + 1;
                    tmp = text.substring(0, start);
                    tmp = tmp.concat(text.substring(end, text.length()));
                } // else
                text = tmp;
            } // if
        } // for
        return text;
    } // removeNoisySpaces
    
    
    public static void moveFilesForCrossValidation(String dirName, int folds, int test) {
        File dir = new File(dirName);
        if (!dir.isDirectory()) {
            System.err.println(dir.getName() + " is not a directory.");
            return;
        } // if
        
        File train1Dir = new File(dir.getAbsolutePath() + File.separator + "train1");
        train1Dir.mkdir();
        File train2Dir = new File(dir.getAbsolutePath() + File.separator + "train2");
        train2Dir.mkdir();
        File testDir = new File(dir.getAbsolutePath() + File.separator + "test");
        testDir.mkdir();
        
        FileInputStream fis;
        FileOutputStream fos;
        File curr;
        byte []buff;
        int length;
        Random r = new Random(System.currentTimeMillis());
        
        for (int i = 0; i < folds; i++) {
            DirectoryManager dirm = new DirectoryManager(dir.getAbsolutePath() + File.separator + i);
            
            if (i == test) {
                while ( (curr = dirm.getNextFile()) != null) {
                    try {
                        if (curr.isFile()) {
                            length = (int) curr.length();
                            buff = new byte[length];
                            fis = new FileInputStream(curr);
                            fis.read(buff);
                            fis.close();
                            
                            fos = new FileOutputStream(testDir.getAbsolutePath() +
                                    File.separator + curr.getName());
                            fos.write(buff);
                            fos.close();
                        }
                    } // try
                    catch (Exception e) {
                        e.printStackTrace();
                    } // catch
                } // while
            } // if
            else {
                int perc = 50;
                while ( (curr = dirm.getNextFile()) != null) {
                    try {
                        if (curr.isFile()) {
                            length = (int) curr.length();
                            buff = new byte[length];
                            fis = new FileInputStream(curr);
                            fis.read(buff);
                            fis.close();
                            
                            if (length < 6144) {
                                // train1
                                if (Math.abs(r.nextInt() % 100) < perc) {
                                    perc -= 3;
                                    fos = new FileOutputStream(train1Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // if
                                // train2
                                else {
                                    perc += 3;
                                    fos = new FileOutputStream(train2Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // else
                            } // if
                            else if (length < 12288) {
                                // train1
                                if (Math.abs(r.nextInt() % 100) < perc) {
                                    perc -= 40;
                                    fos = new FileOutputStream(train1Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // if
                                // train2
                                else {
                                    perc += 40;
                                    fos = new FileOutputStream(train2Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // else
                            } // else if
                            else {
                                // train1
                                if (Math.abs(r.nextInt() % 100) < perc) {
                                    perc -= 80;
                                    fos = new FileOutputStream(train1Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // if
                                // train2
                                else {
                                    perc += 80;
                                    fos = new FileOutputStream(train2Dir.getAbsolutePath() +
                                            File.separator + curr.getName());
                                } // else
                            } // else
                            
                            fos.write(buff);
                            fos.close();
                        } // if
                    } // try
                    catch (Exception e) {
                        e.printStackTrace();
                    } // catch
                } // while
            } // else
        } // for
    } // moveFilesForCrossValidation
    
    public static String removeGreekStressMarks(String text) {
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        
        text = text.replaceAll("", "");
        text = text.replaceAll("", "");
        
        return text;
    } // removeGreekStressMarks
    
    public static void sortFile(String fName) {
        try {
            File f = new File(fName);
            byte []buff = new byte[(int) f.length()];
            FileInputStream fis = new FileInputStream(f);
            fis.read(buff);
            fis.close();
            String text = new String(buff);
            text = ipl.nerc.preprocessing.TextPreprocessor.removeNoisySpaces(text);
            
            //text = Auxiliary.removeGreekStressMarks(text);
            
            StringTokenizer strTok = new StringTokenizer(text, "\n");
            String array[] = new String[strTok.countTokens()];
            int i = 0;
            while (strTok.hasMoreTokens()) {
                array[i] = strTok.nextToken();
                i++;
            } // while
            
            Arrays.sort(array);
            
            text = "";
            for (i = 0; i < array.length; i++) {
                text += array[i] + "\n";
            } // for
            
            FileOutputStream fos = new FileOutputStream(f);
            fos.write(text.getBytes());
            fos.close();
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
    } // sortFile
    
    public static void selectRandomFiles(String indir, String outdir) {
        DirectoryManager dirm = new DirectoryManager(indir);
        File curr;
        File outputDir = new File(outdir);
        if (!outputDir.exists()) {
            outputDir.mkdir();
        } // if
        
        int big_percent = 950; // NEA 955 - BHMA 975
        int small_percent = 985; // NEA 990 - BHMA 990
        Random random = new Random(System.currentTimeMillis());
        
        try {
            while ((curr = dirm.getNextFile()) != null) {
                if (curr.length() < 2048) {
                    if ((random.nextInt() % 1000) > small_percent) {
                        FileInputStream fis = new FileInputStream(curr);
                        byte[] buff = new byte[ (int) curr.length()];
                        fis.read(buff);
                        fis.close();
                        String text = new String(buff);
                        String fileName = outdir + File.separator + curr.getName();
                        FileOutputStream fos = new FileOutputStream(fileName);
                        fos.write(text.getBytes());
                        fos.close();
                    } // if
                } // if
                else {
                    if ((random.nextInt() % 1000) > big_percent) {
                        FileInputStream fis = new FileInputStream(curr);
                        byte[] buff = new byte[ (int) curr.length()];
                        fis.read(buff);
                        fis.close();
                        String text = new String(buff);
                        String fileName = outdir + File.separator + curr.getName();
                        FileOutputStream fos = new FileOutputStream(fileName);
                        fos.write(text.getBytes());
                        fos.close();
                    } // if
                } // else
            } // while
        } // try
        catch (Exception e) {
            e.printStackTrace();
        } // catch
    } // selectRandomFiles
    
} // end of class Auxiliary
