/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2007  X.Vassilakos
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import java.applet.Applet;
import java.io.File;
import java.net.MalformedURLException;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

public class Constants {
    
    
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////// Tags  ///////////////////////////////////
    public final static String NONE = "NONE";
    public final static String PERSON = "PERSON";
    public final static String ORGANIZATION = "ORGANIZATION";
    public final static String PER_ORG= PERSON + "_" + ORGANIZATION;
    public final static String LOCATION = "LOCATION";
    public final static String MONEY = "MONEY";
    public final static String PERCENT = "PERCENT";
    public final static String DATE = "DATE";
    public final static String TIME = "TIME";
    public final static String DATE_TIME = "DATE_TIME";
    
    public final static String ARTICLE = "ARTICLE";
    public final static String TITLE = "TITLE";
    public final static String SENTENCE = "SENTENCE";
    public final static String TIMEX = "TIMEX";
    public final static String ARTICLE_START_TAG = "<ARTICLE>";
    public final static String ARTICLE_END_TAG = "</ARTICLE>";
    public final static String TITLE_START_TAG = "<TITLE>";
    public final static String TITLE_END_TAG = "</TITLE>";
    public final static String SENTENCE_START_TAG = "<SENTENCE>";
    public final static String SENTENCE_END_TAG = "</SENTENCE>";
    public final static String PARAGRAPH_START_TAG = "<PARAGRAPH>";
    public final static String PARAGRAPH_END_TAG = "</PARAGRAPH>";
    public final static String SUBTITLE = "SUBTITLE";
    public final static String PARAGRAPH = "PARAGRAPH";
    public final static String SUBTITLE_START_TAG = "<SUBTITLE>";
    public final static String SUBTITLE_END_TAG = "</SUBTITLE>";
    public final static String PARAGRAPH_TAG = "<PARAGRAPH>";
    public final static String PERSON_START_TAG = "<ENAMEX TYPE=\"PERSON\">";
    public final static String PERSON_END_TAG = "</ENAMEX>";
    public final static String ORGANIZATION_START_TAG = "<ENAMEX TYPE=\"ORGANIZATION\">";
    public final static String ORGANIZATION_END_TAG = "</ENAMEX>";
    public final static String LOCATION_START_TAG = "<ENAMEX TYPE=\"LOCATION\">";
    public final static String LOCATION_END_TAG = "</ENAMEX>";
    public final static String MONEY_START_TAG = "<NUMEX TYPE=\"MONEY\">";
    public final static String MONEY_END_TAG = "</NUMEX>";
    public final static String PERCENT_START_TAG = "<NUMEX TYPE=\"PERCENT\">";
    public final static String PERCENT_END_TAG = "</NUMEX>";
    public final static String DATE_START_TAG = "<TIMEX TYPE=\"DATE\">";
    public final static String DATE_END_TAG = "</TIMEX>";
    public final static String TIME_START_TAG = "<TIMEX TYPE=\"TIME\">";
    public final static String TIME_END_TAG = "</TIMEX>";
    
    public final static String PERSON_START_TAG_OPEN = "<ENAMEX TYPE=\"PERSON\"";
    public final static String ORGANIZATION_START_TAG_OPEN = "<ENAMEX TYPE=\"ORGANIZATION\"";
    public final static String LOCATION_START_TAG_OPEN = "<ENAMEX TYPE=\"LOCATION\"";
    public final static String MONEY_START_TAG_OPEN = "<NUMEX TYPE=\"MONEY\"";
    public final static String PERCENT_START_TAG_OPEN = "<NUMEX TYPE=\"PERCENT\"";
    public final static String DATE_START_TAG_OPEN = "<TIMEX TYPE=\"DATE\"";
    public final static String TIME_START_TAG_OPEN = "<TIMEX TYPE=\"TIME\"";
    
    //============================= tags =======================================
    //==========================================================================
    
    
    
    
    public final static String FIRST_PASS = "FIRST_PASS";
    public final static String SECOND_PASS = "SECOND_PASS";
    public final static String FINANCIAL_ARTICLES_TYPE = "FINANCIAL_ARTICLES";
    public final static String GENERAL_ARTICLES_TYPE = "GENERAL_ARTICLES";
    
    ////////////////////////////////////////////////////////////////////////////
    //////////////////////// Paths to all files used ///////////////////////////
    public final static String
            
             /**
             * Path to all files parent directory
             */
            FILES_DIR_PATH = "./files/",
            ////////////////////////////////////////////////////////////////////
            ////////////////////////Sample files////////////////////////////////
            SAMPLE_DIR_PATH = FILES_DIR_PATH + "sample_data/",
            
            SAMPLE_INPUT_EXAMPLES_DIR = SAMPLE_DIR_PATH + "input_examples/",
            SAMPLE_TEST_DIR = SAMPLE_DIR_PATH + "sample_testing_data/",
            SAMPLE_TRAIN_FP_DIR = SAMPLE_DIR_PATH + "sample_training_first_pass/",
            SAMPLE_TRAIN_SP_DIR = SAMPLE_DIR_PATH + "sample_training_second_pass/",
            SAMPLE_ACTIVE_LEARNING_POOL_DIR = SAMPLE_DIR_PATH + "sample_training_active_learning/",
            SAMPLE_TRAIN_SPLITTER_DIR = SAMPLE_DIR_PATH + "sample_training_splitter/",
            SAMPLE_TRAIN_TIMEX_DIR = SAMPLE_DIR_PATH + "sample_training_timex/",
            
            
            /////////////////////////////////////////////////////////////////////
            ///////////////////////// System paths //////////////////////////////
            SYSTEM_DIR_PATH = "./system/",
            /** Path to file containing training vectors for Person, first pass*/
            FP_PERSON_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + FIRST_PASS + "_TRAIN_" + PERSON,
            /** Path to file containing training vectors for Person, second pass*/
            SP_PERSON_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + SECOND_PASS + "_TRAIN_" + PERSON,
            /** Path to file containing training vectors for Organization, first pass*/
            FP_ORGANIZATION_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + FIRST_PASS + "_TRAIN_" + ORGANIZATION,
            /** Path to file containing training vectors for Organization, second pass*/
            SP_ORGANIZATION_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + SECOND_PASS + "_TRAIN_" + ORGANIZATION,
            /**
             * Path to file containing training vectors for the initial classifier used
             * for starting active learning, for category Person, first pass
             */
            FP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + FIRST_PASS + "_initTRAIN_" + PERSON,
            /**
             * Path to file containing training vectors for the initial classifier used
             * for starting active learning, for category Person, second pass
             */
            SP_PERSON_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + SECOND_PASS + "_initTRAIN_" + PERSON,
            /**
             * Path to file containing training vectors for the initial classifier used
             * for starting active learning, for category Organization, first pass
             */
            FP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + FIRST_PASS + "_initTRAIN_" + ORGANIZATION,
            /**
             * Path to file containing training vectors for the initial classifier used
             * for starting active learning, for category Organization, second pass
             */
            SP_ORGANIZATION_ACTIVE_LEARNING_INITIAL_TRAIN_VECTORS_FILE = SYSTEM_DIR_PATH + "training_vectors/" + SECOND_PASS + "_initTRAIN_" + ORGANIZATION,
            //========================= SYSTEM's files =========================
            //==================================================================
            
            
           
            /**
             * Path to pretrained classifier and frequency list files for financial articles
             */
            FINANCIAL_DIR_PATH = "./financial_articles_files/",
            /**
             * Path to pretrained classifier and frequency list files for general articles
             */
            GENERAL_DIR_PATH = "./general_articles_files/",
            
            /**
             * Path to .arrf files. Arrf files are files containing vectors that can
             * be parsed by WEKA. (used for IG feature evaluation )
             */
            ARRF_DIRECTORY_PATH = FILES_DIR_PATH + "/arffs/",
            
            
            
            
            /////////////////////////////////////////////////////////////////////
            /////////////////// Paths frequency lists ///////////////////////////
            
            LISTS_DIRECTORY_PATH = FILES_DIR_PATH + "/lists/",
            
            LISTS_GENERAL_DIRECTORY_PATH = GENERAL_DIR_PATH + "/lists/",
            //LISTS_GENERAL_FP_DIRECTORY_PATH = LISTS_GENERAL_DIRECTORY_PATH,
            //LISTS_GENERAL_SP_DIRECTORY_PATH = LISTS_GENERAL_DIRECTORY_PATH,
            
            LISTS_FINANCIAL_DIRECTORY_PATH = FINANCIAL_DIR_PATH + "/lists/",
           // LISTS_FINANCIAL_FP_DIRECTORY_PATH = LISTS_FINANCIAL_DIRECTORY_PATH,
           // LISTS_FINANCIAL_SP_DIRECTORY_PATH = LISTS_FINANCIAL_DIRECTORY_PATH,
            
            FULL_LISTS_DIRECTORY_PATH = LISTS_DIRECTORY_PATH + "Active_Learning_full_lists/",
            
            //================== Paths frequency lists ===========================
            //====================================================================
            
            RESULTS_DIRECTORY_PATH = FILES_DIR_PATH + "/results/",
            ENTITIES_RECONIZED_DIR_PATH = RESULTS_DIRECTORY_PATH + "/entities_recognized/",
            PERSON_ENTITIES_FILE_PATH = ENTITIES_RECONIZED_DIR_PATH + PERSON + "S.txt",
            ORGANIZATION_ENTITIES_FILE_PATH = ENTITIES_RECONIZED_DIR_PATH + ORGANIZATION + "S.txt",
            
            
            ////////////////////////////////////////////////////////////////////
            //////////////////ACTIVE LEARNING PATHS/////////////////////////////
            /**
             * Path to active learning files directory.
             */
            ACTIVE_LEARNING_DIR_PATH = SYSTEM_DIR_PATH + "/active_learning/",
            
            ACTIVE_LEARNING_TEST_DIR = SAMPLE_TEST_DIR,
            
            /**
             * Path to directory-pool of files used for active learning training
             */
            ACTIVE_LEARNING_POOL_OF_FILES_DIR_PATH  = SAMPLE_ACTIVE_LEARNING_POOL_DIR,
            
            /**
             * Path to directory where files for (sub)pool average similarity are saved
             */
            ACTIVE_LEARNING_SIMILARITIES_DIR_PATH = ACTIVE_LEARNING_POOL_OF_FILES_DIR_PATH + "avg_similarity/",
            
            /**
             * File which contains all ambiguous vectors
             * found in active learning experiments (PASS = first).
             */
            FP_PER_AMBIGUOUS_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/FP_PERSON_ambiguousVectors.txt",
            
            /**
             * File which contains all  ambiguous vectors (category = ORGANIZATION)
             * found in active learning experiments (PASS=first).
             */
            FP_ORG_AMBIGUOUS_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/FP_ORGANIZATION_ambiguousVectors.txt",
            
            
            /**
             * File which contains all ambiguous vectors (category = PERSON)
             * found in active learning experiments (PASS=second).
             */
            SP_PER_AMBIGUOUS_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/SP_PERSON_ambiguousVectors.txt",
            
            /**
             * File which contains all ambiguous vectors (category = ORGANIZATION)
             * found in active learning experiments (PASS=second).
             */
            SP_ORG_AMBIGUOUS_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/SP_ORGANIZATION_ambiguousVectors.txt",
            
            /**
             * Path to file for the used vectors of the first subpool
             * of active learning training vectors (first pass)
             */
            FP_USED_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/First_Pass_used_vectors.txt",
            
            
            
            /**
             * Path to file for the used vectors of the first subpool
             * of active learning training vectors (second pass)
             */
            SP_USED_VECTORS_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/Second_Pass_used_vectors.txt",
            
            FP_VECTORS_INFO_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + Constants.FIRST_PASS + "_vectorsInfo.txt",
            
            SP_VECTORS_INFO_FILE_PATH =
            ACTIVE_LEARNING_DIR_PATH + Constants.SECOND_PASS + "_vectorsInfo.txt",
            
            /**
             * Path to the directory where all batch objects are saved in order
             * to compare for "global diversity active learning"
             */
            ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/Global_Diversity/",
            
            ACTIVE_LEARNING_SAMPLE_POOL_DIR_PATH =
            ACTIVE_LEARNING_DIR_PATH + "/Representativeness/",
            
            /**
             * Path to the text file where all bathes in String represantation
             * are saved in order
             * to compare for "global diversity active learning"
             */
            ACTIVE_LEARNING_FORMER_BATCHES_FILE_PATH =
            ACTIVE_LEARNING_FORMER_BATCHES_DIR_PATH + "Former_batches.txt",
            
            
            ACTIVE_LEARNING_FP_DECISION_LISTS_PARENT_DIR = Constants.ACTIVE_LEARNING_DIR_PATH + "/" + Constants.FIRST_PASS + "_decision_lists",
             
            ACTIVE_LEARNING_FP_ALL_DECISION_LISTS_FILENAME = "all_decisions_htab.obj",
            
            ACTIVE_LEARNING_FP_SURE_DECISIONS_FILENAME = "sure_lists_htab.obj";
    
    
    
    
    //====================== ACTIVE LEARNING PATHS =============================
    //==========================================================================
    
    
    
    /////////////////////////////////////////////////////////////////////////////////
    ///////////////////////// all classifiers paths /////////////////////////////////
    public static String
            
            
            /**Path to directory of Classifiers files.*/
            CLASSIFIERS_DIR_PATH = FILES_DIR_PATH + "/classifiers/",
            
            
            
            ////////////////////// NAMED ENTITY CLASSIFIERS //////////////////////////////
            /**Path to classifier model for Person, first pass.*/
            
            FINANCIAL_CLASSIFIERS_DIR_PATH = FINANCIAL_DIR_PATH + "/classifiers/",
            GENERAL_CLASSIFIERS_DIR_PATH = GENERAL_DIR_PATH + "/classifiers/",
            
            FP_PERSON_CLASSIFIER_FILE_PATH =
            CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to classifier model for Person, second pass.*/
            SP_PERSON_CLASSIFIER_FILE_PATH =
            CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to classifier model for Organization, first pass.*/
            FP_ORGANIZATION_CLASSIFIER_FILE_PATH =
            CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
            /**Path to classifier model for Organization, second pass.*/
            
            SP_ORGANIZATION_CLASSIFIER_FILE_PATH =
            CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
            
            /**Path to general articles classifier model for Person, one pass.*/
            FP_PERSON_GENERAL_CLASSIFIER_FILE_PATH =
            GENERAL_CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to general articles classifier model for Person, second pass.*/
            SP_PERSON_GENERAL_CLASSIFIER_FILE_PATH =
            GENERAL_CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to general articles classifier model for Organization, one pass.*/
            FP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH =
            GENERAL_CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
            
            /**Path to general articles classifier model for Organizarion, second pass.*/
            SP_ORGANIZATION_GENERAL_CLASSIFIER_FILE_PATH =
            GENERAL_CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
            
            /**Path to financial articles classifier model for Person, one pass.*/
            FP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH =
            FINANCIAL_CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to financial articles classifier model for Person, second pass.*/
            SP_PERSON_FINANCIAL_CLASSIFIER_FILE_PATH =
            FINANCIAL_CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.PERSON + "_CLASSIFIER",
            
            /**Path to financial articles classifier model for Organization, one pass.*/
            FP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH =
            FINANCIAL_CLASSIFIERS_DIR_PATH + Constants.FIRST_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
            
            /**Path to financial articles classifier model for Organizarion, second pass.*/
            SP_ORGANIZATION_FINANCIAL_CLASSIFIER_FILE_PATH =
            FINANCIAL_CLASSIFIERS_DIR_PATH + Constants.SECOND_PASS + "_" + Constants.ORGANIZATION + "_CLASSIFIER",
                     
            
            
            //////////////////////////// TIMEX //////////////////////////////////
            /**Path to timex patterns file.*/
            TIMEX_PATERNS_FILE_PATH = CLASSIFIERS_DIR_PATH + "/TIMEX_PATTERNS",
            
            /**Path to timex patterns file for financial articles.*/
            TIMEX_PATERNS_FINANCIAL_ARTICLES_FILE_PATH = FINANCIAL_CLASSIFIERS_DIR_PATH + "/TIMEX_PATTERNS",
            
            /**Path to timex patterns file for general articles.*/
            TIMEX_PATERNS_GENERAL_ARTICLES_FILE_PATH =  GENERAL_CLASSIFIERS_DIR_PATH + "/TIMEX_PATTERNS",
            //========================= TIMEX ===================================
            
            
            //////////////////////////// SENTENCE SPLITTER ////////////////////////////
            /**Path to sentence splitter classifier file.*/
            SENTENCE_SPLITTER_CLASSIFIER_FILE_PATH =
            CLASSIFIERS_DIR_PATH + "SENTENCE_SPLITTER_CLASSIFIER",
            
            /**Path to sentence splitter classifier file, for financial articles.*/
            SENTENCE_SPLITTER_CLASSIFIER_FINANCIAL_FILE_PATH =
            FINANCIAL_CLASSIFIERS_DIR_PATH + "/SENTENCE_SPLITTER_CLASSIFIER",
            
            /**Path to sentence splitter classifier file, for general articles.*/
            SENTENCE_SPLITTER_CLASSIFIER_GENERAL_FILE_PATH =
            GENERAL_CLASSIFIERS_DIR_PATH + "/SENTENCE_SPLITTER_CLASSIFIER";
    //========================= SENTENCED SPLITTER ===========================
    
    //============================= all classifiers paths ==================================
    //======================================================================================
    
    
    
    
    ////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////// Sounds and Images ///////////////////////////////////////
    
    public final static String
            
            SOUND_CLICK_01_FILE = "/sounds/click_01.wav",
            SOUND_BACK_FILE = "/sounds/back.wav",
            SOUND_CATEG_FILE = "/sounds/categ.wav",
            SOUND_DISCONNECT_FILE = "/sounds/disconnect.wav",
            SOUND_NEXT_FILE = "/sounds/next.wav",
            SOUND_NOTIFY_FILE = "/sounds/notify.wav",
            SOUND_ALERT_01_FILE = "/sounds/alert_01.wav",
            SOUND_PROCESS_END_FILE = "/sounds/process_ended_[gong].au",
            SOUND_START_ACTION_FILE = "/sounds/start_action.wav",
            
            
            IMA_INFO_01_01 = "/icons/info_01.01.JPG",
            IMA_WARN_01 = "/icons/warning_01.gif";
    
    //===================== Sounds and images paths ==================================
    //================================================================================
    
    
    
    
    
    
    /**
     * Number of features used in first pass training for category "PERSON"
     */
    public final static int PER_FEAT_NUM_FP = 165 ;
    
    /**
     * Number of features used in second pass training for category "PERSON"
     */
    public final static int PER_FEAT_NUM_SP = 207; // PER_FEAT_NUM_FP + 56 -14;
    
    /**
     * Number of features used in first pass training for category "ORGANIZATION"
     */
    public final static int ORG_FEAT_NUM_FP = 170;
    
    /**
     * Number of features used in second pass training for category "ORGANIZATION"
     */
    public final static int ORG_FEAT_NUM_SP = 217; // ORG_FEAT_NUM_FP + 56 - 9 ;
    
    
    
    /***************************************************************
     ***************************************************************
     * default c and gamma parameter values :
     ***************************************************************
     ***************************************************************/
    
    /**
     * Inital parameter c before SVM-tuning.
     */
    public final static double INIT_C = 1.0;
    
    /**
     * Inital parameter gamma before SVM-tuning.
     */
    public final static double INIT_GAMMA = 0.01;
    
    /**
     * parameter c after SVM-tuning for :
     *
     * -category=PERSON
     * -Pass=first pass
     * -number of features=165 .
     */
    public final static double DEFAULT_PER_C_FP = 8.0;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category=PERSON
     * -Pass=first pass
     * -number of features=Constants.PER_FEAT_NUM_FP
     */
    public final static double DEFAULT_PER_GAMMA_FP = 0.0078125;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category=ORGANIZATION
     * -Pass=first pass
     *  -number of features = Constants.PER_FEAT_NUM_FP.
     */
    public final static double DEFAULT_PER_C_SP = 9.513656920021768;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.PER_FEAT_NUM_FP.
     */
    public final static double DEFAULT_PER_GAMMA_SP = 0.002762135864009951;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category = PERSON
     * -Pass = second pass
     * -number of features = Constants.PER_FEAT_NUM_SP.
     */
    public final static double DEFAULT_ORG_C_FP = 2.378414230005442;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category=ORGANIZATION
     * -Pass=first pass
     * -number of features = Constants.ORG_FEAT_NUM_FP.
     */
    public final static double DEFAULT_ORG_GAMMA_FP = 0.018581361171917516;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.ORG_FEAT_NUM_FP.
     */
    public final static double DEFAULT_ORG_C_SP = 4.675108994244982;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.ORG_FEAT_NUM_SP.
     */
    public final static double DEFAULT_ORG_GAMMA_SP = 0.0078125; // 0.15625;
    
    
    
    
    
    /************************************************************************
     ************************************************************************
     ************ svm parameter values used by the system *******************
     ************************************************************************
     ************************************************************************/
    
    /**
     * parameter c after SVM-tuning for :
     *
     * -category=PERSON
     * -Pass=first pass
     * -number of features=165 .
     */
    public static double PER_C_FP = DEFAULT_PER_C_FP;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category=PERSON
     * -Pass=first pass
     * -number of features=Constants.PER_FEAT_NUM_FP
     */
    public static double PER_GAMMA_FP = DEFAULT_PER_GAMMA_FP;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category=ORGANIZATION
     * -Pass=first pass
     *  -number of features = Constants.PER_FEAT_NUM_FP.
     */
    public static double PER_C_SP = DEFAULT_PER_C_SP;
    /**
     * A threshold used for considering a decision (about classifiing a token) made by an SVM classifier,
     * as a "sure enought decision".
     * If the canonical posibility assigned for a decision is grater than or equal to the threshold, then this
     * decision is sure enought.
     *
     * @see MyClassifier.findFPDecisionsForSP
     */
    public static double SURE_DEC_MARGIN = 0.8;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.PER_FEAT_NUM_FP.
     */
    public static double PER_GAMMA_SP = DEFAULT_PER_GAMMA_SP;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category = PERSON
     * -Pass = second pass
     * -number of features = Constants.PER_FEAT_NUM_SP.
     */
    public static double ORG_C_FP = DEFAULT_ORG_C_FP;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category=ORGANIZATION
     * -Pass=first pass
     * -number of features = Constants.ORG_FEAT_NUM_FP.
     */
    public static double ORG_GAMMA_FP = DEFAULT_ORG_GAMMA_FP;
    /**
     * parameter c after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.ORG_FEAT_NUM_FP.
     */
    public static double ORG_C_SP = DEFAULT_ORG_C_SP;
    /**
     * parameter gamma after SVM-tuning for :
     *
     * -category = ORGANIZATION
     * -Pass = second pass
     * -number of features = Constants.ORG_FEAT_NUM_SP.
     */
    public static double ORG_GAMMA_SP = DEFAULT_ORG_GAMMA_SP;
    
    
    
    public static double PER_AVG_POOL_SIM = 0.0;
    public static double ORG_AVG_POOL_SIM = 0.0;
    public static double PER_ORG_AVG_POOL_SIM = 0.7350819007066595194714778770238;
    public static double LOCAL_DIVER_THRESHOLD = 0.82;
    
    
    public static String toTimeString(long start, long end){
        
        long time = end - start;
        
        long hours = time / 3600000;
        time -= hours * 3600000;
        long mins  = time / 60000;
        time -= mins * 60000;
        long secs  = time / 1000;
        
        // System.out.println("Start = "+ start);
        // System.out.println("end = "+ end);
        
        return  hours + " hours, " + mins + " minutes, " + secs + " seconds";
        
    }
    
    public static  String toTimeString(long start){
        
        return toTimeString(start, System.currentTimeMillis());
        
    }
    
    public static  String toDigitalTimeString(long start, long end){
        
        long time = end - start;
        
        /*
         * 1 sec = 1000
         * 1 min = 60000
         * 1 hour = 3600000
         * 1 day =  24* 1 hour
         */
        String digits = "";
        
        long h = time/3600000;
        long x = time%3600000;
        long min = x/60000;
        long sec = x%60000;
        sec/=1000;
        
        digits = h + ":" + min + ":" + sec;
        
        return digits;
        
    }
    
    public static String toDigitalTimeString(long start){
        
        return toTimeString(start, System.currentTimeMillis());
        
    }
    
    
    public final static void CHANGE_SVM_PARAM_VALUES(
            String category, String cFP, String cSP, String gammaFP,
            String gammaSP) throws NumberFormatException{
        
        if (cFP.trim().startsWith("-")||
                cSP.trim().startsWith("-")||
                gammaFP.trim().startsWith("-")||
                gammaSP.trim().startsWith("-")
                ){
            throw new NumberFormatException("Non of the svm parameter values should be negative.");
        }
        
        
        if(category.equals(PERSON)){
            
            PER_C_FP = Double.valueOf(cFP);
            PER_C_SP = Double.valueOf(cSP);
            PER_GAMMA_FP = Double.valueOf(gammaFP);
            PER_GAMMA_SP = Double.valueOf(gammaSP);
            
        }else {//if(category.equals(ORGANIZATION)){
            
            ORG_C_FP = Double.valueOf(cFP);
            ORG_C_SP = Double.valueOf(cSP);
            ORG_GAMMA_FP = Double.valueOf(gammaFP);
            ORG_GAMMA_SP = Double.valueOf(gammaSP);
            
        }
        
        
        
    }
    
    
    
    
} // end of class Finals
