package ipl.util;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.StringTokenizer;
import javax.swing.JFileChooser;

public class NativeTuner {
    
    static { 
        System.loadLibrary("./lib/native/cross_validation");
    } // static
    
    public NativeTuner() {
    } // constructor: NativeTuner
    
    public static native double cross_validation(char[] vectorsFilename, int size, double c, double gamma, int crossValFolds);
   
    private static void printParametersAndExit(String str) {
        System.err.println(str);
        System.err.println();
        System.err.println();
        System.err.println(" e.g. java ipl.sentence_splitter.Tuner [-c from:to:step -g from:to:step -v fold-num]");
        System.err.println();
        System.err.println("-c");
        System.err.println("\tThe parameter c of svm\t\tdefault 0:5:1");
        System.err.println("-g");
        System.err.println("\tThe parameter gamma of svm\tdefault -5:5:1");
        System.err.println("-v");
        System.err.println("\tv-fold cross validation\t\tdefault 5");
        System.err.println();
        System.exit(1);
    } // printParametersAndExit
   
    private double[] tune(String trainVectorsPath, 
            double cfrom, double cto, double cstep,
            double gfrom, double gto, double gstep,
            int crossValFolds) {
        
        double max = 0.0;
        double maxc = 0.0, maxg = 0.0;
        
        for (double x = cfrom; x <= cto; x += cstep) {
            
            for (double y = gfrom; y <= gto; y += gstep) {
                
                double acc = cross_validation(trainVectorsPath.toCharArray(), trainVectorsPath.length(),
                        Math.pow(2.0, x), Math.pow(2.0, y), crossValFolds);
                            
                if (acc > max) {
                
                    max = acc;
                    maxc = Math.pow(2.0, x);
                    maxg = Math.pow(2.0, y);
                
                } // if
                
                System.out.println("\n");
                System.out.println(x + " " + y + " " + acc + " | " + maxc + " " + maxg + " " + max);
                System.out.println("*************************************************************");
                
            } // for
            
        } // for
        
        System.out.println("\n\n");
        System.out.println("c = " + maxc);
        System.out.println("g = " + maxg);
        System.out.println("accuracy = " + max);
        
        // return an array containing maximum results: 0:c 1:gamma 2:accuracy
        return (new double[]{maxc, maxg, max});
          
    } // tune
    
    
   
    public double[] tune(String trainVecsFile,
            double cFrom, double co, double cStep,
            double gFrom, double go, double gStep,
            int crossValFolds, PrintStream pOut){ 
        
        double []results = tune(trainVecsFile, cFrom, co, cStep, gFrom, go, gStep, crossValFolds);
        
        pOut.println(//"Iteration 1:\n" +
                "Step of c: " + cStep + ", from power o c: " + cFrom + ", to power o c: " + co);
        pOut.println("Step of gamma: " + gStep + ", from power o gamma: " + gFrom + ", to power o gamma: " + go);
        pOut.println("Cross validation folds: " + crossValFolds);
        pOut.println("___________________________________________");
        
        pOut.println("C: " + results[0]);
        pOut.println("Gamma: " + results[1]);
        pOut.println("Accuracy: " + results[2]);
        
        cFrom = Math.log(results[0]) / Math.log(2) - 1;
        co = Math.log(results[0]) / Math.log(2) + 1;
        
        gFrom = Math.log(results[1]) / Math.log(2) - 1;
        go = Math.log(results[1]) / Math.log(2) + 1;
        
        /*
        cStep /= 4;
        gStep /= 4;
        
        
        results = tune(trainVecsFile, cFrom, co, cStep, gFrom, go, gStep, crossValFolds);
        
        pOut.println("Iteration 2(final):\n" +
                "Step of c: " + cStep + ", from power o c: " + cFrom + ", to power o c: " + co);
        pOut.println("Step of gamma: " + gStep + ", from power o gamma: " + gFrom + ", to power o gamma: " + go);
        pOut.println("Cross validation folds: " + crossValFolds);
        pOut.println("___________________________________________");
        
        pOut.println("C: " + results[0]);
        pOut.println("Gamma: " + results[1]);
        pOut.println("Accuracy: " + results[2]);
        */
        
        return results;
        
    }
    /**
     *
     * @param args String[]
     */
    public static void main(String args[]) {
        
        String s = "-c 0:5:1 -g -5:5:1 -v 5";
        StringTokenizer ss = new StringTokenizer(s);
        args = new String[ss.countTokens()];
        int iii=0;
        while(ss.hasMoreElements()){
            args[iii++] = ss.nextToken();
        }
        
        if ((args.length % 2 == 1)) {
            printParametersAndExit("Invalid parameters number!");
        } // if
        
        StringTokenizer strTok;
        double cfrom = 0.0;
        double cto = 5.0;
        double cstep = 1.0;
        double gfrom = -5.0;
        double gto = 5.0;
        double gstep = 1.0;
        int v = 5;
        for (int i = 0; i < args.length; i += 2) {
            if (args[i].length() != 2) {
                printParametersAndExit("Invalid parameters!");
            } // if
            if (args[i].charAt(0) != '-') {
                printParametersAndExit("Invalid parameters!");
            } // if
            char param = args[i].charAt(1);
            switch (param) {
                case 'c':
                    strTok = new StringTokenizer(args[i + 1], ":");
                    if (strTok.countTokens() != 3) {
                        printParametersAndExit("Invalid parameter c!");
                    } // if
                    cfrom = Double.parseDouble(strTok.nextToken());
                    cto = Double.parseDouble(strTok.nextToken());
                    cstep = Double.parseDouble(strTok.nextToken());
                    break;
                case 'g':
                    strTok = new StringTokenizer(args[i + 1], ":");
                    if (strTok.countTokens() != 3) {
                        printParametersAndExit("Invalid parameter g!");
                    } // if
                    gfrom = Double.parseDouble(strTok.nextToken());
                    gto = Double.parseDouble(strTok.nextToken());
                    gstep = Double.parseDouble(strTok.nextToken());
                    break;
                case 'v':
                    v = Integer.parseInt(args[i + 1]);
                    if (v < 2) {
                        printParametersAndExit("v must >= 2");
                    } // if
                    break;
                default:
                    printParametersAndExit("Invalid parameters!");
                    break;
            } // switch
        } // for
        
        NativeTuner tuner = new NativeTuner();
        
        JFileChooser jCh = new JFileChooser("..");
        jCh.setDialogTitle("Chose Classifer file");
        jCh.setFileSelectionMode(jCh.FILES_ONLY);
        if(jCh.showDialog(null, "ok") == jCh.APPROVE_OPTION){
            
            try {
                
                PrintStream pOut = new PrintStream(new java.io.FileOutputStream(jCh.getSelectedFile().getName() + ".tuneResults"));
                long start = System.currentTimeMillis();
                
                tuner.tune(jCh.getSelectedFile().getCanonicalPath(), cfrom, cto, cstep, gfrom, gto, gstep, v, pOut); 
                System.out.println(jCh.getSelectedFile().getName() + " Tuned. \n" +
                        "Time needed: " + Constants.toDigitalTimeString(start) + "\n" +
                        "Results saved in file: " + jCh.getSelectedFile().getName() + ".tune");
                
                pOut.close();
                
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            
        }
        
    } // end of main
    
} // end of class NativeTuner
