package ipl.util.file_manager;

import ipl.core.Tokenizer;
import ipl.util.Constants;
import java.io.*;
import java.util.List;

public abstract class FileManager {
    /**
     *      ,      .
     */
    protected FileOutputStream fos;
    
    /**
     *       ,    
     * fos  .
     * @param file String
     */
    public FileManager(String file) {
        try {
            File outf = new File(file);
            fos = new FileOutputStream(outf);
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
    } // constructor: FileManager
    
    /**
     *      () ,  
     *         Strings.
     * @param attrVector String[]
     */
    public abstract void setData(String[] attrVector);
    
    /**
     *       .
     */
    public void closeFile() {
        try {
            fos.close();
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
    } // closeArffFile
    
    public static File mergeFiles(List<File> files, String pathToMerge) throws FileNotFoundException, IOException{
        
        File merge = new File(pathToMerge);
        FileOutputStream out = new FileOutputStream(merge);
        
        for(File tmp: files){
            
            FileInputStream in = new FileInputStream(tmp);
            byte []bytes = new byte[in.available()];
            in.read(bytes);
            in.close();
            
            out.write(bytes);
            
        }
        out.close();
        
        return merge;
        
    }
    
    public static File mergeFiles(File[] files, String pathToMerge) throws FileNotFoundException, IOException{
        
        File merge = new File(pathToMerge);
        FileOutputStream out = new FileOutputStream(merge);
        
        for(File tmp: files){
            
            FileInputStream in = new FileInputStream(tmp);
            byte []bytes = new byte[in.available()];
            in.read(bytes);
            in.close();
            
            out.write(bytes);
            
        }
        out.close();
        
        return merge;
        
    }
    
    public static File mergeFilesByPaths(List<String> filePaths, String pathToMerge) throws FileNotFoundException, IOException{
        
        File merge = new File(pathToMerge);
        FileOutputStream out = new FileOutputStream(merge);
        
        for(String tmp: filePaths){
            
            FileInputStream in = new FileInputStream(tmp);
            byte []bytes = new byte[in.available()];
            in.read(bytes);
            in.close();
            
            out.write(bytes);
            
        }
        out.close();
        
        return merge;
        
    }
    
    public static File mergeFilesByPaths(String[] filePaths, String pathToMerge) throws FileNotFoundException, IOException{
        
        File merge = new File(pathToMerge);
        FileOutputStream out = new FileOutputStream(merge);
        
        for(String tmp: filePaths){
            
            FileInputStream in = new FileInputStream(tmp);
            byte []bytes = new byte[in.available()];
            in.read(bytes);
            in.close();
            
            out.write(bytes);
            
        }
        out.close();
        
        return merge;
        
    }
    
    public static String read(String filename) {
        String text = "";
        
        try {
            File file = new File(filename);
            
            if (!file.exists()) {
                System.err.println("FileManager.class: Error. file:\n" + filename + "\ndoes not exist");
                System.exit(1);
            } // if
            if (file.isDirectory()) {
                System.err.println("FileManager.class: Error. Path:\n" + filename + "\n" +
                        "refers to a directory. Must refer to a file");
               System.exit(1);
            } // if
            
            byte buff[] = new byte[(int) file.length()];
            FileInputStream fis = new FileInputStream(file);
            fis.read(buff);
            fis.close();
            text = new String(buff);
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
        
        return text;
    } // read
    public static String read(File file) {
        String text = "";
            
        try {
            
            if (!file.exists() || !file.isFile()) {
                System.err.println("Error reading the file " + file.getName());
                System.exit(1);
            } // if
            
            byte buff[] = new byte[(int) file.length()];
            FileInputStream fis = new FileInputStream(file);
            fis.read(buff);
            fis.close();
            text = new String(buff);
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
        
        return text;
    } // read
   
    public static void write(Tokenizer tokens, String filePath) {
       
        String text = Constants.ARTICLE_START_TAG + "\n";
        int size = tokens.size();
        for (int i = 0; i < size; i++) {
            String curr = tokens.get(i).getPrevChar() + tokens.getStartSubtitle(i);
            if (i == 0) {
                curr += Constants.SENTENCE_START_TAG;
            } // if
            else if (tokens.get(i-1).getEndOfPeriod()) {
                curr += Constants.SENTENCE_START_TAG;
            } // else if
            curr += tokens.getStartTag(i) + tokens.get(i).getToken() + tokens.getEndTag(i);
            if (tokens.get(i).getEndOfPeriod()) {
                curr += Constants.SENTENCE_END_TAG;
            } // if
            curr += tokens.getEndSubtitle(i);
            text += curr;
        } // for
        if (!text.endsWith(Constants.SENTENCE_END_TAG)) {
            text += Constants.SENTENCE_END_TAG;
        } // if
        text += "\n" + Constants.ARTICLE_END_TAG;
        
        try {
            
            new File(filePath).getParentFile().mkdirs();
            FileOutputStream fos = new FileOutputStream(filePath);
            fos.write(text.getBytes());
            fos.close();
        
        } // try
        catch (IOException e) {
            e.printStackTrace();
        } // catch
    } // write
    
} // end of class FileManager
